note
	description: "[
		Command used to add one node to another as a child.
		]"
	author: "Jimmy J. Johnson"
	date: "24 Feb 04"

class
	ADD_NODE_COMMAND

inherit

	JJ_COMMAND
		redefine
			text,
			affected_objects,
			execute,
			undo,
			is_executable,
			is_undoable
		end

feature -- Access

	text: STRING
			-- Name of this command
		do
			Result := "Add Node"
		end

	parent_node: NODE
			-- Node to which the new node will be added.

	child_node: NODE
			-- Node which will be added as child to `parent_node'.

	affected_objects: LINKED_SET [ANY]
			-- Set of objects changable by this command
		do
			create Result.make
			Result.extend (parent_node)
			Result.extend (child_node)
		end

feature -- Element change

	set_parent_node (a_node: like parent_node)
			-- Change `parent_node'
		require
			node_exists: a_node /= Void
		do
			parent_node := a_node
		ensure
			parent_node_was_set: parent_node = a_node
		end

	set_child_node (a_node: like child_node)
			-- Change `parent_node'
		require
			node_exists: a_node /= Void
		do
			child_node := a_node
		ensure
			child_node_was_set: child_node = a_node
		end

feature -- Basic operations

	execute
			-- Perform the command
		do
			parent_node.adopt (child_node)
			Precursor {JJ_COMMAND}
		end

	undo
			-- Undo the command
		do
			parent_node.disown (child_node)
			Precursor {JJ_COMMAND}
		end

feature -- Status report

	is_executable: BOOLEAN
			-- Can the command be executed?
		do
			Result := parent_node /= Void and child_node /= Void
		end

	is_undoable: BOOLEAN
			-- Can the command be undone?
		do
			Result := parent_node /= Void and child_node /= Void
		end

end
