note
	description: "[
			A COMMAND_MANAGER that adds the capability to optionally persist
			the objects changed by the command just executed or undone.
			]"
	author: "Jimmy J. Johnson	"
	date: "$20 Apr 06"

class
	COMMAND_MANAGER_WITH_PERSISTENCE

inherit

	COMMAND_MANAGER
		redefine
			execute,
			undo
		end

feature -- Status report

	is_persisting_changes: BOOLEAN
			-- Will changes made to PERSISTENTs in `changed_objects'
			-- be made persistent (e.g. written to disk)?
			-- Changed my `set_persist' and `set_no_persist'.
			-- Default is False.

feature -- Status setting

	set_persist
			-- Make `is_persisting_changes' True, so changed objects will
			-- be saved to perminent media.
		do
			is_persisting_changes := True
		end

	set_no_persist
			-- Make `is_persisting_changes' True, so changed objects will
			-- be saved to perminent media.
		do
			is_persisting_changes := False
		end

feature -- Basic operations

	execute
			-- Execute the command at `last_executed_index'.  This normally is the last undone command
		local
			c: JJ_COMMAND
		do
				-- Get the command at the current index.
			c := i_th (last_executed_index)
			Precursor {COMMAND_MANAGER}
			optional_persist (c.affected_objects)
		end

	undo
			-- Reverse execute the last command.
		local
			c: JJ_COMMAND
		do
				-- Get the command at the current index.
			c := i_th (last_executed_index)
			Precursor {COMMAND_MANAGER}
			optional_persist (c.affected_objects)
		end

feature {NONE} -- Implementation

	optional_persist (a_list: LINKED_SET [ANY])
			-- `Perist' any of the objects in `a_list' that happen
			-- to be PERSISTENT, if `is_persisting_changes'.
-- Fix me!!	Make this work for all types, not just PERSISTENTs.
-- 			Since DATASTORE has a once feature containing all the DATASTOREs
--			in a system, pick one and call `store' for all the `changed_objects';
--			if the changed object is a PERSISTENT then may be able to call
--			its `persist' feature.

		local
			p: PERSISTENT
		do
			if is_persisting_changes then
				from a_list.start
				until a_list.exhausted
				loop
					p ?= a_list.item
					if p /= Void then
						p.touch
					end
					a_list.forth
				end
				from a_list.start
				until a_list.exhausted
				loop
					p ?= a_list.item
					if p /= Void then
						p.persist
					end
					a_list.forth
				end
			end
		end

end
