note
	description: "[
			Command used to change a field of an {EDITABLE}.
			]"
	author: "Jimmy J. Johnson"
	date: "27 Jan 05"

class
	EDIT_COMMAND

inherit

	JJ_COMMAND
		redefine
			text,
			affected_objects,
			execute,
			undo,
			is_executable,
			is_undoable
		end

create
	default_create

feature -- Access

	text: STRING
			-- Name of this command
		do
			Result := "Edit record"
		end

	record: EDITABLE
			-- Object to be changed.

	field: FIELD
			-- FIELD which identifies the "item" to be changed

	value: ANY
			-- New value

	affected_objects: LINKED_SET [ANY]
			-- Set of objects changable by this command
		do
			create Result.make
			Result.extend (record)
		end

feature -- Element change

	set_record (a_record: like record)
			-- Change `record'
		require
			record_exists: a_record /= Void
		do
			record := a_record
		ensure
			record_assigned: record = a_record
		end

	set_field (a_field: FIELD)
			-- Change `field'
		require
			field_exists: a_field /= Void
		do
			field := a_field
		ensure
			field_assigned: field = a_field
		end

	set_value (a_value: ANY)
			-- Change `value'.
		require
			value_exists: a_value /= Void
		do
			value := a_value
		ensure
			value_assigned: value = a_value
		end

feature -- Basic operations

	execute
			-- Perform the command
		do
			if record.has_value (field.label) then
				old_field := field
				old_value := record.value (field.label)
			end
			record.extend_value (value, field.label)
			Precursor {JJ_COMMAND}
		end

	undo
			-- Undo the command
		do
			if old_field = Void then
					-- the `value' and `field' must be removed
				record.remove_value (field.label)
			else
					-- the `old_value' must be restored
				record.extend_value (old_value, field.label)
			end
			Precursor {JJ_COMMAND}
		end

feature -- Status report

	is_executable: BOOLEAN
			-- Can the command be executed?
		do
			Result := Precursor {JJ_COMMAND} and then
					record /= Void and field /= Void and value /= Void
		end

	is_undoable: BOOLEAN
			-- Can the command be undone?
		do
			Result := Precursor {JJ_COMMAND} and then
				(record /= Void and (old_field /= Void implies old_value /= Void)) and
				record.has_value (field.label)
		end

feature {NONE} -- Implementation

	old_value: ANY
			-- Hold on to the old value in case of `undo'

	old_field: FIELD
			-- Hold the old field.  If Void we know that the record previously
			-- did not have a field for the one just added by `execute', so
			-- `undo' will remove  `field'.

end
