note
	description: "[
		Commands for undo and redo of an action; for history functions.
		]"
	author: "Jimmy J. Johnson"

class
	JJ_COMMAND

create
	default_create

feature -- Access

	text: STRING
			-- Texual representation of what the command does.
		do
			Result := "do nothing"
		end

feature -- Basic operations

	execute
			-- Execute the action represented by the command.
		require
			executable: is_executable
		do
			was_executed := True
		end

	undo
			-- Reverse the action performed in `execute'.
		require
			undoable: is_undoable
		do
			was_executed := False
		end

	affected_objects: LINKED_SET [ANY]
			-- List of object whose views must be updated
		do
			create Result.make
		ensure
			result_exists: Result /= Void
		end

feature -- Status report

	is_executable: BOOLEAN
			-- Can the command be executed?
		do
			Result := not was_executed
		end

	is_undoable: BOOLEAN
			-- Can the command be undone?
		do
			Result := was_executed
		end

	was_executed: BOOLEAN
			-- Has the command been executed?

end

