note
	description	: "Root class for jj_vision."
	author		: "Jimmy J. Johnson"
	date		: "19 Jul 03"

class
	JJ_APPLICATION

inherit

	SHARED
		export
			{NONE}
				all
--			{ANY}
--				persistence_manager
		undefine
			default_create,
			copy
		end

	EV_APPLICATION
		redefine
			destroy
		end

create
	make_and_launch

feature {NONE} -- Initialization

	make_and_launch
			-- Initialize and launch application
		do
			default_create
			prepare
			launch
		end

	prepare
			-- Prepare the application by either loading it from a previous
			-- execution or create a new one if an old one doesn't exist.
		local
			w: like first_window
			t: like target
			ident: NATURAL_32
		do
--			initialize_directories
			rebuild_application
			if not is_application_loaded then
					-- For some reason, version 6.0 has a bug which requires I
					-- explicitly call `default_create' in the following line.
--				create target.default_create
				create t
				create w
--				set_target (t)
--				w.set_target (t)
				w.show
			end
		end

	destroy
			-- Save the state of all main_windows and end the application.
		do
--			save_system_state
			Precursor {EV_APPLICATION}
		end

	initialize_directories
			-- Set up the datastores for the system info.
		local
			d: DIRECTORY
		do
			create d.make (Default_data_directory_name)
			if not d.exists then
				d.create_dir
			end
			create d.make (Default_settings_directory_name)
			if not d.exists then
				d.create_dir
			end
		end

feature -- Access

	target: ANY
			-- The info this system will manipulate.

	make_new_target
			-- Create a new `target' for the application.
		local
			w: JJ_MAIN_WINDOW
			t: like target
		do
			create t
			set_target (t)
			from main_windows.start
			until main_windows.exhausted
			loop
				w := main_windows.item
				w.draw
				main_windows.forth
			end
		end

	first_window: JJ_MAIN_WINDOW
			-- Anchor to describe the type of the windows in the program.
		require
			has_at_least_one_window: has_windows
		do
			Result := main_windows.first
		ensure
			result_exists: Result /= Void
		end

--	command_manager: COMMAND_MANAGER is
--			-- Manages the COMMAND's called by the application to allow undo/redo capabilities.
--		once
--			create Result
--		end

	application_state: APPLICATION_STATE
			-- The state of the application.
		do
			create Result
		end

feature -- Element change

	set_target (a_target: like target)
			-- Change `target'
		require
			target_exists: a_target /= Void
		local
--			ident: OBJECT_ID
		do
				-- Force the creation (on the settings datastore) of an id for the new target
--			ident := persistence_manager.Settings_datastore.object_id (a_target)
			target := a_target
--			persistence_manager.extend_object (target)
		ensure
			target_assigned: target = a_target
		end

feature -- Status report

	has_windows: BOOLEAN
			-- Does at least one window exist in the application?
		do
			Result := not main_windows.is_empty
		end

feature -- Basic operations

	rebuild_application
			-- Rebuild the windows as they were on the last application exit.
		local
			f: RAW_FILE
			app_state: like application_state
			a: ANY
		do
				-- Attempt to read the `application_state' from the disk in order
				-- to begin this execution where the last left off.
			create f.make (merged_file_name (Default_settings_directory_name, "Application State"))
			if f.exists and then f.is_readable then
				f.open_read
				if f.is_readable then
					app_state ?= f.retrieved
				end
			end
			if app_state /= Void then
					-- Force a call to `widget' causing the creation of new windows.
				a := app_state.widget
				is_application_loaded := True
			end
		end

	save_system_state
			-- Save the state of the system for restoration on next execution.
		local
			fn: STRING
			f: RAW_FILE
		do
			fn := merged_file_name (Default_settings_directory_name, "Application State")
			create f.make (fn)
			if f.exists then
				f.wipe_out
--			else	
--				f.create_read_write
			end
			f.create_read_write
			f.general_store (application_state)
			f.close
		end

feature {NONE} -- Implementation

	is_application_loaded: BOOLEAN
			-- Was the last application state retrieved from a persistent file and
			-- now ready to be used to initialize the application?
			-- Set by `load_application_state'.

	Default_application_state_name: STRING = "Application State"
			-- To be used as file name of file containing the `state'.

end

