note
	description: "[
		This can be used as base class for the main window of a program, allowing
		the program to have multiple main windows, with standard buttons and actions
		such as undo/redo, open, save, etc."
		To enable undo/redo create a descendant of {JJ_COMMAND} and add it to the 
		`command_mangaer' with `add_command'.
		]"
	author: "Jimmy J. Johnson"
	date: "18 Jul 03"

class
	JJ_MAIN_WINDOW

inherit

	VIEW
		undefine
--			default_create,
			copy
		redefine
			initialize,
			initialize_interface,
			draw,
			destroy,
			is_destroyed,
			state,
			set_split_manager
		end

	EV_TITLED_WINDOW
		rename
			object_id as ise_object_id,
			item as ev_item,
			is_empty as ev_is_empty
		redefine
			initialize,
			is_in_default_state,
			destroy,
			is_destroyed
		end

create
	default_create

feature {NONE} -- Initialization

	initialize
			-- Build the interface for this window.
--		local
--			sys: like system
		do
			interface_table.block
			create split_manager
			Precursor {EV_TITLED_WINDOW}
			Precursor {VIEW}
			main_windows.extend (Current)
			interface_table.register_widget (Current, generating_type)
				-- Create and add the menu bar.
			build_standard_menu_bar
			show_menu
				-- Create and add the toolbar.
			build_main_window_tool_bar
			upper_bar.extend (create {EV_HORIZONTAL_SEPARATOR})
			upper_bar.extend (tool_bar_box)
			upper_bar.extend (create {EV_HORIZONTAL_SEPARATOR})
				-- Create and add the status bar.
			build_standard_status_bar
			lower_bar.extend (standard_status_bar)
				-- Set up window attributes
--			set_system (sys)		-- commented out for dynamic editor
--			set_target (system)
			set_size (Window_width, Window_height)
			set_actions
--			set_widget_states
			set_preferences_dialog (Default_preferences_dialog)
				-- Set up the "split-managed" cell, bar, and menu.
			set_split_manager (split_manager)
			interface_table.resume
			draw
		end

	is_in_default_state: BOOLEAN
			-- Is the window in its default state?
			-- Redefined to ignore this check.
			-- Left as routine so it can be redefined.
		do
			Result := True
		end

	initialize_interface
			-- Create INTERFACE_ITEMs which could be used by Current.
			-- Called by `initialize'.
		do
			interface_table.add_item_with_tuple (<<"System not saved text", "System not saved", "The system has not been saved since last change.", "Changes have been made, but the system has not been save.", "icon_red_cross.ico">>)
			interface_table.add_item_with_tuple (<<"not in system text", "Not in system", "The displayed interface names are not used by the program.", "The displayed interface names are not used by the program.", "icon_exec_quit_color.ico">>)
			interface_table.add_item_with_tuple (<<"Not allowed pixmap", "Not allowed", "Operation not allowed", "The operation you are attempting is not allowed.", "icon_delete_color.ico">>)
				-- Only add an interface item for Current if its `interface_name' has
				-- not already been added; the other items are specific for this tool
				-- and most likely will not be redefine, so no need to check inclusion
				-- of these in the `interface_table'.
			if not interface_table.has_key (generating_type) then
				interface_table.add_item_with_tuple (<< generating_type, "JJ_MAIN_WINDOW", "Main Window of the program", "At least one main window is always created.", "icon_new_development_tool_color.ico">>)
			end
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW}.file_menu", "File", "File actions", "Menu item containing 'open', 'save', 'save_as', etc.", "icon_open_file_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - new target", "New target", "Create a new target", "Makes a new target and put in current window", "icon_dotnet_import_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - save", "Save", "Save the current system", "This lets you select the file name.", "icon_save_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - open", "Open", "Open an existing file", "This opens a dialog box from which to select a file.", "icon_open_file_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - exit", "Exit", "Exit the program", "Promts to save and ends the program.", "icon_close_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW}.edit_menu", "Edit", "Edit actions", "Menu item containing 'undo', 'redo', 'cut', 'copy', etc.", "icon_format_clickable_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - undo", "Undo", "Undo last command", "Returns to previous state by undoing the last action.", "icon_undo_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - redo", "Redo", "Redo last undone command", "Executes the last undone command.", "icon_redo_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW}.tool_menu", "Tools", "Tools menu", "Contains tools menu items", "icon_tool_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - preferences", "Preferences", "Set preferences", "Allows setting of window parameters.", "icon_system_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW}.window_menu", "Window", "Window actions", "Menu item containing 'new_window', etc", "icon_dummy.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - new window", "New window", "Open an untargetted window", "Creates a new window.", "icon_new_development_tool_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - maximize all", "Maximize all", "Maximizes all windows", "This item allows you to maximize all system windows.", "icon_maximize_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - minimize all", "Minimize all", "Minimizes all windows", "This item allows you to minimize all system windows.", "icon_minimize_all_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - raise all", "Raise all", "Raises all windows", "This item alls you to restore all system windows.", "icon_restore_all_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW}.help_menu", "Help", "Help menu", "Contains help menu items", "icon_dummy.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - help", "Help", "Help", "Get help about an item", "icon_help_tool_color.ico">>)
			interface_table.add_item_with_tuple (<<"{JJ_MAIN_WINDOW} - help about", "About", "Display info about program", "Opens a dialog containing program information.", "icon_system_info_color.ico">>)
				-- Call Precursor after current is set up.  Specifically after the interface
				-- item for Current's `interface_name' is added.  (see above)
			Precursor {VIEW}
		end

	set_actions
			-- Add actions to the widgets
		do
				-- add actions to window
			close_request_actions.extend (agent on_close_window)
				-- add actions to File menu items
			file_new_target_item.select_actions.extend (agent on_make_new_target)
			file_open_item.select_actions.extend (agent on_file_open)
			file_save_item.select_actions.extend (agent on_file_save)
			file_exit_item.select_actions.extend (agent on_exit)
				-- add actions to the Tool menu items
			tool_preferences_item.select_actions.extend (agent on_show_preferences_dialog)
				-- add actions to Window menu items
			window_new_item.select_actions.extend (agent on_make_new_window)
			window_maximize_all_item.select_actions.extend (agent on_maximize_all)
			window_minimize_all_item.select_actions.extend (agent on_minimize_all)
			window_raise_all_item.select_actions.extend (agent on_raise_all)
--			window_preferences_item.select_actions.extend (agent on_show_preferences_window)
				-- add actions to Help menu items
			help_about_item.select_actions.extend (agent on_about)
				-- add actions to buttons
			new_target_button.set_pebble_function (agent on_get_target)
			new_target_button.select_actions.extend (agent on_make_new_target)
			new_target_button.drop_actions.extend (agent on_drop_target)
			open_button.select_actions.extend (agent on_file_open)
			save_button.select_actions.extend (agent on_file_save)
			undo_button.select_actions.extend (agent command_manager.undo)
			redo_button.select_actions.extend (agent command_manager.execute)
			new_window_button.drop_actions.extend (agent on_drop_in_new_window)
			new_window_button.select_actions.extend (agent on_make_new_window)
			minimize_all_button.select_actions.extend (agent on_minimize_all)
			raise_all_button.select_actions.extend (agent on_raise_all)
--			preferences_button.select_actions.extend (agent on_show_preferences_window)
		end

	build_standard_menu_bar
			-- Create and populate `standard_menu_bar'.
		require
			menu_bar_not_yet_created: standard_menu_bar = Void
		do
				-- Create, register, and fill the "File" menu
			create file_menu
			create file_new_target_item
			create file_save_item
			create file_open_item
			create file_exit_item
			interface_table.register_widget (file_menu, "{JJ_MAIN_WINDOW}.file_menu")
			interface_table.register_widget (file_new_target_item, "{JJ_MAIN_WINDOW} - new system")
			interface_table.register_widget (file_save_item, "{JJ_MAIN_WINDOW} - save")
			interface_table.register_widget (file_open_item, "{JJ_MAIN_WINDOW} - open")
			interface_table.register_widget (file_exit_item, "{JJ_MAIN_WINDOW} - exit")
			file_menu.extend (file_new_target_item)
			file_menu.extend (file_open_item)
			file_menu.extend (file_save_item)
			file_menu.extend (create {EV_MENU_SEPARATOR})
			file_menu.extend (file_exit_item)
				-- Create, register, and fill the "Edit" menu
			create edit_menu
			create edit_undo_item
			create edit_redo_item
			interface_table.register_widget (edit_menu, "{JJ_MAIN_WINDOW}.edit_menu")
			interface_table.register_widget (edit_undo_item, "{JJ_MAIN_WINDOW} - undo")
			interface_table.register_widget (edit_redo_item, "{JJ_MAIN_WINDOW} - redo")
			edit_menu.extend (edit_undo_item)
			edit_menu.extend (edit_redo_item)
				-- Create, register, and fill the "Tools" menu
			create tool_menu
			create tool_preferences_item
			interface_table.register_widget (tool_menu, "{JJ_MAIN_WINDOW}.tool_menu")
			interface_table.register_widget (tool_preferences_item, "{JJ_MAIN_WINDOW} - preferences")
			tool_menu.extend (tool_preferences_item)
				-- Create and fill the "Window" menu
			create window_menu
			create window_new_item
			create window_maximize_all_item
			create window_minimize_all_item
			create window_raise_all_item
			interface_table.register_widget (window_menu, "{JJ_MAIN_WINDOW}.window_menu")
			interface_table.register_widget (window_new_item, "{JJ_MAIN_WINDOW} - new window")
			interface_table.register_widget (window_maximize_all_item, "{JJ_MAIN_WINDOW} - maximize all")
			interface_table.register_widget (window_minimize_all_item, "{JJ_MAIN_WINDOW} - minimize all")
			interface_table.register_widget (window_raise_all_item, "{JJ_MAIN_WINDOW} - raise all")
			window_menu.extend (window_new_item)
			window_menu.extend (window_maximize_all_item)
			window_menu.extend (window_minimize_all_item)
			window_menu.extend (window_raise_all_item)
				-- Create, register, and fill the "Help" menu
			create help_menu
			create help_about_item
			interface_table.register_widget (help_menu, "{JJ_MAIN_WINDOW}.help_menu")
			interface_table.register_widget (help_about_item, "{JJ_MAIN_WINDOW} - help about")

			help_menu.extend (help_about_item)
				-- Create and fill the `standard_menu_bar'
			create standard_menu_bar
			standard_menu_bar.extend (file_menu)
			standard_menu_bar.extend (edit_menu)
			standard_menu_bar.extend (tool_menu)
			standard_menu_bar.extend (window_menu)
			standard_menu_bar.extend (help_menu)
		ensure
			menu_bar_created:
				standard_menu_bar /= Void and then
				not standard_menu_bar.is_empty
		end

	build_main_window_tool_bar
			-- Create and populate the standard toolbar.
		require
			toolbar_not_yet_created: main_window_tool_bar = Void
		do
				-- Create the box and tool bar
			create tool_bar_box
			create main_window_tool_bar
				-- Create the buttons.
			create new_target_button
			create open_button
			create save_button
			create undo_button
			create redo_button
			create new_window_button
			create minimize_all_button
			create raise_all_button
			create help_button
				-- Register buttons
			interface_table.register_widget (new_target_button, "{JJ_MAIN_WINDOW} - new target")
			interface_table.register_widget (open_button, "{JJ_MAIN_WINDOW} - open")
			interface_table.register_widget (save_button, "{JJ_MAIN_WINDOW} - save")
			interface_table.register_widget (undo_button, "{JJ_MAIN_WINDOW} - undo")
			interface_table.register_widget (redo_button, "{JJ_MAIN_WINDOW} - redo")
			interface_table.register_widget (new_window_button, "{JJ_MAIN_WINDOW} - new window")
			interface_table.register_widget (minimize_all_button, "{JJ_MAIN_WINDOW} - minimize all")
			interface_table.register_widget (raise_all_button, "{JJ_MAIN_WINDOW} - raise all")
			interface_table.register_widget (help_button, "{JJ_MAIN_WINDOW} - help")
				-- Add the buttons to the toolbar
			create main_window_tool_bar
			main_window_tool_bar.extend (new_target_button)
			main_window_tool_bar.extend (open_button)
			main_window_tool_bar.extend (save_button)
			main_window_tool_bar.extend (create {EV_TOOL_BAR_SEPARATOR})
			main_window_tool_bar.extend (undo_button)
			main_window_tool_bar.extend (redo_button)
			main_window_tool_bar.extend (create {EV_TOOL_BAR_SEPARATOR})
			main_window_tool_bar.extend (new_window_button)
			main_window_tool_bar.extend (minimize_all_button)
			main_window_tool_bar.extend (raise_all_button)
			main_window_tool_bar.extend (create {EV_TOOL_BAR_SEPARATOR})
			main_window_tool_bar.extend (help_button)
			main_window_tool_bar.extend (create {EV_TOOL_BAR_SEPARATOR})
				-- Add the tool bar to the box
			tool_bar_box.extend (main_window_tool_bar)
			tool_bar_box.disable_item_expand (main_window_tool_bar)
		ensure
			toolbar_created: main_window_tool_bar /= Void and then  not main_window_tool_bar.is_empty
		end

	build_standard_status_bar
			-- Create and populate the status bar at bottom of window.
		require
			status_bar_not_yet_created:
				standard_status_bar = Void and then
				standard_status_label = Void
		do
				-- Create the status bar.
			create standard_status_bar
			standard_status_bar.set_border_width (2)
					-- Populate the status bar.
			create standard_status_label.make_with_text ("Add your status text here...")
			standard_status_label.align_text_left
			standard_status_bar.extend (standard_status_label)
		ensure
			status_bar_created:
				standard_status_bar /= Void and then
				standard_status_label /= Void
		end

feature -- Access

	state: MAIN_WINDOW_STATE
			-- Used to `persist' the state of the widget.
			-- Redefined as attribute so it gets stored with the object.
			-- Call `make_state' to create a new state from Current.
		do
			create Result.make (Current)
		end

feature -- Element change

	set_split_manager (a_manager: SPLIT_MANAGER)
			-- Change the `split_manager'
		do
				-- Remove the old `split_manager' items from current.
			if has (split_manager.cell) then
				prune (split_manager.cell)
			end
			if upper_bar.has (split_manager.bar) then
				upper_bar.prune (split_manager.bar)
			end
			if window_menu.has (split_manager.menu) then
				window_menu.prune (split_manager.menu)
			end
				-- Change the `split_manager'.
			Precursor {VIEW} (a_manager)
				-- Add the new items to Current
			extend (split_manager.cell)
			upper_bar.extend (split_manager.bar)
			window_menu.extend (split_manager.menu)
		end


--	restore_with_state (a_state: MAIN_WINDOW_STATE) is
--			-- Set up window parameters as read from a file with name `a_file_name'.
--			-- `A_file_name' should include the full path and name of the file.
--		require
--			state_exists: a_state /= Void
--		local
--			t: like target
--		do
--			t ?= a_state.target
--			if t /= Void then
--				set_target (t)
--			end
--			set_position (a_state.x, a_state.y)
--			if a_state.is_maximized then
--				maximize
--			elseif a_state.is_minimized then
--				minimize
--			else
--				restore
--			end
--			set_size (a_state.width, a_state.height)
--			split_manager.restore_with_state (a_state.split_manager_state)
--		end

	show_menu
			-- Show the menu bar
		do
			set_menu_bar (standard_menu_bar)
		end

	hide_menu
			-- Hide the menu bar
		do
			remove_menu_bar
		end

	show_button_text
			-- Show text on all the buttons
		do
--			view_manager.show_button_text
			interface_table.show_button_text
		end

	hide_button_text
			-- Show text on all the buttons
		do
--			view_manager.hide_button_text
			interface_table.hide_button_text
		end

	set_preferences_dialog (a_window: EV_TITLED_WINDOW)
			-- Make `a_window' available to Current so `a_window' can be shown
			-- when the `preferences_button' or `preferences_item' is selected.
		require
			window_exists: a_window /= Void
		do
			preferences_dialog := a_window
		ensure
			window_registered: preferences_dialog = a_window
		end

feature -- Basic operations

	draw
			-- Update the whole JJ_MAIN_WINDOW
		local
			app: JJ_APPLICATION
			a: ANY
			e: EDITABLE
			s: STRING
		do
				-- This simply puts the `display_name' (from EDITABLE) or `generating_type' of
				-- the application's `target' (not the current window's target) into the title
				-- bar of the window.
			app ?= (create {EV_ENVIRONMENT}).application
			check
				app_exists: app /= Void
					-- Because this class is only used by JJJ_APPLICATIONs
			end
			s := app.generating_type + ":  "
			a := app.target
			if a /= Void then
				e ?= app.target
				if e /= Void then
					s := s + e.display_name
				else
					if target /= Void then
						s := s + target.generating_type
					else
						s := s + "Void target"
					end
				end
			else
				s := s + interface_table.short_name ("Empty")
			end
			set_title (s)
			set_widget_states
		end

	destroy
			-- Destroy underlying native toolkit object.
			-- Render `Current' unusable.
		do
			Precursor {VIEW}
			Precursor {EV_TITLED_WINDOW}
		end

feature -- Status report

	is_destroyed: BOOLEAN
			-- Is `Current' no longer usable?
		do
			Result := Precursor {VIEW} and Precursor {EV_TITLED_WINDOW}
		end

feature {NONE} -- Actions

	on_show_preferences_dialog
			-- React to a press of the preferences button or menu by
			-- showing the `preferences_dialog'.
		require
			preferences_dialog_exists: preferences_dialog /= Void
		do
			preferences_dialog.show
			if preferences_dialog.is_minimized then
				preferences_dialog.restore
			end
			preferences_dialog.raise
		end

	on_file_open
			-- React to a request to read a system from disk.
		do
--			system.validate_file_name_and_open
			set_widget_states
		end

	on_file_save
			-- React to a request to save the system to disk.
		local
			w: JJ_MAIN_WINDOW
--			ff: STANDARD_INTERFACE_FILE_FACILITIES
		do
--			create ff
--			ff.set_persistent (system)
--			ff.validate_file_name_and_save
			set_save_object (target)
			validate_file_name_and_save
--			if system.is_okay then
					-- Change the title of all JJ_MAIN_WINDOWs
				from main_windows.start
				until main_windows.exhausted
				loop
					w := main_windows.item
					if w.target = target then
						w.draw
					end
					main_windows.forth
				end
--			else
--				-- save operation failed.  Write failure?
--			end
--			set_widget_states
		end

	on_get_target: like target
			-- Needed to be able to `set_pebble_function' for buttons so the
			-- button can pick the current `target' as a pebble during a pick.
		do
			Result := target
		end

	on_make_new_target
			-- Create a new target after querrying user to save
			-- current target if necessary.
		local
			app: JJ_APPLICATION
		do
			app ?= (create {EV_ENVIRONMENT}).application
			check
				app_exists: app /= Void
					-- Because this class is only used by JJJ_APPLICATIONs
			end
			app.make_new_target
			set_target (app.target)
			draw
		end

	on_about
			-- Display the About dialog.
		do
			about_dialog.show
		end

	on_maximize_all
			-- Maximize all the main windows
		do
			main_windows.do_all (agent {JJ_MAIN_WINDOW}.maximize)
		end

	on_minimize_all
			-- Minimize all the main windows
		do
			main_windows.do_all (agent {JJ_MAIN_WINDOW}.minimize)
		end

	on_raise_all
			-- Raise all the main windows
		do
			main_windows.do_all (agent {JJ_MAIN_WINDOW}.restore)
		end

	on_drop_in_new_window (a_target: like target)
			-- React to `a_target' drop on the new_window button.
		require
			target_exists: a_target /= Void
		local
			w: like Current
		do
			create w
			w.set_target (a_target)
			w.show
		end

	on_drop_target (a_target: like target)
			-- React to drop of `a_target' (on a button?) by changing
			-- the `target' and setting up views.
		require
			target_exists: a_target /= Void
		do
--			system.uncancel
--			system.ask_to_save
--			if not system.is_cancelled.item then
--				set_system (a_system)
--			end
		end

	on_make_new_window
			-- Create a new JJ_MAIN_WINDOW (like Current) and
			-- target it to the same `system'.
		local
			w: like Current
		do
			create w
			if target /= Void then
				w.set_target (target)
			end
			w.show
		end

	on_exit
			-- End the execution, insuring to save unsaved targets.
		local
			w: JJ_MAIN_WINDOW
		do
			from main_windows.start
			until main_windows.exhausted or else is_cancelled.item
			loop
				w := main_windows.item
				if w.target /= Void then	-- and then not w.target.is_persistent then
					set_save_object (w.target)
					ask_to_save
				end
				main_windows.forth
			end
			if not is_cancelled.item then
				end_application
			end
		end

feature {JJ_MAIN_WINDOW, APPLICATION_STATE} -- Implementation (support routines)

	on_close_window
				-- A request to close the window has occured.
				-- If this is the only window displaying `system' then give user
				-- chance to save the `system' before closing window.
				-- Remove the `a_window' from `main_windows' list and destroy it.
				-- If it is the last window then end the application.
		local
			jj_app: JJ_APPLICATION
		do
			uncancel
			jj_app ?= (create {EV_ENVIRONMENT}).application
			check
				jj_app_exists: jj_app /= Void
					-- Because JJ_MAIN_WINDOW (this class) is for use in a JJ_APPLICATION.
			end
--			if jj_app.persistence_manager.is_pending then
				ask_to_save
--			end
			if not is_cancelled.item then
				if main_windows.count = 1 then
						-- About to close the last window and therefore end the application,
						-- so save it's state before destroying this last window.
					end_application
				else
					main_windows.start
					main_windows.prune (Current)
					destroy
				end
			end
		end

feature {COMMAND_MANAGER} -- Implementation

	set_widget_states
			-- Update the state of the buttons and menu items
		do
				-- Item always enabled
--			file_menu.enable_sensitive
--			file_new_system_item.enable_sensitive
--			file_open_item.enable_sensitive
--			file_exit_item.enable_sensitive
--			help_menu.enable_sensitive
--			help_about_item.enable_sensitive
--			new_window_button.enable_sensitive
--			new_system_button.enable_sensitive
--			open_button.enable_sensitive
--			help_button.enable_sensitive
				-- Items conditionally enabled
			if target = Void then
				file_save_item.disable_sensitive
				save_button.disable_sensitive
--			elseif target.is_touched then
			elseif command_manager.is_at_marked_state then
				file_save_item.disable_sensitive
				save_button.disable_sensitive
			else
				file_save_item.enable_sensitive
				save_button.enable_sensitive
			end
			if command_manager.is_undoable then
				edit_undo_item.enable_sensitive
				undo_button.enable_sensitive
			else
				edit_undo_item.disable_sensitive
				undo_button.disable_sensitive
			end
			if command_manager.is_executable then
				edit_redo_item.enable_sensitive
				redo_button.enable_sensitive
			else
				edit_redo_item.disable_sensitive
				redo_button.disable_sensitive
			end
		end

feature {NONE} -- Implementation (support routines)

	is_target_in_other_window: BOOLEAN
			-- Is `target' also contained in some other JJ_MAIN_WINDOW?
			-- Used to determine if it must be saved when Current is closed.
		do
			if target /= Void then
				from main_windows.start
				until main_windows.exhausted or else Result
				loop
					Result := main_windows.item /= Current and then main_windows.item.target = target
					main_windows.forth
				end
			end
		end

	end_application
			-- Close down the application
		do
			(create {EV_ENVIRONMENT}).application.destroy
		end

	preferences_dialog: EV_TITLED_WINDOW
			-- The window, if any, displayed when the `preferences_button' or
			-- `preferences_item' is selected.
			-- Can be used as a dialog for setting up the views.

	Default_preferences_dialog: PREFERENCES_WINDOW
			-- For setting window parameters
		once
			create Result
			Result.close_request_actions.extend (agent Result.hide)
		end

	about_dialog: ABOUT_DIALOG
			-- Show minimal information about the program.
		once
			create Result
				-- Hide when something else is clicked so it can be non-modal
				-- and so the user does not have to hit the "okay" button or
				-- explicitely close the dialog.
			Result.close_request_actions.extend (agent Result.hide)
			Result.focus_out_actions.extend (agent Result.hide)
		end

feature {NONE} -- Menus

	standard_menu_bar: EV_MENU_BAR
			-- Standard menu bar for this window.

	file_menu: EV_MENU
			-- "File" menu for this window (contains New, Open, Close, Exit...)

	file_new_target_item: EV_MENU_ITEM
			-- "File/New system"

	file_open_item: EV_MENU_ITEM
			-- "File/Open"

	file_save_item: EV_MENU_ITEM
			-- "File/Save"

	file_exit_item: EV_MENU_ITEM
			-- "File/Exit"

	edit_menu: EV_MENU
			-- "Edit" menu (contains Undo, Redo, Cut, Copy, etc...)

	edit_undo_item: EV_MENU_ITEM
			-- "Edit/Undo"

	edit_redo_item: EV_MENU_ITEM
			-- "Edit/Redo"

	tool_menu: EV_MENU
			-- "Tools" menu (contains Preferences, etc...)

	tool_preferences_item: EV_MENU_ITEM
			-- "Tools/Preferences"

	window_menu: EV_MENU
			-- "Window" menu for this window (contains Minimize all, Raise all, etc...)

	window_new_item: EV_MENU_ITEM
			-- "Window/New"

	window_maximize_all_item: EV_MENU_ITEM
			-- "Window/Maximize all"

	window_minimize_all_item: EV_MENU_ITEM
			-- "Window/Minimize all"

	window_raise_all_item: EV_MENU_ITEM
			-- "Window/Raise all"

	help_menu: EV_MENU
			-- "Help" menu for this window (contains About...)

	help_about_item: EV_MENU_ITEM
			-- "Help/About"

feature {NONE} -- Toolbars and buttons

	tool_bar_box: EV_VERTICAL_BOX
			-- Holder for `main_window_tool_bar'.  Other items can
			-- be added here to make a custom bar or to `upper_bar'
			-- to place another tool_bar below this one.

	main_window_tool_bar: EV_TOOL_BAR
			-- Standard toolbar for this window

	new_window_button: EV_TOOL_BAR_BUTTON
			-- Create a new system

	new_target_button: EV_TOOL_BAR_BUTTON
			-- Create a new system for this window

	open_button: EV_TOOL_BAR_BUTTON
			-- Opens an existing system

	save_button: EV_TOOL_BAR_BUTTON
			-- To save the system

	minimize_all_button: EV_TOOL_BAR_BUTTON
			-- To minimize all the JJ_MAIN_WINDOWs

	raise_all_button: EV_TOOL_BAR_BUTTON
			-- To restore all the JJ_MAIN_WINDOWs

	help_button: EV_TOOL_BAR_BUTTON
			-- Opens help engine

	undo_button: EV_TOOL_BAR_BUTTON
			-- To undo the last command

	redo_button: EV_TOOL_BAR_BUTTON
			-- To redo the last undone command

feature {NONE} -- Status bar

	standard_status_bar: EV_STATUS_BAR
			-- Standard status bar for this window

	standard_status_label: EV_LABEL
			-- Label situated in the standard status bar.
			--
			-- Note: Call `standard_status_label.set_text (...)' to change the text
			--       displayed in the status bar.

feature {NONE} -- Implementation / Constants

	Window_title: STRING = "Standard Interface JJ_MAIN_WINDOW"
			-- Title of the window.

	Window_width: INTEGER = 800
			-- Initial width for this window.

	Window_height: INTEGER = 600
			-- Initial height for this window.

--feature {NONE} -- Inapplicable

--	save (a_file: RAW_FILE)
--			-- Write the data to `a_file'.
--			-- Redefined to do nothing; this is taken care of in `on_file_save'.
--		do
--		end

--	read (a_file: RAW_FILE)
--			-- Read the data from `a_file' and assign it to Current.
--			-- Redefined to do nothing; this is taken care of in `on_file_open'.
--		do
--		end

invariant

--	system_exists: system /= Void

end

