note
	description: "[
			Extends {JJ_FILE_FACILITIES} with features for creating file-naming, file-
			saving dialogs and such.
		]"
	author: "Jimmy J. Johnson"
	date: "24 Sep 04"

class
	JJ_VISION_FILE_FACILITIES

inherit

	JJ_FILE_FACILITIES

feature -- Access

	image_directory: FILE_NAME
			-- Directory where the system will look for image files (icons,
			-- pixmaps, cursors, etc.)
			-- The default is the directory named "images" under the `program_directory'.
		once
			create Result.make_from_string (Default_image_directory_name)
		end

feature -- Element change

	set_image_directory (a_name: PATH_NAME)
			-- Change the `name' of the `image_directory.
		require
			name_exists: a_name /= Void
			name_is_valid: is_valid_file_name (a_name)
		do
			image_directory.make_from_string (a_name.string)
		ensure
			image_directory_assigned: equal (image_directory.string, a_name.string)
		end

feature {NONE} -- Constants

	Unix_image_extension: STRING = "PNG"
			-- File extension for cursor shape images on Unix

	Windows_image_extension: STRING = "ICO"
			-- File extension for cursor shape images on Windows

feature {NONE} -- Implementation

	image_file_name (a_name: STRING): STRING
			-- File name for image named `a_name'
			--| Input is file name without extension.
			--| Depending on OS file extension is added.
		require
			name_exists: a_name /= Void
			valid_name: is_valid_file_name (a_name)
			physical_image_directory_exists: file_exists (image_directory)
		local
			ext: STRING
			frm: LINEAR [STRING_32]
--			frm: LINEAR [STRING]
			f: FILE_NAME
		do
			Result := merged_file_name (image_directory, a_name)
			if not a_name.string.has ('.') then
				frm := (create {EV_ENVIRONMENT}).supported_image_formats
				from
					frm.start
				until
					ext /= Void or frm.after
				loop
					if equal (frm.item.as_string_8, Windows_image_extension) then
						ext := Windows_image_extension.twin
						ext.to_lower
					end
					frm.forth
				end
				create f.make_from_string (Result.string + "." + ext)
				if ext = Void or else not file_exists (f) then
					ext := Unix_image_extension.twin
					ext.to_lower
				end
				create f.make_from_string ("." + ext)
				Result := merged_file_name (Result, f)
			end
		ensure
			Result_exists: Result /= Void
			Result_is_valid: is_valid_file_name (Result)
		end

feature {NONE} -- Implementation (constants)

	Default_image_directory_name: STRING = "..\..\bitmaps\ico"
			-- Default name of the image directory to be concatinated

end

