note
	description: "Global objects for Standard Interface cluster"
	author: "Jimmy J. Johnson"
	date: "10 May 03"

class
	SHARED

inherit

	ANY

	JJ_VISION_FILE_FACILITIES
		undefine
			default_create,
			is_equal
		end

feature --{NONE} -- Access, restricted to descendants

	main_windows: LINKED_LIST [JJ_MAIN_WINDOW]
			-- List of all open JJ_MAIN_WINDOWs
		once
			create Result.make
		end

	interface_table: INTERFACE_TABLE
			-- Holds the strings for interface items.
		once
			create Result
		end

	command_manager: COMMAND_MANAGER
			-- Used to undo/redo commands for this system
		once
			create Result
		end

--	persistence_manager: PERSISTENCE_MANAGER
--			-- Manages the timing and options of persisting objects for the system.
--		once
--			create Result
--			Result.set_commit_immediate
--		end

feature -- Access

	save_object: CELL [ANY]
			-- Holds the object for passing to the various dialogs for
			-- storing and loading.
			-- Declared as "once" to prevent adding an attribute to every
			-- descendent of SHARED.
		once
			create Result.put (Void)
		end

feature -- Element change

	set_save_object (a_value: ANY)
			-- Change the value of `save_object'.
		require
			save_object_exists: a_value /= Void
		do
			save_object.put (a_value)
		ensure
			value_assigned: save_object.item = a_value
		end

feature -- Basic operations

	frozen ask_to_save
			-- Show the `ask_to_save_dialog'
		local
			a: JJ_APPLICATION
			w: EV_WINDOW
		do
			a ?= (create {EV_ENVIRONMENT}).application
			check
				should_not_happen: a /= Void
					-- because only JJJ_APPLICATIONS should use this.  really ???
			end
			w := a.first_window
			ask_to_save_dialog.show_modal_to_window (a.first_window)
		end

	show_save_dialog
			-- Show the `file_save_dialog'.
			-- Also, passed as an agent in `ask_to_save_dialog'.
		do
			file_save_dialog.set_title (interface_table.short_name ("{VIEWABLE}.file_save_dialog"))
			file_save_dialog.show_modal_to_window (main_windows.first)
		end

	validate_file_name_and_save
			-- Get a file name and write `save_object' to disk.
		require
--			save_object_exists: save_object.item /= Void
		local
			s: STRING
			f: RAW_FILE
		do
			s := file_save_dialog.file_title
			if is_valid_file_name (s) then
--				create f.make_open_write (s)
--				f.general_store (save_object)
			else
				file_save_dialog.set_title (interface_table.short_name ("{VIEWABLE}.file_save_dialog"))
				file_save_dialog.show_modal_to_window (main_windows.first)
			end
		end

	validate_file_name_and_open
			-- Get a file name and read a object into `save_object'
		require
			save_object_exists: save_object.item /= Void
		local
			s: STRING
			f: RAW_FILE
			a: ANY
		do
			s := file_open_dialog.file_name
			if is_valid_file_name (s) then
				create f.make_open_read (s)
				a := f.retrieved
				check
					object_read_in: a /= Void
						-- Because the filename was valid?
				end
				set_save_object (a)
			else
				file_open_dialog.set_title (interface_table.short_name ("{VIEWABLE}.file_open_dialog"))
				file_open_dialog.show_modal_to_window (main_windows.first)
			end
		end

feature -- Status report

	is_cancelled: BOOLEAN_REF
			-- Was an action (in a dialog) cancelled?
			-- A once function so any JJ_MAIN_WINDOW can recognize it.
		once
			create Result
		end

feature -- Status setting

	cancel
			-- A cancel button in one of the dialogs was pressed.
			-- Sets `is_cancelled' to True.
			-- Should be reset to False after it is checked.
		do
			is_cancelled.set_item (True)
		end

	uncancel
			-- Resets is_cancelled to False
		do
			is_cancelled.set_item (False)
		end

feature {NONE} -- Implementation

	File_open_dialog: EV_FILE_OPEN_DIALOG
			-- Standard
		once
			create Result
--			Result.filters.extend (["*.*", "All files"])
			Result.open_actions.extend (agent validate_file_name_and_open)
			Result.cancel_actions.extend (agent cancel)
			interface_table.register_widget (Result, "{VIEWABLE}.file_open_dialog")
		end

	Ask_to_save_dialog: EV_QUESTION_DIALOG
			-- Asks the user if he wants to save the system.
		local
			dc: EV_DIALOG_CONSTANTS
		do
			create dc
			create Result
--			Result.set_text ("System is not saved!%N%N  Save?")
--			Result.button (dc.Ev_yes).select_actions.extend (agent show_save_dialog)
			Result.set_text ("Some objects have changed!  Commit changes?")
			Result.button (dc.ev_yes).select_actions.extend (agent commit_changes)
			Result.button (dc.Ev_cancel).select_actions.extend (agent cancel)
		end

	commit_changes
			-- Save any changed objects.
		local
			jj_app: JJ_APPLICATION
		do
			jj_app ?= (create {EV_ENVIRONMENT}).application
			check
				jj_app_exists: jj_app /= Void
					-- Because this is used by JJ_APPLICATIONs
			end
--			jj_app.persistence_manager.commit
		end

	File_save_dialog: EV_FILE_SAVE_DIALOG
			-- Standard
		once
			create Result
--			Result.filters.extend (["*.*", "All files"])
			Result.save_actions.extend (agent validate_file_name_and_save)
			Result.cancel_actions.extend (agent cancel)
			interface_table.register_widget (Result, "{VIEWABLE}.file_save_dialog")
		end

end

