note
	description	: "[
				This sets up buttons and menus for "selecting" multiple views and different
				formats for those views within a `cell'.  The `cell', `menu', and `bar' can
				then be added to your container as appropriate.
				
				It manages the placement of VIEWs in a `cell', allowing nested
				split areas to be hidden/shown using buttons and/or menus.  The views are placed
				in `cell' which can be placed in an EV_CONTAINER.
				
				Views are added to Current by first telling Current how to create/obtain a new view
				using `register_view_function'.  (`Register_view_function' takes as argument a function
				returning a VIEW.)  The next step is to call `create_view' passing the index
				to the "registered function" to use to create the view.  Here is the pattern as 
				it is used in feature `initialize' as it is redefined in a descendent of MAIN_WINDOW:
				
				feature initialize is
						-- Set up the window.
						-- Redefine to add the views to the window.
					do
						Precursor {MAIN_WINDOW}
						view_manager.enable_mode_changes
						view_manager.register_view_function (agent new_test_tool)
						view_manager.register_view_function (agent new_edit_view)
						view_manager.create_view (1)
						view_manager.create_view (1)
						view_manager.set_split_vertical
						view_manager.create_view (2)
						view_manager.create_view (2)
						set_name ("VIEW_MANAGER Tester program")
					end
					
				This will create 3 views and place the first two in a horizontal split area and then
				add a third window beneith those two in the bottom half of a vertical split area.
				The first two views are of the type returned by "function_to_create_a_view" and the
				third window is of the type returned by "another_function".  In this way any number
				of views can be added to current, but they can only be of the two type that are
				"registered".
				
				If the number of views added to Current is at least two then the `bar' is filled
				with a set of buttons and 'menu' is built with menu items for selecting the
				"mode" of the `cell'.  When `is_in_split_mode' (the default mode) a number of
				views will be displayed in EV_SPLIT_AREAs, recursively, within `cell'.  When
				`is_maximized_mode' only the `selected_view' will be placed in `cell'.  Finally,
				when `is_notebook_mode' the views will be displayed as tabs in an EV_NOTEBOOK 
				within `cell'.
				
				Here is how the `bar' looks when `is_maximized_mode' or `is_notebook_mode'.
				
					PB = `preferences_button'
					SB = `split_button'
					MB = `maximize_button'
					NB = `notebook_button'
					RB = an EV_TOOL_BAR_RADIO_BUTTON
					
				  |---------------------- bar --------------------------------------|
				  | |--- mode_bar ---|  |--------------- radio bar -------------|   |
				  | |  |SB| |MB| |NB|   |  |  |RB| |RB| ... to number of views  |   |
				
				When one of the radio buttons is pressed the corresponding view is brought to
				the top.

				When `is_split_mode' the radio buttons are replaced with EV_TOOL_BAR_TOGGLE_BUTTONs.
				
					TB = an EV_TOOL_BAR_TOGGLE_BUTTON

				  |------------------------------- bar ---------------------------|
				  | |--- mode_bar ---|  |-------------- toggle bar -----------|   |
				  | | |SB| |MB| |NB| |  |   |TB| |TB| ... to number of views  |   |

				When a toggle button is selected the corresponding view becomes visible in a split
				area calculated on the fly.

				The items in `menu' are kept in parallel with the buttons allowing menu selection
				to accomplish the same things as the buttons.
				]"
	author		: "Jimmy J. Johnson"
	date		: "31 Aug 03"

class
	VIEW_MANAGER

inherit

	SHARED
		export
			{NONE} all
		redefine
			default_create
		end

	ANY
		redefine
			default_create
		end

create
	default_create

feature {NONE} -- Initialization

	default_create
			-- Set up the window.
		do
			create views.make
			create visible_views.make
			create disabled_views.make
			create splits.make
			create splitter_positions.make
			create notebook
			create radio_buttons.make
			create toggle_buttons.make
			create radio_items.make
			create toggle_items.make
				-- create exported widgets
			create bar
			create menu
			create cell
				-- create tool bars
			create toggle_bar
			create radio_bar
			create mode_bar
				-- create buttons and menus
			create single_mode_button
			create split_mode_button
			create notebook_mode_button
			create mode_menu
			create toggle_menu
			create radio_menu
			create single_mode_item
			create split_mode_item
			create notebook_mode_item
			initialize_interface_items
				-- "Register" the widgets (see `interface_table')
			interface_table.register_widget (single_mode_button, "{VIEW_MANAGER} - single_mode")
			interface_table.register_widget (split_mode_button, "{VIEW_MANAGER} - split_mode")
			interface_table.register_widget (notebook_mode_button, "{VIEW_MANAGER} - notebook_mode")
				-- Set up the `mode_bar' and its buttons.
--			mode_bar.extend (create {EV_TOOL_BAR_SEPARATOR})
			mode_bar.extend (single_mode_button)
			mode_bar.extend (split_mode_button)
			mode_bar.extend (notebook_mode_button)
--			mode_bar.extend (create {EV_TOOL_BAR_SEPARATOR})
			split_mode_button.enable_select
				-- Set up menus which do not change
			menu.set_text ("Views")
			mode_menu.set_text ("Mode")
			radio_menu.set_text ("Select view")
			toggle_menu.set_text ("Select view")
			single_mode_item.set_text ("Maximize Mode")
			split_mode_item.set_text ("Split Mode")
			notebook_mode_item.set_text ("Notebook Mode")
			mode_menu.extend (single_mode_item)
			mode_menu.extend (split_mode_item)
			mode_menu.extend (notebook_mode_item)
			split_mode_item.enable_select
				-- Build the bar and set actions
			selected_view := default_view
--			hide_button_text
			build_bars
--			interface_table.update_interface
			set_actions
				-- Assume split mode
			is_split_mode := True
		end

	set_actions
			-- Set up actions widgets and containers
		do
				-- Actions for `notebook'
			notebook.selection_actions.extend (agent on_tab_selected)
				-- Actions for buttons
--			preferences_button.select_actions.extend (agent on_show_preferences_dialog)
			split_mode_button.select_actions.extend (agent set_split_mode)
			single_mode_button.select_actions.extend (agent set_single_mode)
			notebook_mode_button.select_actions.extend (agent set_notebook_mode)
				-- Actions for menu items
--			preferences_item.select_actions.extend (agent on_show_preferences_dialog)
			single_mode_item.select_actions.extend (agent set_single_mode)
			split_mode_item.select_actions.extend (agent set_split_mode)
			notebook_mode_item.select_actions.extend (agent set_notebook_mode)
		end

	initialize_interface_items
			-- Set up the `interface_table' to include items needed by Current.
		do
			interface_table.add_item_with_tuple (<<"{VIEW_MANAGER} - single_mode", "Single mode", "Switch view to single mode", "Makes only one view visible in the window.", "icon_format_text_color.ico">>)
			interface_table.add_item_with_tuple (<<"{VIEW_MANAGER} - split_mode", "Split mode", "Switch view to split mode", "Makes multiple views visible in the window.", "icon_new_development_tool_color.ico">>)
			interface_table.add_item_with_tuple (<<"{VIEW_MANAGER} - notebook_mode", "Tab mode", "Switch view to tab mode", "Makes views visible in the notebook tabs.", "icon_folders_color.ico">>)
			interface_table.add_item_with_tuple (<<"{VIEW_MANAGER} - create_view_string", "Toggle/Raise ", "Toggles or Raises the associated view", "Select this item to Toggle or raise the associated view.", "icon_dummy.ico">>)
		end

feature -- Access

	menu: EV_MENU
			-- Menu of items to toggle views on and off.
			-- Can be inserted into window menu.

	bar: EV_HORIZONTAL_BOX
			-- Bar of buttons used to manage views.
			-- Can be inserted into a toolbar.

	cell: EV_CELL
			-- The container which will be managed.  The views will be placed
			-- into `cell' based on the user's button, or menu selections.
			-- This should be extended into a widget.

	selected_view: VIEW
			-- The currently selected view.

	i_th_view (a_index: INTEGER): VIEW
			-- Get the i-th view
		require
			valid_index: is_valid_view_index (a_index)
		do
			Result := views.i_th (a_index)
		end

	last_view: VIEW
			-- The last view in the list.
		require
			not_empty: not view_list.is_empty
		do
			Result := views.last
		ensure
			result_exists: result /= Void
		end

	view_list: LINEAR [VIEW]
			-- Linear representation of the views managed by Current
		do
			Result := views.linear_representation
		end

	default_view: DEFAULT_VIEW
			-- Create an EV_LABEL as a place holder
		do
			create Result
			Result.set_pebble (Result)
			Result.set_text ("Empty:  no VIEW")
		end

feature -- Element change

--	set_preferences_dialog (a_window: EV_TITLED_WINDOW) is
--			-- Make `a_window' available to Current so `a_window' can be shown
--			-- when the `preferences_button' or `preferences_item' is selected.
--		require
--			window_exists: a_window /= Void
--		do
--			preferences_dialog := a_window
--		ensure
--			window_registered: preferences_dialog = a_window
--		end

	restore_state (a_state: VIEW_MANAGER_STATE)
			-- Restore Current to state as represented in `a_state'
-- FIX ME!!  work on `disabled_view' restoration.
		require
			state_exists: a_state /= Void
		local
			i: INTEGER
			vs: VIEW_STATE
		do
				-- Show all the views so they will have a parent, because the `draw'
				-- feature of some views may depend on the parent.  `Draw' may be
				-- called when an object is placed into the view during the view's
				-- `restore_state' feature.
			show_all_views
			from a_state.view_states.start
			until a_state.view_states.exhausted
			loop
				i := a_state.view_states.index
				vs := a_state.view_states.item
--				views.i_th (i).restore_state (vs)
				a_state.view_states.forth
			end
				-- Before losing references to all the old splits make sure that
				-- they are empty so any widgets which they contain can later be
				-- placed into another split area.
			splits.do_all (agent {EV_SPLIT_AREA}.wipe_out)
				-- Clean out the containers; but not the views.
			splits.wipe_out
			splitter_positions.wipe_out
			notebook.wipe_out
			visible_views.wipe_out

			is_mode_frozen := a_state.is_mode_frozen
			is_view_frozen := a_state.is_view_frozen
			is_single_mode := a_state.is_single_mode
			is_split_mode := a_state.is_split_mode
			is_notebook_mode := a_state.is_notebook_mode
			is_creating_vertical_splits := a_state.is_creating_vertical_splits
					-- This loop puts the views (using the indexes from `visible_views'
					-- in `a_state') to rebuild `visible_views'.
			from a_state.visible_views.start
			until a_state.visible_views.exhausted
			loop
				i := a_state.visible_views.item
				visible_views.extend (views.i_th (i))
				a_state.visible_views.forth
			end
					-- Create correct split areas from `a_state'
			from a_state.splits.start
			until a_state.splits.exhausted
			loop
				if a_state.splits.item then
					splits.extend (create {EV_VERTICAL_SPLIT_AREA})
				else
					splits.extend (create {EV_HORIZONTAL_SPLIT_AREA})
				end
				a_state.splits.forth
			end
					-- Reset the splitter positions to what was stored in `a_state'.
			from a_state.splitter_positions.start
			until a_state.splitter_positions.exhausted
			loop
				i := a_state.splitter_positions.item
				splitter_positions.extend (i)
				a_state.splitter_positions.forth
			end
			if a_state.selected_view /= 0 then
				selected_view := views.i_th (a_state.selected_view)
			else
				selected_view := Default_view
			end
--			build_buttons
			syncronize
		end

feature -- Status report

	is_view_frozen: BOOLEAN
			-- Is the set of visible views frozen (i.e. can the user toggle a view
			-- on or off)?  (Assuming there is more than one view.)
			-- This affects the interface, not the programmers ability to change.

	is_mode_frozen: BOOLEAN
			-- Is the "mode" frozen (i.e. if `is_single_mode' always single mode)?
			-- This affects the interface, not the programmers ability to change.
			-- See `enable_mode_changes' and `disable_mode_changes'.

	is_single_mode: BOOLEAN
			-- Is only one view in `cell'?

	is_split_mode: BOOLEAN
			-- Is the view in split mode (multiple views showing)?

	is_notebook_mode: BOOLEAN
			-- Is the view in notebook mode (multiple views in an EV_NOTEBOOK)?

	is_creating_vertical_splits: BOOLEAN
			-- Used by `new_split_area' to determine the type of EV_SPLIT_AREA to create.
			-- When true, new views are put into EV_VERTICAL_SPLIT_AREAs;
			-- When false, new views are put into EV_HORIZONTAL_SPLIT_AREAs.

feature -- Status setting

	enable_mode_changes
			-- Allow the *user* (not to be confused with *programer*) to change
			-- the way the contained views (if there is more than one view) are
			-- presented in `cell' by setting `is_mode_frozen' to True.
			-- If more than one view is present and `is_view_frozen' then
			-- buttons and menus for changing the mode will be made available to
			-- the programmer in `bar' and `menu' which can then be presented to
			-- the user.
		do
			is_mode_frozen := False
			build_bars
			build_cell
		ensure
			mode_is_changable: not is_mode_frozen
		end

	disable_mode_changes
			-- Prevent *user* changes to the "mode" by setting `is_mode_changable'
			-- to False.  The user will see what ever appearance has been set
			-- by the programmer using the `set_..._mode' features.
			-- See `set_split_mode', `set_single_mode',and `set_notebook_mode'.
		do
			is_mode_frozen := True
		ensure
			mode_is_not_changable: is_mode_frozen
		end

	enable_view_changes
			-- Allow the *user* to toggle the visible views.
			-- Used to add the buttons and menues from `bar'.
		do
			is_view_frozen := False
		ensure
			views_are_changable: not is_view_frozen
		end

	disable_view_changes
			-- Prevent the *user* from toggling the visible views.
			-- Used to hide or remove the buttons and menues from `bar'.
		do
			is_view_frozen := True
		ensure
			views_not_changable: is_view_frozen
		end

	set_single_mode
			-- Make the `cell' hold only one widget, either the `selected_view' or,
			-- if there are no views, a `default_view'.
--		require
--			mode_is_changable: is_mode_changable
		do
			if not is_single_mode then
				save_split_positions
				is_single_mode := True
				is_split_mode := False
				is_notebook_mode := False
--				on_mode_change
				save_split_positions
				syncronize
			end
			select_view (selected_view)
		ensure
			is_single_mode: is_single_mode
		end

	set_split_mode
			-- Make the `cell' hold the views recursively in EV_SPLIT_AREAs.
			-- If there are no views then `cell' contains a `default_view'.
--		require
--			is_mode_changable: is_mode_changable
		do
			if not is_split_mode then
				is_split_mode := True
				is_single_mode := False
				is_notebook_mode := False
--				on_mode_change
				save_split_positions
				syncronize
			end
			select_view (selected_view)
		ensure
			is_split_mode: is_split_mode
		end

	set_notebook_mode
			-- Make `cell' hold the views in an EV_NOTEBOOK.
			-- If there are no views then `cell' contains a `default_view'.
--		require
--			is_mode_changable: is_mode_changable
		do
			if not is_notebook_mode then
				is_notebook_mode := True
				is_single_mode := False
				is_split_mode := False
--				on_mode_change
				save_split_positions
				syncronize
			end
			select_view (selected_view)
		ensure
			is_notebook_mode: is_notebook_mode
		end

	set_split_vertical
			-- Make new views be put into EV_VERTICAL_SPLIT_AREAs.
		do
			is_creating_vertical_splits := True
		ensure
			is_creating_vertical_splits: is_creating_vertical_splits
		end

	set_split_horizontal
			-- Make new views be put into EV_HORIZONTAL_SPLIT_AREAs.
		do
			is_creating_vertical_splits := False
		ensure
			is_not_creating_vertical_splits: not is_creating_vertical_splits
		end

feature -- Querry

	has_view (a_view: VIEW): BOOLEAN
			-- Does current contain `a_view'?
		require
			view_exists: a_view /= Void
		do
			Result := views.has (a_view)
		ensure
			valid_result: Result implies views.has (a_view)
		end

	is_valid_view_index (a_index: INTEGER): BOOLEAN
			-- Is `a_index' in range from 1 to the number of views?
		do
			Result := views.valid_index (a_index)
		end

feature -- Basic operations

	wipe_out_views
			-- Clean out all the views in preperation for rebuilding.
		do
				-- Remove all objects from the VIEWs
			from views.start
			until views.exhausted
			loop
				views.item.remove_all_targets
				views.forth
			end
				-- Wipe out the containers
			views.wipe_out
			splits.wipe_out
			splitter_positions.wipe_out
			notebook.wipe_out
			visible_views.wipe_out
			disabled_views.wipe_out
				-- Wipe out the EV_CONTAINERs
			toggle_bar.wipe_out
			radio_bar.wipe_out
			toggle_menu.wipe_out
			radio_menu.wipe_out
				-- Reset the view to default state
			selected_view := default_view
			build_bars
		end

	extend_view (a_view: VIEW)
			-- Add `a_view', allowing Current to manage it's placement and
			-- make sure the view is the `selected_view'.
			-- This has the side effect on the parent (if any) of `a_view'
			-- in that it will remove `a_view' from it's parent before
			-- allowing Current to manage it.
		require
			view_exists: a_view /= Void
			not_has_view: not has_view (a_view)
		local
			pv: EV_CONTAINER
			ev_w: EV_WIDGET
		do
				-- Make sure the new view is not in some other widget.
			pv ?= a_view
			ev_w ?= a_view
			if pv /= Void and then pv.parent /= Void then
				pv.parent.prune (ev_w)
			end
					-- Must add a split area for each added view.
--			splits.extend (new_split_area)
			make_new_split_area
			views.extend (a_view)
			interface_table.block
			create_interface_items (a_view)
			interface_table.resume
			show_view (a_view)
			interface_table.update_interface
		ensure
			has_view: has_view (a_view)
			view_selected: selected_view = a_view
			split_area_for_each_view: splits.count = views.count
		end

	prune_view (a_view: VIEW)
			-- Remove `a_view' from current and clean out all the corresponding
			-- interface items.  Make sure a view (perhaps the `default_view')
			-- remains selected.
		require
			view_exists: a_view /= Void
		local
			i: INTEGER
			sa: EV_SPLIT_AREA
		do
			if has_view (a_view) then
				save_split_positions
				destroy_interface_items (a_view)
					-- Fix the node containing `a_view'.
				if i <= 2 then
						-- It was one of the first two views so delete the first split area
					splits.go_i_th (1)
					sa := splits.item
					splits.remove
				else
					splits.go_i_th (i - 1)
					sa := splits.item
					splits.remove
				end
					-- Must remove any views from the deleted split area so the views
					-- do not have a parent when `build_cell is called.
				sa.wipe_out
				views.start
				views.prune (a_view)
				visible_views.prune (a_view)
				disabled_views.prune (a_view)
				if selected_view = a_view then
						-- Must find some other view for it
					if views.count >= 1 then
						selected_view := views.first
					else
						selected_view := Default_view
					end
				end
--				if views.count = 1 then
--						-- It was two so rebuild the `bar' to remove
--						-- the `toggle_bar' or `radio_bar'.
					build_bars
--				end
				build_cell
				restore_split_positions
			end
		ensure
			not_has_view: not has_view (a_view)
			not_visible: not visible_views.has (a_view)
			not_disabled: not disabled_views.has (a_view)
			not_selected: selected_view /= a_view
		end

	select_view (a_view: VIEW)
			-- Make `a_view' the `selected_view'
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view) or else a_view.same_type (Default_view)
--			has_view: has_view (a_view)
		do
			selected_view := a_view
			syncronize
		ensure
			view_was_selected: selected_view = a_view
		end

	toggle_view (a_view: VIEW)
			-- If `a_view' is hidden then show it; if it is shown hide it.
			-- The effect from this will show up in split mode.
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view)
		local
			i: INTEGER
		do
			i := visible_views.count
			if visible_views.has (a_view) and then visible_views.count > 1 then
				visible_views.prune (a_view)
				save_split_positions
				syncronize
			else
				visible_views.extend (a_view)
				restore_split_positions
				select_view (a_view)
			end
			check
				count_changed_if_had_more_than_one_view:
					i > 1 implies (visible_views.count = i + 1 or else
									visible_views.count = i - 1)
			end
		ensure
			toggle_items_agree: toggles_agree_with_visible_items
		end

	show_all_views
			-- Make all the views visible
		do
			from views.start
			until views.exhausted
			loop
				visible_views.extend (views.item)
				views.forth
			end
			syncronize
		end

	show_view (a_view: VIEW)
			-- Make `a_view' visible
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view)
		do
			if not visible_views.has (a_view) then
				visible_views.extend (a_view)
				restore_split_positions
			end
			select_view (a_view)
		ensure
			view_visible: visible_views.has (a_view)
			view_selected: selected_view = a_view
		end

	hide_view (a_view: VIEW)
			-- React to a request to hide `a_view'
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view)
			is_in_split_mode: is_split_mode
		do
			if visible_views.count > 1 then
				visible_views.prune (a_view)
				if selected_view = a_view then
						-- Must select another
					select_view (visible_views.first)
				end
				syncronize
			end
		ensure
			view_not_visible: not visible_views.has (a_view)
			non_hidden_view_visible: (old selected_view = a_view) implies selected_view = visible_views.first
		end

	disable_view (a_view: VIEW)
			-- Prevent `a_view' from appearing and also remove the
			-- buttons and menus which will allow it to be shown.
			-- Use `enable_view' to restore it to normal.
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view)
		do
			save_split_positions
			disabled_views.extend (a_view)
--			build_bars
--			build_cell
			syncronize
		ensure
			is_in_disabled_views: disabled_views.has (a_view)
		end

	enable_view (a_view: VIEW)
			-- Sets the view handling for `a_view' back to normal
			-- by restoring the corresponding buttons and menues.
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view)
		do
			disabled_views.prune (a_view)
--			build_bars
--			build_cell
--			restore_split_positions
			syncronize
		ensure
			view_not_disabled: not disabled_views.has (a_view)
		end

	select_i_th_view (a_index: INTEGER)
			-- Make the `selected_view' reference the i-th view in `views'.
		require
			valid_index: is_valid_view_index (a_index)
		local
			wv: VIEW
		do
			wv := views.i_th (a_index)
			select_view (wv)
		ensure
			valid_result: selected_view = views.i_th (a_index)
		end

	set_single_mode_view (a_view: VIEW)
			-- Set to single mode with `a_view' visible, filling the `cell'.
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view)
--			mode_is_changable: is_mode_changable
		do
			set_single_mode
			if selected_view /= a_view then
				select_view (a_view)
			end
		ensure
			is_single_mode: is_single_mode
			correct_view_selected: selected_view = a_view
		end

	set_split_mode_view (a_view: VIEW)
			-- Multiple views can be visible within `cell'.
			-- Ensure `a_view' is the selected view.
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view)
--			mode_is_changable: is_mode_changable
		do
			set_split_mode
			if selected_view /= a_view then
				select_view (a_view)
			end
		ensure
			is_split_mode: is_split_mode
			correct_view_selected: selected_view = a_view
		end

	set_notebook_mode_view (a_view: VIEW)
			-- Change to notebook style with `a_view' on top.
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view)
--			mode_is_changable: is_mode_changable
		do
			set_notebook_mode
			if selected_view /= a_view then
				select_view (a_view)
			end
		ensure
			is_notebook_mode: is_notebook_mode
			correct_view_selected: selected_view = a_view
		end

feature {NONE} -- Implementation (Actions)

	on_mode_change
			-- The "mode" has changed, so rebuild the `bar' and `menu'
			-- and syncronize other items.
		do
			save_split_positions
			syncronize
		end

	on_tab_selected
			-- React to a request from the `notebook to select `a_view'
		require
			is_notebook_mode: is_notebook_mode
		local
			wv: VIEW
		do
			wv ?= notebook.selected_item
			select_view (wv)
		end

feature {NONE} -- Implementation (Basic operations)

	create_interface_items (a_view: VIEW)
			-- Make the buttons, menues, agents, etc. associated with this tool.
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view)
			view_not_disabled: not disabled_views.has (a_view)
		local
			tb: EV_TOOL_BAR_TOGGLE_BUTTON
			rb: EV_TOOL_BAR_RADIO_BUTTON
			ti: EV_CHECK_MENU_ITEM
			ri: EV_RADIO_MENU_ITEM
			t: TOOL
			ii: INTERFACE_ITEM
			int_item: INTERFACE_ITEM
			s: STRING
--			i: INTEGER
		do
				-- Create an interface item and add to the `interface_table'
			ii := interface_table.interface_item ("{VIEW_MANAGER} - create_view_string")
			s := interface_table.short_name ("{VIEW_MANAGER} - create_view_string") + " " + a_view.generating_type
			if not interface_table.has_key (s) then
				create int_item
				int_item.set_short_name (a_view.generating_type)
				int_item.set_tooltip (ii.short_name + a_view.generating_type)
				int_item.set_help_text (ii.help_text + a_view.generating_type)
				int_item.set_pixmap_file (interface_table.interface_item (a_view.generating_type).pixmap_file)
				interface_table.add_interface_item (int_item, s)
			end
				-- Create the items
			create tb
			create rb
			create ti
			create ri
			interface_table.register_widget (tb, s)
			interface_table.register_widget (rb, s)
			interface_table.register_widget (ti, s)
			interface_table.register_widget (ri, s)
				-- Store in the local lists.
			toggle_buttons.extend (tb)
			radio_buttons.extend (rb)
			toggle_items.extend (ti)
			radio_items.extend (ri)
			check
				toggle_buttons_parallel: toggle_buttons.index_of (tb, 1) = views.index_of (a_view, 1)
					-- because add a button for each added view.
			end
			check
				radio_buttons_parallel: radio_buttons.index_of (rb, 1) = views.index_of (a_view, 1)
					-- because add a button for each added view.
			end
			check
				toggle_items_parallel: toggle_items.index_of (ti, 1) = views.index_of (a_view, 1)
					-- because add an item for each added view.
			end
			check
				radio_items_parallel: radio_items.index_of (ri, 1) = views.index_of (a_view, 1)
					-- because add an item for each added view.
			end
--			toggle_bar.extend (tb)
--			radio_bar.extend (rb)
--			toggle_menu.extend (ti)
--			radio_menu.extend (ri)
				-- Enable the items since it was last selected
			tb.enable_select
			rb.enable_select
			ti.enable_select
			ri.enable_select
				-- Add agents to the new buttons and items
			tb.select_actions.extend (agent toggle_view (a_view))
			rb.select_actions.extend (agent select_view (a_view))
			ti.select_actions.extend (agent toggle_view (a_view))
			ri.select_actions.extend (agent select_view (a_view))
				-- Add actions to the buttons of `a_view' if it is a TOOL
			t ?= a_view
			if t /= Void then
					-- This item is a tool and therefore has buttons.
				t.maximize_actions.extend (agent set_single_mode_view (a_view))
				t.restore_actions.extend (agent set_split_mode_view (a_view))
				t.close_actions.extend (agent hide_view (a_view))
			end
			build_bars
		end

	destroy_interface_items (a_view: VIEW)
			-- Remove the buttons, and menu items corresponding to `a_view'.
		require
			view_exists: a_view /= Void
			has_view: has_view (a_view)
		local
			tb: EV_TOOL_BAR_TOGGLE_BUTTON
			rb: EV_TOOL_BAR_RADIO_BUTTON
			ti: EV_CHECK_MENU_ITEM
			ri: EV_RADIO_MENU_ITEM
			i: INTEGER
		do
			i := views.index_of (a_view, 1)
--			b := toggle_bar.i_th (i)
--			toggle_bar.prune (b)
--			interface_table.unregister_widget (b)
--			b := radio_bar.i_th (i)
--			radio_bar.prune (b)
--			interface_table.unregister_widget (b)
--			mi := radio_menu.i_th (i)
--			radio_menu.prune (mi)
--			interface_table.unregister_widget (mi)
--			mi := toggle_menu.i_th (i)
--			toggle_menu.prune (mi)
--			interface_table.unregister_widget (mi)
			rb := radio_buttons.i_th (i)
			radio_buttons.prune (rb)
			interface_table.unregister_widget (rb)
			tb := toggle_buttons.i_th (i)
			toggle_buttons.prune (tb)
			interface_table.unregister_widget (tb)
			ri := radio_items.i_th (i)
			radio_items.prune (ri)
			interface_table.unregister_widget (ri)
			ti := toggle_items.i_th (i)
			toggle_items.prune (ti)
			interface_table.unregister_widget (ti)
			build_bars
		end

	build_cell
			-- Put the appropriate split area or view into `cell' based on user
			-- button or menu selections.  This must be done whenever the list
			-- of visible_views changes or when a view is disabled.
		local
			v: VIEW
			ev_w: EV_WIDGET
			sa: EV_SPLIT_AREA
			prev_sa: EV_SPLIT_AREA
			i: INTEGER
--			b: EV_TOOL_BAR_RADIO_BUTTON
		do
				-- Must wipeout all the containers to prevent putting a view into
				-- more than one container at a time.
				-- This requires building the notebook here if `is_notebook_mode'
			cell.wipe_out
			notebook.selection_actions.block
			notebook.wipe_out
			splits.do_all (agent {EV_SPLIT_AREA}.wipe_out)
			if is_single_mode then
				ev_w ?= selected_view
				check
					widget_exists: ev_w /= Void
						-- because all managed `views' should be EV_WIDGETS
				end
				cell.extend (ev_w)
			elseif is_notebook_mode then
				from views.start
				until views.exhausted
				loop
					v := views.item
					if not disabled_views.has (v) then
						ev_w ?= v
						check
							widget_exists: ev_w /= Void
								-- because all managed `views' should be EV_WIDGETS
						end
						notebook.extend (ev_w)
						notebook.set_item_text (ev_w, v.generating_type)
					end
					views.forth
				end
				ev_w ?= selected_view
				check
					widget_exists: ev_w /= Void
						-- because all managed `views' should be EV_WIDGETS
				end
				notebook.select_item (ev_w)
				cell.extend (notebook)
			else
				check
					is_split_mode: is_split_mode
							-- because only mode left
				end
					-- The view is is split mode so must find the "truely visible" views.  A
					-- view is "truely visible" if it is in the `visible_views' set and not
					-- disabled.
				if visible_views.count - disabled_views.count = 0 then
						-- All the views are disabled.
					cell.extend (Default_view)
				elseif visible_views.count - disabled_views.count = 1 then
						-- Only one visible view is not also disabled, so find it.
					from visible_views.start
					until not disabled_views.has (visible_views.item)
					loop
						visible_views.forth
					end
					check
						visible_views_not_exhausted: not visible_views.exhausted
							-- because the one view will be found in loop conditional
							-- exiting the loop before the call to first.
					end
					v := visible_views.item
					ev_w ?= v
					check
						widget_exists: ev_w /= Void
							-- because all managed `visible_views' should be EV_WIDGETS
					end
					cell.extend (ev_w)
				else
						-- There are at least two shown views.
						-- Find the first visible view.  If there are two
						-- visible views `v' will be set before the list
						-- becomes exhausted.
					from views.start
						-- NOTE:  The `views' set is used instead of the `visible_views' set
						-- because the order of `views' is important.  The order in which Views
						-- were added to `visible_views' and `disabled_views' is not significant
						-- and could give an unpredictable result.
					until v /= Void or else views.exhausted
					loop
						if visible_views.has (views.item) and then not disabled_views.has (views.item) then
							v := views.item
							ev_w ?= v
							check
								widget_exists: ev_w /= Void
									-- because all managed `views' should be EV_WIDGETS
							end
							prev_sa := splits.i_th (views.index)
							prev_sa.set_first (ev_w)
							cell.extend (prev_sa)
						end
						views.forth
						check
							not_exhausted: not views.exhausted
								-- because with at least 2 visible and un-disabled views;
								-- `v' will be non-void first
						end
					end
					check
						first_view_found: v /= Void
							-- because there are at least two visible and un-disabled views,
							-- the first view should have been found
					end
							-- Now start adding views within appropriate split areas.
					from 	-- where we left off in `views'
					until views.exhausted
					loop
						v := views.item
						i := views.index_of (v, 1)
						sa := splits.i_th (i)
							-- Must look ahead to determine what goes in the
							-- second half of the `prev_sa', a view or another
							-- split area.
						views.forth
						if visible_views.has (v) and not disabled_views.has (v) then
							ev_w ?= v
							check
								widget_exists: ev_w /= Void
									-- because all managed `views' should be EV_WIDGETS
							end
							if views.exhausted then
								prev_sa.set_second (ev_w)
							else
								prev_sa.set_second (sa)
								sa.set_first (ev_w)
								prev_sa := sa
							end
						end
					end
					restore_split_positions
				end
			end
			notebook.selection_actions.resume
		end

	build_bars
			-- Put the correct buttons in the `bar' and the correct
			-- menu items into `menu'.  Make sure the button and menu
			-- states agree with the state of Current.
		local
			i: INTEGER
		do
				-- build the `radio_bar', `toggle_bar', `toggle_menu', and `radio_menu'
			toggle_bar.wipe_out
			radio_bar.wipe_out
			toggle_menu.wipe_out
			radio_menu.wipe_out
			from views.start
			until views.exhausted
			loop
				if not disabled_views.has (views.item) then
					i := views.index
					toggle_bar.extend (toggle_buttons.i_th (i))
					radio_bar.extend (radio_buttons.i_th (i))
					toggle_menu.extend (toggle_items.i_th (i))
					radio_menu.extend (radio_items.i_th (i))
				end
				views.forth
			end
				--  Build the bar
			bar.wipe_out
			if views.count >= 2 then
				if not is_mode_frozen then
					bar.extend (mode_bar)
					bar.disable_item_expand (mode_bar)
				end
				if not is_view_frozen then
					if is_single_mode or else is_notebook_mode then
						bar.extend (radio_bar)
						bar.disable_item_expand (radio_bar)
					elseif is_split_mode then
						bar.extend (toggle_bar)
						bar.disable_item_expand (toggle_bar)
					end
				end
			end
				-- Build the menu
			menu.wipe_out
			if not is_mode_frozen then
				menu.extend (mode_menu)
			end
			if not is_view_frozen then
				if is_single_mode or else is_notebook_mode then
					menu.extend (radio_menu)
				elseif is_split_mode then
					menu.extend (toggle_menu)
				end
			end
			if views.count >= 2 then
				mode_menu.enable_sensitive
				radio_menu.enable_sensitive
				toggle_menu.enable_sensitive
			else
				mode_menu.disable_sensitive
				radio_menu.disable_sensitive
				toggle_menu.disable_sensitive
			end
		end

	save_split_positions
			-- Save the position of splitters for restoration later.
		local
			sa: EV_SPLIT_AREA
		do
			from
				splits.start
				splitter_positions.start
			until splits.exhausted
			loop
				sa := splits.item
				if sa.full then
					splitter_positions.replace (sa.split_position)
				end
				splits.forth
				splitter_positions.forth
			end
		end

	restore_split_positions
			-- Set the split position of each visible split area to what it was
			-- when it was last visible.
		local
			sa: EV_SPLIT_AREA
			i: INTEGER
			test_min, test_sp: INTEGER
		do
			from
				splits.start
				splitter_positions.start
			until splits.exhausted
			loop
				sa := splits.item
				i := splitter_positions.item
				test_sp := sa.split_position
				test_min := sa.minimum_split_position
				if sa.full then
					if i < sa.minimum_split_position then
						sa.set_split_position (sa.minimum_split_position)
					elseif i > sa.maximum_split_position then
						sa.set_split_position (sa.maximum_split_position)
					else
						sa.set_split_position (i)
					end
				end
				splits.forth
				splitter_positions.forth
			end
		end

	syncronize
			-- Rebuild all the externally accessed widgets such as `cell', `menu',
			-- and `button_bar' reflect the state of the buttons.
			-- Also ensure that at least one view is visible if there are any views.
			-- and update button and menu states.
		do
			build_cell
			build_bars
			syncronize_mode_selection_items
			if not views.is_empty then
				syncronize_tools
--				restore_split_positions
				syncronize_view_selection_items
			end
		end

	syncronize_tools
			-- Make sure any TOOLs handled by the manager are in the correct
			-- state, either maximized or normal.
		local
			t: TOOL
		do
			from views.start
			until views.exhausted
			loop
				t ?= views.item
				if t /= Void then
					if views.count <= 1 then
						t.disable_resize
					else
						t.enable_resize
					end
					if is_single_mode then
						if t.is_resizable then
							t.maximize
						end
					else
						if t.is_resizable then
							t.restore
						end
					end
					if is_split_mode then
						t.close_button.enable_sensitive
						t.maximize_button.enable_sensitive
						t.restore_button.enable_sensitive
					else
						t.close_button.disable_sensitive
						t.maximize_button.disable_sensitive
--						t.restore_button.disable_sensitive
					end
				end
				views.forth
			end
		end

	syncronize_view_selection_items
			-- Make sure the state of the radio and toggle reflect the value
			-- of `selected_view' and items in `visible_views'.
		local
			i: INTEGER
			wv: VIEW
			rb: EV_TOOL_BAR_RADIO_BUTTON
			tb: EV_TOOL_BAR_TOGGLE_BUTTON
			ri: EV_RADIO_MENU_ITEM
			ti: EV_CHECK_MENU_ITEM
		do
					-- Fix the radio items
			i := views.index_of (selected_view, 1)
			rb := radio_buttons.i_th (i)
			ri := radio_items.i_th (i)
			ri.select_actions.block
			ri.enable_select
			ri.select_actions.resume
			rb.select_actions.block
			rb.enable_select
			rb.select_actions.resume
					-- Fix the toggle items
			from views.start
			until views.exhausted
			loop
				i := views.index
				wv := views.item
				if not disabled_views.has (wv) then
					tb := toggle_buttons.i_th (i)
					ti := toggle_items.i_th (i)
					ti.select_actions.block
					tb.select_actions.block
					if visible_views.has (wv) then
						ti.enable_select
						tb.enable_select
					else
						ti.disable_select
						tb.disable_select
					end
					ti.select_actions.resume
					tb.select_actions.resume
				end
				views.forth
			end
		ensure
			selected_view_unchanged: selected_view = old selected_view
			toggle_items_agree: toggles_agree_with_visible_items
			radio_items_agree: radios_agree_with_selected_view
		end

	syncronize_mode_selection_items
			-- Set the button states
		do
			single_mode_button.select_actions.block
			split_mode_button.select_actions.block
			notebook_mode_button.select_actions.block
			single_mode_item.select_actions.block
			split_mode_item.select_actions.block
			notebook_mode_item.select_actions.block
			if is_split_mode then
				split_mode_button.enable_select
				split_mode_item.enable_select
			elseif is_notebook_mode then
				notebook_mode_button.enable_select
				notebook_mode_item.enable_select
			elseif is_single_mode then
				single_mode_button.enable_select
				single_mode_item.enable_select
			else
				check
					no_mode_should_not_happen: False
				end
			end
			single_mode_button.select_actions.resume
			split_mode_button.select_actions.resume
			notebook_mode_button.select_actions.resume
			single_mode_item.select_actions.resume
			split_mode_item.select_actions.resume
			notebook_mode_item.select_actions.resume
		ensure
			split_mode_implication: is_split_mode implies
				split_mode_button.is_selected and split_mode_item.is_selected and
				not notebook_mode_button.is_selected and not notebook_mode_item.is_selected and
				not single_mode_button.is_selected and not single_mode_item.is_selected
			notebook_mode_implication: is_notebook_mode implies
				notebook_mode_button.is_selected and notebook_mode_item.is_selected and
				not single_mode_button.is_selected and not single_mode_item.is_selected and
				not split_mode_button.is_selected and not split_mode_item.is_selected
			single_mode_implication: is_single_mode implies
				single_mode_button.is_selected and single_mode_item.is_selected and
				not notebook_mode_button.is_selected and not notebook_mode_item.is_selected and
				not split_mode_button.is_selected and not split_mode_item.is_selected
		end

	make_new_split_area	--: EV_SPLIT_AREA is
			-- Create a new EV_SPLIT_AREA.  The direction of the split is based
			-- on the direction of the last one stored in `is_last_split_horizontal'.
		local
			sa: EV_SPLIT_AREA
		do
			if is_creating_vertical_splits then
				sa := create {EV_VERTICAL_SPLIT_AREA} -- Result
			else
				sa := create {EV_HORIZONTAL_SPLIT_AREA}	-- Result
			end
--			splits.extend (Result)
--			splitter_positions.extend (Result.split_position)
			splits.extend (sa)
			splitter_positions.extend (sa.split_position)
		end

feature {VIEW_MANAGER_STATE} -- Implementation

--	pixmap_size: INTEGER
			-- Size to make the pixmaps placed on buttons.

	views: LINKED_SET [VIEW]
			-- All the views which Current will manage

	visible_views: LINKED_SET [VIEW]
			-- List of views which are visible if `is_split_mode'.

	disabled_views: LINKED_SET [VIEW]
			-- Set of views managed by Current but which are temporarily
			-- masked from view (along with the corresponding buttons, etc.)

	splits: LINKED_SET [EV_SPLIT_AREA]
			-- Keep track of all split areas created so they can be wiped out.
			-- Otherwise, may attempt to place a widget in the `cell' which is
			-- still in one of the split areas.

	splitter_positions: LINKED_LIST [INTEGER]
			-- The last position of all the split area splitters.
			-- There should remain a one-to-one correspondense between the number
			-- of slit areas in `splits' and the number of intergers in this.

	radios_agree_with_selected_view: BOOLEAN
			-- Used by invariant to make sure the check menus and radio
			-- buttons so the correct view (the same as `selected_view).
		local
			i, disabled_views_encountered: INTEGER
			rb: EV_TOOL_BAR_RADIO_BUTTON
			ri: EV_RADIO_MENU_ITEM
		do
			i := views.index_of (selected_view, 1)
			from views.start
			until views.exhausted or else views.index > i
			loop
				if disabled_views.has (views.item) then
					disabled_views_encountered := disabled_views_encountered + 1
				end
				views.forth
			end
			i := i - disabled_views_encountered
			rb ?= radio_bar.i_th (i)
			ri ?= radio_menu.i_th (i)
			Result := rb.is_selected and ri.is_selected
		end

	toggles_agree_with_visible_items: BOOLEAN
			-- Used by invariant to make sure the buttons in `toggle_bar' and
			-- the check items in `radio_menu' indicate the correct set
			-- of `visible_views'.
		local
			i, disabled_views_encountered: INTEGER
			tb: EV_TOOL_BAR_TOGGLE_BUTTON
			ti: EV_CHECK_MENU_ITEM
			wv: VIEW
		do
			Result := True
			i := 1
			from views.start
			invariant
				i >= 1
				i <= toggle_bar.count
				i <= toggle_menu.count
			until views.exhausted or else not Result
			loop
				wv := views.item
				if disabled_views.has (wv) then
					disabled_views_encountered := disabled_views_encountered + 1
				end
				i := views.index - disabled_views_encountered
				tb ?= toggle_bar.i_th (i)
				ti ?= toggle_menu.i_th (i)
				if visible_views.has (wv) then
					Result := tb.is_selected and ti.is_selected
				else
					Result := not tb.is_selected and not ti.is_selected
				end
				views.forth
			end
		end

feature {NONE} -- Implementation (local containers for VIEW buttons and menu items)

	radio_buttons: LINKED_SET [EV_TOOL_BAR_RADIO_BUTTON]
			-- Buttons corresponding to the VIEWs (one for each view).
			-- Maintained by `create_interface_items' and `destroy_interface_items'.

	toggle_buttons: LINKED_SET [EV_TOOL_BAR_TOGGLE_BUTTON]
			-- Has one button for each view.
			-- Maintained by `create_interface_items' and `destroy_interface_items'.

	radio_items: LINKED_SET [EV_RADIO_MENU_ITEM]
			-- Has one item for each view.
			-- Maintained by `create_interface_items' and `destroy_interface_items'.

	toggle_items: LINKED_SET [EV_CHECK_MENU_ITEM]
			-- Has one item for each view.
			-- Maintained by `create_interface_items' and `destroy_interface_items'.

feature {NONE} -- Implementation (tool bars and buttons)

	notebook: EV_NOTEBOOK
			-- Used internally to build `cell' when `notebook_mode_button' is selected.

	mode_bar: EV_TOOL_BAR
			-- Holds the view mode buttons ('build_mode_button', 'single_mode_button',
			-- `normal_mode_button', and `notebook_mode_button'.

	toggle_bar: EV_TOOL_BAR
			-- Holds buttons which toggle multiple views on and off.

	radio_bar: EV_TOOL_BAR
			-- Holds buttons used to select one of several views.

	single_mode_button: EV_TOOL_BAR_RADIO_BUTTON
			-- Used to put the views into a maximized state

	split_mode_button: EV_TOOL_BAR_RADIO_BUTTON
			-- Puts the manager into normal view mode (all selected views are visible).

	notebook_mode_button: EV_TOOL_BAR_RADIO_BUTTON
			-- Puts the manager into a mode in which the views are
			-- placed withing an EV_NOTEBOOK.

feature {NONE} -- Implementation (menus and items)

	mode_menu: EV_MENU
			-- Holds menu items for changing the mode

	toggle_menu: EV_MENU
			-- Holds menu items used when not `is_maximized' and not `is_builder_mode'
			-- to toggle views on and off or to `maximize' the views.

	radio_menu: EV_MENU
			-- Holds menu items used when `is_maximized' and not `is_builder_mode'
			-- to select one of several views or to `restore' the views.

	single_mode_item: EV_RADIO_MENU_ITEM
			-- Maximizes the `selected_view'

	split_mode_item: EV_RADIO_MENU_ITEM
			-- Reverts to `is_split_mode'

	notebook_mode_item: EV_RADIO_MENU_ITEM
			-- Changes to notebook mode

feature {NONE} -- Implementation

	split_manager: SPLIT_MANAGER
			-- Manages views in the split areas.

invariant

	valid_mode: is_split_mode or is_single_mode or is_notebook_mode
	mode_button_selected: split_mode_button.is_selected or single_mode_button.is_selected or notebook_mode_button.is_selected

	single_mode_implication: is_single_mode implies (not is_split_mode and not is_notebook_mode)
	split_mode_implication: is_split_mode implies (not is_single_mode and not is_notebook_mode)
	notebook_mode_implication: is_notebook_mode implies (not is_split_mode and not is_single_mode)

	selected_view_not_void: selected_view /= Void
	no_views_implies_default_selected: views.count = 0 implies selected_view.same_type (Default_view)
	selected_view_not_disabled: not disabled_views.has (selected_view)
	split_area_for_each_view: splits.count = views.count
	position_for_each_split_area: splitter_positions.count = splits.count

		-- Maintain buttons and menus in parallel to `views'.
	same_number_radio_buttons: radio_buttons.count = views.count
	same_number_toggle_buttons: toggle_buttons.count = views.count
	same_number_radio_items: radio_items.count = views.count
	same_number_toggle_items: toggle_items.count = views.count

--	selected_view_always_visible: selected_view /= Void implies visible_views.has (selected_view)

--	two_or_more_views_and_changable_modes_implies_mode_bar:
--		(views.count >= 2 and is_mode_changable) implies bar.has (mode_bar)
--	mode_bar_implies_two_or_more_views_and_changable_modes:
--		bar.has (mode_bar) implies (views.count >= 2 and is_mode_changable)

--	radio_button_of_selected_item_is_selected: radio_bar /= Void implies radios_agree_with_selected_item
--	notebook_tab_for_each_view: is_notebook_mode implies notebook.count = count


end -- class VIEW_MANAGER
