note
	description: "[
				TOOL used to hold a INTERFACE_TABLE_VIEW to allow editting
				of the "interface_names" for a program.
			]"
	author: "Jimmy J. Johnson"
	date: "14 Jan 04"

class
	INTERFACE_TABLE_TOOL

inherit

	JJ_VISION_FILE_FACILITIES
		export
			{NONE}
				all
		undefine
			default_create,
			copy
		end

	TOOL
		rename
			target as table,
			set_target as set_table
		redefine
			initialize,
			initialize_interface,
			set_actions,
			table,
			set_table,
			draw
		end

create
	default_create

feature {NONE} -- Initialization

	initialize
			-- Set up the window
			-- Create the views in the window using agents in order to demostrate the
			-- ability to recreate a view from a script.  The create view calls could
			-- be scripted somehow.
		do
			create changed_item_keys.make
			create item_view
			create filename_view
			create table_view
			changed_item_keys.compare_objects	-- because it holds strings
				-- Create the new buttons before Precursor {TOOL} so `initialize'
				-- can call `set_actions' without a Void reference.
			create open_button
			create save_button
			create save_as_button
			create apply_button
			create xml_button
				-- "Register" the widgets with the `interface_table' in use by the
				-- current program (from SHARED in the "Standard Interface" cluster).
			interface_table.register_widget (open_button, "{INTERFACE_TABLE_TOOL}.open_button")
			interface_table.register_widget (save_button, "{INTERFACE_TABLE_TOOL}.save_button")
			interface_table.register_widget (save_as_button, "{INTERFACE_TABLE_TOOL}.save_as_button")
			interface_table.register_widget (apply_button, "{INTERFACE_TABLE_TOOL}.apply_button")
			interface_table.register_widget (xml_button, "{INTERFACE_TABLE_TOOL}.xml_button")
			interface_table.register_widget (Current, generating_type)
				-- Create the tool and its views
			Precursor {TOOL}
				-- Add the buttons to the `tool_bar' (from TOOL)
			tool_bar.extend (open_button)
			tool_bar.extend (save_button)
			tool_bar.extend (save_as_button)
			tool_bar.extend (apply_button)
			tool_bar.extend (xml_button)
				-- Set up the views in this tool.
			split_manager.enable_mode_changes
			split_manager.set_split_mode
			split_manager.set_horizontal
			split_manager.extend (filename_view)
			split_manager.set_vertical
			split_manager.extend (table_view)
			split_manager.extend (item_view)
			set_actions_for_views
			set_table (interface_table)
		end

	initialize_interface
			-- Set up the `interface_table' to include items needed by Current.
		do
				-- Only add an interface item for Current if its `interface_name' has
				-- not already been added; the other items are specific for this tool
				-- and most likely will not be redefine, so no need to check inclusion
				-- of these in the `interface_table'.
				-- In redefinitions of this feature call this at the end of the redefinition
				-- so an item for the `interface_name' can be added to the `interface_table'.
			if not interface_table.has_key (generating_type) then
				interface_table.add_item_with_tuple (<< generating_type, "INTERFACE_TABLE_TOOL", "Tool used to display info about the interface names.", "Tool used to display info about the interface names.", "icon_bkpt_info_color.ico">>)
			end
			interface_table.add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL} - default interface names", "Default Interface Names", "Default interface names are displayed.", "The default interface names are displayed.", "icon_dummy.ico">>)
			interface_table.add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}.open_button", "Open", "Open Table", "Read in a file of interface names.", "icon_new_measure_color.ico">>)
			interface_table.add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}.save_button", "Save", "Save Table", "Save the interface names to a file.", "icon_save_measure_color.ico">>)
			interface_table.add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}.save_as_button", "Save as", "Save Table as new file.", "Opens the file save dialog allowing the displayed names to be save as a new file.", "icon_save_measure_color.ico">>)
			interface_table.add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}.apply_button", "Apply", "Apply changes", "Update all the interface items with those displayed in the tool.", "icon_view_measure_plus_color.ico">>)
			interface_table.add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}.xml_button", "temp XML", "Temp button to test XML parser", "Temporary functionf for testing the XML parser routines and classes.", "icon_view_measure_plus_color.ico">>)
				-- Call Precursor after current is set up.  Specifically after the interface
				-- item for Current's `interface_name' is added.  (see above)
			Precursor {TOOL}
		end

	set_actions
			-- Add actions to the widgets
		do
			Precursor {TOOL}
			open_button.select_actions.extend (agent on_open_button_pressed)
			save_button.select_actions.extend (agent on_save_button_pressed)
			save_as_button.select_actions.extend (agent on_save_as_button_pressed)
			apply_button.select_actions.extend (agent on_apply_button_pressed)
--			xml_button.select_actions.extend (agent on_xml_button_pressed)
		end

	set_actions_for_views
			-- Add actions to the views.  This can only be done after the views are
			-- created.  Because `set_actions' is called in the precursor to `initialize'
			-- this feature had to be added in order to access the "views" after they
			-- are created in `initialize' with the calls to `register_view_function'
			-- from `view_manager'.
		do
			table_view.select_actions.extend (agent on_item_selected_in_table_view)
			item_view.short_name_text.key_release_actions.extend (agent on_item_changed)
			item_view.tooltip_text.key_release_actions.extend (agent on_item_changed)
			item_view.help_text_text.key_release_actions.extend (agent on_item_changed)
			item_view.pixmap_file_text.key_release_actions.extend (agent on_item_changed)
			item_view.pixmap_file_text.change_actions.extend (agent on_item_changed (create {EV_KEY}))
		end

feature -- Access

	table: INTERFACE_TABLE
			-- Renamed `target' (from VIEW) and redefined to changed type.

	table_view: INTERFACE_TABLE_VIEW
			-- View for displaying the `interface_table'.

	item_view: INTERFACE_ITEM_VIEW
			-- View for displaying a selected item.

	filename_view: FILENAME_VIEW
			-- View for displaying pixmaps, etc.

feature -- Element change

	set_table (a_table: like table)
			-- Change `table'
		do
			Precursor {TOOL} (a_table)
			table_view.set_table (a_table)
			draw
		end

feature -- Basic operations

	draw
			-- Builds the string shown at top of the tool in `viewable_label'
			-- as the concatination of the `name' of this tool and the name
			-- of the file from which `names' was read.
		do
			target_label.set_text (table.short_name ("{INTERFACE_TABLE_TOOL} - default interface names"))
			if table /= interface_table then
				target_label.set_text (target_label.text + "  " + interface_table.short_name ("not in system text"))
			end
		end

feature {NONE} -- Implementation (actions)

--	on_xml_button_pressed is
--			-- Temporary for testing
--		local
--			p: INTERFACE_FILE_PARSER
--			fn: STRING
--		do
--			fn := "d:\test.xml"
--			io.put_string ("Testing XML file reading %N")
--			create p.make
--			p.set_file_name (fn)
--			p.parse
--		end

	on_open_button_pressed
			-- React to a press of the `open_button' by prompting
			-- the user for a file and then reading in the data.
		do
			set_save_object (interface_table.names)
			validate_file_name_and_open
		end

	on_save_button_pressed
			-- React to a press of the `save_button'.
		do
			set_save_object (interface_table.names)
			validate_file_name_and_save
			draw
		end

	on_save_as_button_pressed
			-- React to a press of the `save_as_button'.
		do
			set_save_object (interface_table.names)
			show_save_dialog
			draw
		end

	on_apply_button_pressed
			-- React to a press of the `apply_button'.
		do
			from changed_item_keys.start
			until changed_item_keys.exhausted
			loop
				table.update_widgets (changed_item_keys.item)
				changed_item_keys.forth
			end
			draw
--			table.names.touch
		end

--	on_table_changed is
--			-- Update the appropriate sub-views with the new `table'.
--		local
--			vl: LINEAR [VIEW]
--			v: VIEW
--		do
--			vl := split_manager.leaf_nodes
--			from vl.start
--			until vl.exhausted
--			loop
--				v := vl.item
--				if v.same_type (table_view) then -- or v.same_type (new_filename_view) then
--					v.add_target (table)
--					v.draw
--				end
--				vl.forth
--			end
--		end

	on_item_selected_in_table_view (a_row: EV_MULTI_COLUMN_LIST_ROW)
			-- React to a selection of an item in the `table_view'.
		local
			ii: INTERFACE_ITEM
		do
			ii ?= a_row.pebble
			check
				interface_item_exists: ii /= Void
					-- because pebbles in rows are INTERFACE_ITEMS (see INTERFACE_TABLE_VEIW.draw)
			end
			item_view.set_interface_item (ii)
			item_view.draw
		end

	on_item_changed (a_key: EV_KEY)
			-- React to a change of one of the texts in the `item_view' by updating
			-- both the interface item of the selected row and the text in the
			-- `selected_row' of the `table_view'.
		local
			r: EV_MULTI_COLUMN_LIST_ROW
			k: STRING
			ii: INTERFACE_ITEM
		do
				-- Get the currently selected row from `table_view'.
			r := table_view.selected_item
				-- Don't know which column has changed so update the interface item
				-- from each column of the currently selected row.
			if r /= Void then
				r.put_i_th (item_view.short_name_text.text, 2)
				r.put_i_th (item_view.tooltip_text.text, 3)
				r.put_i_th (item_view.help_text_text.text, 4)
				r.put_i_th (item_view.pixmap_file_text.text, 5)
					-- Also update the interface item from the text in the row.
					-- The first column of `table_view' (therefore the first item
					-- in the row) is used as a key in the table.
				k := r.i_th (1)
				changed_item_keys.extend (k)
					-- Find the corresponding interface item.
				ii := table.interface_item (k)
				ii.set_short_name (r.i_th (2))
				ii.set_tooltip (r.i_th (3))
				ii.set_help_text (r.i_th (4))
				ii.set_pixmap_file (r.i_th (5))
					-- Update all the other views in which `table' resides
				disable_drawing
				table_view.disable_drawing
				draw_views (table)
				enable_drawing
				table_view.enable_drawing
					-- Also update any other views which contain the changed interface_item.
				item_view.disable_drawing
				draw_views (ii)
				item_view.enable_drawing
			end
		end

feature {NONE} -- Implementation

	changed_item_keys: LINKED_SET [STRING]
			-- Set of keys to the interface items which have changed since the
			-- last time the widgets in have been updated.

	open_button: EV_TOOL_BAR_BUTTON
			-- Open a interface names file for editting.

	save_button: EV_TOOL_BAR_BUTTON
			-- To save the interface names to persistent storage.

	save_as_button: EV_TOOL_BAR_BUTTON
			-- To write the interface names to persistent storage as a new file.

	apply_button: EV_TOOL_BAR_BUTTON
			-- To make the program use the currently editted interface names.

	xml_button: EV_TOOL_BAR_BUTTON
			-- temp button to test xml file reading.
-- FIX ME !!


end

