note
	description: "[
				A VIEW for displaying an EDITABLE using it's `schema'.
				]"
	author: "Jimmy J. Johnson"
	date: "30 Mar 04"

class
	DIALOG_EDITOR_VIEW

inherit

	EV_STOCK_COLORS
		rename
			implementation as colors_implementation
		export
			{NONE} all
		undefine
			default_create,
			is_equal,
			copy
		end

	VIEW
		rename
			target as record,
			set_target as set_record
		undefine
--			default_create,
			copy,
			is_equal
		redefine
			initialize,
			initialize_interface,
			record,
			set_record,
			destroy,
			is_destroyed,
			draw
		end

--	EV_CELL
	EV_FRAME
		redefine
			initialize,
--			data,		-- to make it inaplicable
--			set_data	-- to make it inaplicable
			destroy,
			is_destroyed
		end

create
	default_create

feature {NONE} -- Initialization

	initialize
			-- Create an editor view.
		do
			Precursor {EV_FRAME}
			Precursor {VIEW}
			create changed_controls.make
			create controls.make
			create save_actions
			create control_select_actions
			create scrollable_area
			extend (scrollable_area)
			set_save_on_change
			set_actions
		end

	initialize_interface
			-- Create INTERFACE_ITEMs which could be used by Current.
			-- Called by `initialize'.
		do
			Precursor {VIEW}
			interface_table.add_item_with_tuple (<< generating_type, "DIALOG_EDITOR_VIEW","A DIALOG_EDITOR_VIEW object.", "This object is of type DIALOG_EDITOR_VIEW.", "icon_editing.ico">>)
		end

	set_actions
			--
		do
			scrollable_area.item.drop_actions.extend (agent on_drop_editable)
		end

feature -- Access

	scrollable_area: EDITOR_SCROLL_AREA


	record: EDITABLE
			-- The object that contains the data to be editted.
			-- Data in `record' is reached via feature `value (a_key)'.

	schema: SCHEMA
			-- Current schema in use by the editor.

	save_actions: EV_NOTIFY_ACTION_SEQUENCE
			-- Actions to be performed after the record is saved.

	control_select_actions: EV_NOTIFY_ACTION_SEQUENCE
			-- Actions to be performed when a control is clicked.

feature -- Element change

	set_record (a_record: like record)
			-- Change `record' to `a_record'
			-- Reload the data into the controls
		do
			Precursor {VIEW} (a_record)
			draw
		end

	set_schema (a_schema: like schema)
			-- Change `schema' to `a_schema'.
			-- Uses the pattern described in feature `set_target' from VIEW to
			-- add `a_schema' to the `object_set' (the set of objects contained
			-- in this view) and removes the old schema (if it had one) from
			-- the `object_set'.
			-- Also, removes old/add new fields contained in `a_schema'.
		require
			schema_exists: a_schema /= Void
--		local
--			temp: like record
		do
				-- Save the old `record'
			if a_schema /= schema then
--				temp := record
--				remove_all_targets
--				if temp /= Void then
--					set_record (temp)				
--				end
--				add_target (a_schema)
--				schema := a_schema
--				from schema.start
--				until schema.exhausted
--				loop
--					add_target (schema.field)
--					schema.forth
--				end
--				draw
				if schema /= Void then
					target_set.prune (schema)
				end
				schema := a_schema
				target_set.extend (schema)
				draw
			end
		ensure
			schema_assigned: schema = a_schema
			contains_schema: target_set.has (a_schema)
			old_schema_removed: (old schema /= Void) and (old schema /= schema) implies not target_set.has (old schema)
		end

feature -- Basic operations

	draw
			-- Rebuild and show the view.
		do
			if schema /= Void then
				if is_rebuild_needed then
					build_controls
				end
				if record /= Void then
					fill_controls
					if record.has_schema (schema) then
						enable_controls
					else
						disable_controls
					end
				else
					disable_controls
				end
			end
		end

	disable
			-- Make the controls unresponsive to input.
		do
			is_user_disabled := True
		end

	enable
			-- Make the controls responsive to input.
		do
			is_user_disabled := False
		end

	destroy
			-- Destroy underlying native toolkit object.
			-- Render `Current' unusable.
		do
			Precursor {VIEW}
			Precursor {EV_FRAME}
		end

feature -- Status report

	is_destroyed: BOOLEAN
			-- Is `Current' no longer usable?
		do
			Result := Precursor {VIEW} and Precursor {EV_FRAME}
		end

	is_user_disabled: BOOLEAN
			-- Has the user requested the controls to be disabled?

	is_save_on_change: BOOLEAN
			-- Are changes made in the controls automatically
			-- saved to `record'?  In other words, is `record'
			-- updated anytime a change is made, or must it
			-- be done manually with a call to `save_record'?

feature -- Status setting

	set_save_on_change
			-- Make automatic updates to `record' whenever
			-- a change is made in any control.
		do
			is_save_on_change := True
		end

	set_save_on_request
			-- Require `save_record' to be called in order to
			-- accept any changes make in any control.
		do
			is_save_on_change := False
		end

feature -- Basic operations

	disable_controls
			-- Disable all controls.
		local
			c: CONTROL
		do
			from controls.start
			until controls.exhausted
			loop
				c := controls.item
				c.disable_sensitive
				c.parent.set_foreground_color (Red)
				c.parent.propagate_foreground_color
				controls.forth
			end
		end

	enable_controls
			-- Enable all controls.
		local
			c: CONTROL
		do
			from controls.start
			until controls.exhausted
			loop
				c := controls.item
				c.enable_sensitive
				c.parent.set_foreground_color (Black)
				c.parent.propagate_foreground_color
				controls.forth
			end
		end

feature {NONE} -- Implementation

	build_controls
			-- Create controls for each FIELD in `schema'.
		require
			schema_exists: schema /= Void
		local
			f: FIELD
			c: CONTROL
			ef: EDITOR_FIXED
		do
			ef ?= scrollable_area.item
			check
				editor_exists: ef /= Void
					-- Because `scrollable_area' is an EDITOR_SCROLL_AREA which *should* hold an EDITOR_FIXED.
			end
			ef.wipe_out
			controls.wipe_out
			from schema.start
			until schema.exhausted
			loop
				f := schema.field
				c := f.as_widget
					-- add actions to each control as it is created
				c.valid_change_actions.extend (agent on_control_changed (c))
				c.select_actions.extend (agent on_control_selected (c))
--				c.display.pointer_button_press_actions.force_extend (agent on_control_selected (c))
--				c.label.pointer_button_press_actions.force_extend (agent on_control_selected (c))
					-- put the control into the fixed
				ef.extend (c)
					-- place the control in the correct spot
				scrollable_area.position_control (c)
					-- keep track of the control
				controls.extend (c)
				if is_user_disabled then
					disable_controls
				end
				schema.forth
			end
			if not is_user_disabled then
				enable_controls
			end
			time_drawn := schema.time_modified.twin
		end

	save_record
			-- Get the data from each control and put it into the
			-- the record if the data is valid.
		require
			record_exitsts: record /= Void
		local
			dat: ANY	-- for testing
			con: CONTROL	-- control
			c: EDIT_COMMAND
		do
			from changed_controls.start
			until changed_controls.off
			loop
				con := changed_controls.item
				if con.is_display_valid and then not con.field.is_calculated then
					dat := con.value
					c := new_edit_command
--					c.set_record (record)
					c.set_value (con.value)
					c.set_field (con.field)
						-- Disable drawing to keep this control from being updated
						-- when the EDIT_COMMAND is executed.
					disable_drawing
					command_manager.add_command (c)
					enable_drawing
--					record.extend_value (con.value, con.field.label)
				end
				changed_controls.forth
			end
			save_actions.call ([])
			changed_controls.wipe_out
		end

	new_edit_command: EDIT_COMMAND
			-- Used by `save_record' to get an EDIT_COMMAND
		require
			record_exists: record /= Void
		do
			create Result
			Result.set_record (record)
		end

	fill_controls
			-- Put the data from the record into the corresponding control
		require
			record_exists: record /= Void
		local
			key: STRING
			dat: ANY	--COMPARABLE		-- data
			con: CONTROL	-- control
		do
			from
				controls.start
			until
				controls.exhausted
			loop
				key := controls.item.field.label
				con := controls.item
				dat := record.value (key)
				con.set_data (dat)
				con.refresh
				controls.forth
			end
		end

feature {CONTROL} -- implementation

	on_drop_editable (a_editable: EDITABLE)
			-- React to drop of `a_editable' on Current except if
			-- it is a FIELD.  (FIELDs are handled by FIELD_EDITOR_VIEW.)
-- Fix me !!!  This makes the interface inconsistent by allowing a drop
--				of a type that is really not allowed.
		require
			editable_exists: a_editable /= Void
		do
				-- The `parent_tool' (a EDIT_TOOL) takes care of the special
				-- case, when `a_editable' is a FIELD.
			parent_tool.set_target (a_editable)
		end

	on_control_changed (a_control: CONTROL)
			-- A change has been made to value in `a_control'
		require
			control_exists: a_control /= Void
		do
			changed_controls.extend (a_control)
			if is_save_on_change then
				save_record
				disable_drawing
				draw_views (record)
				enable_drawing
			end
		end

	on_control_selected (a_control: CONTROL)
			-- React to `a_control' being selected.
		require
			control_exists: a_control /= Void
		do
			control_select_actions.call ([a_control])
		end

feature {NONE} -- Implementation

	controls: LINKED_SET [CONTROL]
			-- All the controls in the pages.

	changed_controls: LINKED_SET [CONTROL]
			-- Controls whose data has changed since last save

	is_rebuild_needed: BOOLEAN
			-- Used internally to let `draw' know that the controls
			-- need to be rebuilt, because the schema has changed
			-- since the last call to `draw'.  This prevents calling
			-- `build_controls' (and unnecessarily recreating the
			-- controls again) on every call to `draw'.
		require
			schema_exists: schema /= Void
		do
			Result := time_drawn = Void or else
				(time_drawn /= Void and then time_drawn < schema.time_modified)
		end

	time_drawn: YMDHMS_TIME
			-- The time the controls were last drawn.
			-- This is used to allow the controls for a schema to be
			-- redrawn only after the `schema' has changed, not every
			-- time a new `record' is set.
			-- Updated by `build_controls'.

feature {NONE} -- Inaplicable

--	data: ANY
--			-- Not to be used
--			
--	set_data (a_data: like data) is
--			-- Not to be used
--		do
--			check
--				False
--			end
--		end

end -- class DIALOG_EDITOR_VIEW
