note
	description: "[
				A VIEW to be used to display the names of files as filtered.
				]"
	author: "Jimmy J. Johnson"
	date: "26 Jan 04"

class
	FILENAME_VIEW

inherit

	EV_VERTICAL_BOX
		rename
			object_id as ise_object_id
		redefine
			initialize,
			is_in_default_state,
			destroy,
			is_destroyed
		end

	VIEW
		rename
			target as table,
			set_target as set_table
		undefine
			copy,
			is_equal
		redefine
			initialize,
			initialize_interface,
			table,
			set_table,
			draw,
			destroy,
			is_destroyed
		end

create
	default_create

feature {NONE} -- Initialization

	initialize
			-- Create an editor view.
		do
			create browse_button
			create directory_text
			create file_list
			Precursor {EV_VERTICAL_BOX}
			Precursor {VIEW}
			extend (browse_button)
			extend (directory_text)
			extend (file_list)
			disable_item_expand (browse_button)
			disable_item_expand (directory_text)
			interface_table.register_widget (browse_button, "{FILENAME_VIEW}.browse_button")
			browse_button.set_text (interface_table.interface_item ("{FILENAME_VIEW}.browse_button").short_name)
			set_actions
		end

	initialize_interface
			-- Create INTERFACE_ITEMs which could be used by Current.
			-- Called by `initialize'.
		do
				-- Only add an interface item for Current if its `interface_name' has
				-- not already been added; the other items are specific for this tool
				-- and most likely will not be redefine, so no need to check inclusion
				-- of these in the `interface_table'.
				-- In redefinitions of this feature call this at the end of the redefinition
				-- so an item for the `interface_name' can be added to the `interface_table'.
			if not interface_table.has_key (generating_type) then
				interface_table.add_item_with_tuple (<< generating_type, "{FILENAME_VIEW}", "This is a FILENAME_VIEW", "This FILENAME_VIEW is for displaying a list of pixmap file names.", "icon_folders_color.ico">>)
			end
			interface_table.add_item_with_tuple (<<"{FILENAME_VIEW}.browse_button", "Browse", "Press to select a directory", "Press this button to open a dialog for selecting the directory to find pixmap files.", "icon_folders_color.ico">>)
				-- Call Precursor after current is set up.  Specifically after the interface
				-- item for Current's `interface_name' is added.  (see above)
			Precursor {VIEW}
		end

	set_actions
			-- Add actions to the widgets and view.
		do
			browse_button.select_actions.extend (agent on_browse_button_pressed)
			directory_text.change_actions.extend (agent on_directory_text_changed)
		end

feature {NONE} -- Contract support

	is_in_default_state: BOOLEAN
			-- Is `Current' in its default state?
		do
			Result := True
		end

feature -- Access

	table: INTERFACE_TABLE
			-- Object in this view.
			-- Redefined `object' (from VIEW) to change type.

feature -- Element change

	set_table (a_table: INTERFACE_TABLE)
			-- Change `table'
		do
			Precursor {VIEW} (a_table)
			if not is_view_empty then
				directory_text.set_text ("FILENAME_VIEW.set_table")
			end
		end

feature {NONE} -- Actions

	on_browse_button_pressed
			-- React to a press of the browse button
		do
			directory_dialog.show_modal_to_window (parent_window)
		end

	on_directory_selected
			-- React to a directory selection from the `directory_dialog'.
		do
			directory_text.set_text (directory_dialog.directory)
		end

	on_directory_text_changed
			-- React to a change of the `directory_text'.
		do
			draw
		end

feature -- Basic operations

	draw
			-- Redraw the view
		local
			d: DIRECTORY
			lin: ARRAYED_LIST [STRING]
			s, ext: STRING
			i: EV_LIST_ITEM
			p: EV_PIXMAP
		do
			file_list.wipe_out
			create d.make_open_read (directory_text.text)
			if d.exists then
				lin := d.linear_representation
				from lin.start
				until lin.exhausted
				loop
					s := lin.item
					ext := s.twin
					ext.keep_tail (3)
					if equal (ext, "ico") or equal (ext, "png") then
						create p
						p.set_with_named_file (directory_dialog.directory + "\" + s)
						create i.make_with_text (s)
						i.set_pixmap (p)
						i.set_pebble (directory_dialog.directory + "\" + s)
						file_list.extend (i)
					end
					lin.forth
				end
			end
			d.close
		end

	destroy
			-- Destroy underlying native toolkit object.
			-- Render `Current' unusable.
		do
			Precursor {VIEW}
			Precursor {EV_VERTICAL_BOX}
		end

feature -- Status report

	is_destroyed: BOOLEAN
			-- Is `Current' no longer usable?
		do
			Result := Precursor {VIEW} and Precursor {EV_VERTICAL_BOX}
		end

feature {NONE} -- Implementation

	browse_button: EV_BUTTON
			-- To open a directory dialog

	directory_text: EV_TEXT
			-- To display the directory (or path).

	file_list: EV_LIST
			-- To display the filenames of the files

	directory_dialog: EV_DIRECTORY_DIALOG
			-- The standard direcory selection dialog.
		once
			create Result
			Result.set_start_directory ("d:/eiffel54/studio/bitmaps/ico")
			Result.ok_actions.extend (agent on_directory_selected)
		end

end -- class FILENAME_VIEW
