note
	description: "VIEW for editting the strings in an INTERFACE_ITEM."
	author: "Jimmy J. Johnson"
	date: "28 Jan 04"

class
	INTERFACE_ITEM_VIEW

inherit

	VIEW
		rename
			target as interface_item,
			set_target as set_interface_item
		undefine
			copy,
			is_equal
		redefine
			initialize,
			initialize_interface,
			interface_item,
			draw,
			destroy,
			is_destroyed
		end

	EV_FIXED
		rename
			object_id as ise_object_id
		redefine
			initialize,
			is_in_default_state,
			destroy,
			is_destroyed
		end

create
	default_create

feature {NONE} -- Initialization

	initialize
			-- Set up the window
		do
			Precursor {EV_FIXED}
			Precursor {VIEW}
			create short_name_label
			create short_name_text
			create tooltip_label
			create tooltip_text
			create help_text_label
			create help_text_text
			create pixmap_file_label
			create pixmap_file_text
			create pixmap_label
			create pixmap
			interface_table.register_widget (short_name_label, "{INTERFACE_ITEM_VIEW}.short_name_label")
			interface_table.register_widget (tooltip_label, "{INTERFACE_ITEM_VIEW}.tooltip_label")
			interface_table.register_widget (help_text_label, "{INTERFACE_ITEM_VIEW}.help_text_label")
			interface_table.register_widget (pixmap_file_label, "{INTERFACE_ITEM_VIEW}.pixmap_file_label")
			interface_table.register_widget (pixmap_label, "{INTERFACE_ITEM_VIEW}.pixmap_label")
			extend (short_name_label)
			extend (short_name_text)
			extend (tooltip_label)
			extend (tooltip_text)
			extend (help_text_label)
			extend (help_text_text)
			extend (pixmap_file_label)
			extend (pixmap_file_text)
			extend (pixmap_label)
			build_widgets
		end

	initialize_interface
			-- Create INTERFACE_ITEMs which could be used by Current.
			-- Called by `initialize'.
		do
				-- Only add an interface item for Current if its `interface_name' has
				-- not already been added; the other items are specific for this tool
				-- and most likely will not be redefine, so no need to check inclusion
				-- of these in the `interface_table'.
				-- In redefinitions of this feature call this at the end of the redefinition
				-- so an item for the `interface_name' can be added to the `interface_table'.
			if not interface_table.has_key (generating_type) then
				interface_table.add_item_with_tuple (<< generating_type, "{INTERFACE_ITEM_VIEW}", "This is an INTERFACE_ITEM_VIEW", "This view is for editting INTERFACE_ITEMs.", "icon_dummy.ico">>)
			end
			interface_table.add_item_with_tuple (<<"{INTERFACE_ITEM_VIEW}.short_name_label", "Menu Item", "Change short_name", "Change the menu item string for this interface item.", "icon_dummy.ico">>)
			interface_table.add_item_with_tuple (<<"{INTERFACE_ITEM_VIEW}.tooltip_label", "Tooltip", "Change tooltip", "Change the tooltip string for this interface item.", "icon_dummy.ico">>)
			interface_table.add_item_with_tuple (<<"{INTERFACE_ITEM_VIEW}.help_text_label", "Help Text", "Change help_text", "Change the help_text string for this interface item.", "icon_dummy.ico">>)
			interface_table.add_item_with_tuple (<<"{INTERFACE_ITEM_VIEW}.pixmap_file_label", "Pixmap File", "The pixmap file name", "The name of the file from which to make a for this interface item.", "icon_dummy.ico">>)
			interface_table.add_item_with_tuple (<<"{INTERFACE_ITEM_VIEW}.pixmap_label", "Pixmap", "The pixmap", "This is how the pixmap will appear.", "icon_.ico">>)
				-- Call Precursor after current is set up.  Specifically after the interface
				-- item for Current's `generating_type' is added.  (see above)
			Precursor {VIEW}
		end

	is_in_default_state: BOOLEAN
			-- Is the window in its default state?
			-- Redefined to ignore this check.
			-- Left as routine so it can be redefined.
		do
			Result := True
		end

	build_widgets
			-- Position the widgets and set the texts.
		do
				-- Set the widgets' text
			short_name_label.set_text (interface_table.interface_item ("{INTERFACE_ITEM_VIEW}.short_name_label").short_name)
			tooltip_label.set_text (interface_table.interface_item ("{INTERFACE_ITEM_VIEW}.tooltip_label").short_name)
			help_text_label.set_text (interface_table.interface_item ("{INTERFACE_ITEM_VIEW}.help_text_label").short_name)
			pixmap_file_label.set_text (interface_table.interface_item ("{INTERFACE_ITEM_VIEW}.pixmap_file_label").short_name)
			pixmap_label.set_text (interface_table.interface_item ("{INTERFACE_ITEM_VIEW}.pixmap_label").short_name)
				-- Set the position of the widgets
			set_item_position (short_name_label, Spacing, Spacing)
			set_item_position (short_name_text, short_name_label.x_position + short_name_label.width + Spacing,
								short_name_label.y_position)
			set_item_width (short_name_text, 150)
			set_item_position (tooltip_label, Spacing,
								short_name_text.y_position + short_name_text.height + Spacing)
			set_item_position (tooltip_text, tooltip_label.x_position + tooltip_label.width + Spacing,
								tooltip_label.y_position)
			set_item_width (tooltip_text, 150)
			set_item_position (help_text_label, Spacing,
								tooltip_text.y_position + tooltip_text.height + Spacing)
			set_item_width (help_text_text, 200)
			set_item_height (help_text_text, 60)
			set_item_position (help_text_text, help_text_label.x_position + help_text_label.width + Spacing,
								help_text_label.y_position)
			set_item_position (pixmap_file_label, Spacing,
								help_text_text.y_position + help_text_text.height + Spacing)
			set_item_position (pixmap_file_text, pixmap_file_label.x_position + pixmap_file_label.width + Spacing,
								pixmap_file_label.y_position)
			set_item_width (pixmap_file_text, 250)
			set_item_position (pixmap_label, Spacing,
								pixmap_file_text.y_position + pixmap_file_text.height + Spacing)
		end

feature -- Access

	interface_item: INTERFACE_ITEM
			-- The object this view will display and edit.
			-- Redefined to change type.

feature -- Basic operations

	draw
			-- Redraw the view
		do
			if not is_view_empty then
				short_name_text.set_text (interface_item.short_name)
				tooltip_text.set_text (interface_item.tooltip)
				help_text_text.set_text (interface_item.help_text)
				pixmap_file_text.set_text (interface_item.pixmap_file)
			end
		end

	destroy
			-- Destroy underlying native toolkit object.
			-- Render `Current' unusable.
		do
			Precursor {VIEW}
			Precursor {EV_FIXED}
		end

feature -- Status report

	is_destroyed: BOOLEAN
			-- Is `Current' no longer usable?
		do
			Result := Precursor {VIEW} and Precursor {EV_FIXED}
		end

feature {INTERFACE_TABLE_TOOL} -- Implementation

	short_name_label: EV_LABEL
			-- Label paired with `short_name_text'.

	short_name_text: EV_TEXT_FIELD
			-- To edit the `short_name' of `interface_item'.

	tooltip_label: EV_LABEL
			-- Label paired with `tool_tip_text'.

	tooltip_text: EV_TEXT_FIELD
			-- To edit the `tooltip' of `interface_item'..

	help_text_label: EV_LABEL
			-- Label paired with `help_text_text'.

	help_text_text: EV_TEXT
			-- To edit the `help_text' of `interface_item'.

	pixmap_file_label: EV_LABEL
			-- Label paired with `pixmap_file_text'.

	pixmap_file_text: EV_TEXT_FIELD
			-- To edit the `pixmap_file' from `interface_item'.

	pixmap_label: EV_LABEL
			-- Label paired with `pixmap'

	pixmap: EV_PIXMAP
			-- To display the pixmap as created from the file in `pixmap_file_text'.

feature {NONE} -- Implementation (constants)

	Spacing: INTEGER = 20

end -- class INTERFACE_ITEM_VIEW
