note
	description	: "[
				This class ties together the concept of EV_FIGURE_WORLD, EV_DRAWING_AREA,
				and EV_DRAWING_AREA_PROJECTOR in a simpler interface.  
				In order to fake out Eiffel Vision to allow a widget to be decreased in
				size (I call this the `set_minimum_size' problem'), the drawing
				area is contained in an EV_FIXED allowing the drawing area's size
				to follow that of the fixed, and thereby, indirectly the size of the
				parent window.  
				
				While this worked for the editor classes it does not seem to work here.
				I don't know why.  Is the problem with `resize_fixed' or with the fact
				that, in the graphics program where it is used, I am also using the
				"jj_vision" classes.  Perhaps the problem is there.???
				]"
	author		: "Jimmy J. Johnson"
	date		: "27 Sep 03"

class
	JJ_DRAWING_AREA_VIEW

inherit

	EV_FRAME
		redefine
			initialize,
			destroy,
			is_destroyed
		end

	VIEW
		undefine
--			default_create,
			copy
		redefine
			initialize,
			draw,
			destroy,
			is_destroyed
		end

create
	default_create

feature {NONE} -- Initialization

	initialize
			-- Set up the widget
		do
			Precursor {VIEW}
			Precursor {EV_FRAME}
			create fixed
			create world
			create drawing_area
			create projector.make (world, drawing_area)
			extend (drawing_area)
			set_actions
			draw
		end

	set_actions
		do
			resize_actions.force_extend (agent draw)
		end

feature -- Access

	world: EV_FIGURE_WORLD
			-- World which Current will manipulate.

	drawing_area: EV_DRAWING_AREA
			-- Area on which `world' will be projected.
			-- It is contained within a `fixed' area of Current.
			-- Exported to allow direct drawing on it.

feature -- Element change

	set_world (a_world: EV_FIGURE_WORLD)
			-- Change the `world' of figures to be displayed
		require
			world_exists: a_world /= Void
		do
			world := a_world
			projector.set_world (a_world)
		end

feature -- Basic operations

	destroy
			-- Destroy the underlying screen object.
		do
			Precursor {EV_FRAME}
			Precursor {VIEW}
		end

feature -- Status report

	is_destroyed: BOOLEAN
			-- Is the underlying implementation object destroyed?
		do
			Result := Precursor {EV_FRAME} and then Precursor {VIEW}
		end

feature {NONE} -- Drawing / Refresh operations

	draw
			-- Build the view
		do
			drawing_area.clear
			if projector /= Void then
				projector.project
			end
		end

feature {NONE} -- Implementation

	projector: EV_DRAWING_AREA_PROJECTOR
			-- For projecting the world onto the drawing area.

	fixed: EV_FIXED
			-- Container placed in the EV_FRAME to control size of `drawing_area'.

end
