note
	description: "[
			Used as common ancestor to all "windows" in a system built using the 
			"standard interface" cluster.  It provides a way through feature `draw_views'
			for updating all the views which contain the argument to that feature. 
			Objects which this view will be	responsible for displaying are contained in
			feature `target_set'.  The view can force a redraw of all other views
			containing a certain temp_object (preferably an temp_object which it contains) by using
			feature `draw_views' and passing the changed temp_object.  Alternatively, the view
			can force the redraw of views containing any of several objects by passing a set
			of changed objects to feature `draw_views_with_set'.  This class undefines 
			features from ANY with the intent that this class will be ancestor along with
			some effected EV_WIDGET which will effect those features.
			NOTE:  LINKED_SET was used as a client for feature `target_set'; using
			inheritance causes too many name clashes with EV_WIDGET.
			NOTE:  Views which are `is_destroyed' are removed from the global `views' set
			in feature `draw_views'.  This elliminated the need to force descendents to
			redefine `destroy' and `is_destroyed'.  It is simpler this way.
			]"
	instructions: "[
			 Must redefine `draw'.
--			 When redefining `initialize' call precursor of VIEW after the precursor for
--			 the EV_WIDGET because of pre and post-conditions involving `is_initialized'.
			]"
	author		: "Jimmy J. Johnson"
	date		: "18 Jul 03"

deferred class
	VIEW

inherit

	REFACTORING_HELPER
		undefine
			default_create
		end

	SHARED
		undefine
			default_create
		end

		-- NOTE:  Removed EV_WIDGET from the inheritance clause because of the assertions in both
		-- the pre and post-conditions involving "is_initialized".  Precondition is violated if
		-- Precorsor {some EV_WIDGET} is called before Precursor {VIEW}.  ('initialize' can only
		-- be called once for an EV_WIDGET.)  If Precursor {VIEW} is called first the post-condition
		-- is violated becuase the EV_WIDGET is never initialized.
		-- However, doing it this way weakens the precondition "is_initialized" because the precondition
		-- to VIEW is always True (there is not an explicit require clause in `initialize' from VIEW).
--	EV_WIDGET
--		rename
----			default_create as widget_default_create,	-- frozen feature (not undefinable) not to be used
----			implementation as widget_implementation		-- attribute (not undefinable) not to be used
--		undefine
----			initialize,
--			is_in_default_state,
--			is_equal,
--			copy
--		redefine
--			destroy,
--			is_destroyed
--		end

feature {NONE} -- Initialization

	initialize
--	make
			-- Initialize the view and insert it into a global list of views.
			-- `Initialize_interface_items' used by Current into `interface_table'
			-- from SHARED and `register_widget' with the `interface_table'.
		local
			a: EV_ANY
		do
			create split_manager
			create target_set.make
			views.extend (Current)
			initialize_interface
				-- If current is an Eiffel_Vission temp_object then register it.
			a ?= Current
			if a /= Void then
				interface_table.register_widget (a, generating_type)
			end
		end

	initialize_interface
			-- Create INTERFACE_ITEMs which could be used by Current.
			-- Called by `initialize'.
			-- If redefined call this feature after setting up the redefined object.
		do
				-- Only add an interface item for Current if its `interface_name' has
				-- not already been added; the other items are specific for this tool
				-- and most likely will not be redefine, so no need to check inclusion
				-- of these in the `interface_table'.
				-- In redefinitions of this feature call this at the end of the redefinition
				-- so an item for the `interface_name' can be added to the `interface_table'.
			if not interface_table.has_key (generating_type) then
				interface_table.add_item_with_tuple (<< generating_type, "VIEW","A VIEW temp_object.", "This temp_object is of type VIEW.", "icon_shell_color.ico">>)
			end
		end

feature -- Access

--	application: JJ_APPLICATION
--			-- Convience feature for obtaining the current application.
--		local
--			app: JJ_APPLICATION
--		once
--			app ?= (create {EV_ENVIRONMENT}).application
--			check
--				jj_application_exists: app /= Void
--					-- Because VIEWs are used in JJ_APPLICATIONs.
--			end
--		ensure
--			result_exists: Result /= Void
--		end

--	command_manager: COMMAND_MANAGER
--			-- Manages the COMMAND's called by the system to allow undo/redo capabilities.
--			-- (This is a handle to the `command_manager' from a JJ_APPLICATION; putting it
--			-- here instead of in SHARED allows redefinition of the `command_manager' in
--			-- descendents of JJ_APPLICATION.
--		once
--			Result := application.command_manager
--		end

	split_manager: SPLIT_MANAGER
			-- Manages the placement of sub-views withing current.

	target: ANY
			-- The primary target in this view.  This feature along with
			-- `set_target' allows the view to handle one target specially.

	target_item: ANY
			-- Current target in the set.
		require
			readable: not is_view_off
		do
			Result := target_set.item
		ensure
			valid_result: Result /= Void
		end

	objects: LINEAR [ANY]
			-- A copy of the list of targets stored in this view.
		do
			target_set.start
			Result := target_set.duplicate (target_set.count)
		end

	parent_window: JJ_MAIN_WINDOW
			-- The JJ_MAIN_WINDOW if any which contains this view.
			-- Must redefine to change type.
		local
			c: EV_CONTAINABLE
		do
				-- The assignment attempt became necessary when EV_WIDGET
				-- was removed from inheritance clause.
			c ?= Current
			check
				current_is_a_containable: c /= Void
			end
			Result := recursive_parent_window (c)
		ensure
			valid_result: Result /= Void
		end

	parent_tool: TOOL
			-- The TOOL, if any, in which Current resides.
		local
			p: EV_CONTAINER
		do
			p := parent
			from Result ?= p
			until Result /= Void
			loop
				p := p.parent
				Result ?= p
			end
		end

	parent: EV_CONTAINER
			-- Parent of the current view.
			-- To be effected by joining with an EV_ class
		deferred
		end

	state: VIEW_STATE
			-- A snapshot of the current look of the view
		do
			create Result.make (Current)
		end

feature -- Element change

	set_split_manager (a_manager: SPLIT_MANAGER)
			-- Change `split_manager'
		require
			manager_exists: a_manager /= Void
		do
			split_manager := a_manager
		ensure
			manager_assigned: split_manager = a_manager
		end

	set_target (a_target: like target)
			-- Change the value of `target' and add it to the `target_set' (the set
			-- of objects contained in this view.  The old target is removed from
			-- the set.
			-- This feature can be used as a pattern if a descendant wants to give
			-- special treatment to a single target.
--		require
--			target_exists: a_target /= Void
		do
			if target /= Void then
				target_set.prune (target)
			end
			if a_target /= Void then
				target_set.extend (a_target)
			end
			target := a_target
		ensure
			target_assigned: target = a_target
			contains_target: a_target /= Void implies target_set.has (a_target)
			old_target_removed: (old target /= Void) and (old target /= target) implies not target_set.has (old target)
		end

	add_target (a_target: like target_item)
			-- Add `a_target' to the list of items in this view.  Use this feature
			-- for views which require more objects than simply a `target' when
			-- the other objects do not need special handling.  (See `set_target'
			-- for pattern usage if the other objects need special handling.)
		require
			target_exists: a_target /= Void
		do
			target_set.extend (a_target)
		ensure
			has_target: target_set.has (a_target)
		end

	prune_target (a_target: like target_item)
			-- Remove `a_object' from the view.
			-- Make `target' Void if `a_target' was the `target'.
		require
			target_exists: a_target /= Void
		do
			target_set.prune (a_target)
			if target = a_target then
				target := Void
			end
		ensure
			target_removed: not has_target (a_target)
			target_voided_if_same: a_target = old target implies target = Void
			target_unchanded_if_not_same: a_target /= old target implies target = old target
		end

	remove_all_targets
			-- Remove all the objects from this view.
		do
			target_set.wipe_out
			target := Void
		ensure
			has_no_targets: target_set.is_empty
			target_voided: target = Void
		end

feature -- Basic operations

	draw
			-- Draw the current view.
			-- Default does nothing
		require
			view_is_drawable: not is_draw_disabled
		do
		end

	draw_views (a_target: ANY)
			-- Draw the views which contain `a_target'.
			-- This also cleans any "destroyed" views from the `views' set.
		require
			target_exists: a_target /= Void
		local
			marks: LINKED_SET [VIEW]	-- Views marked for removal.
			v: VIEW
		do
			create marks.make
			from views.start
			until views.exhausted
			loop
				v := views.item
				if v.is_destroyed then
					marks.extend (v)
				elseif not v.is_draw_disabled and then
						v.has_target (a_target) then
					v.draw
				end
				views.forth
			end
			from marks.start
			until marks.exhausted
			loop
				views.prune (marks.item)
				marks.forth
			end
		end

	draw_views_with_set (a_set: LINEAR [ANY])
			-- Draw the views which contain any of the objects in `a_set'.
			-- This also cleans any "destroyed" views from the `views' set.
		require
			set_exists: a_set /= Void
		local
			marks: LINKED_SET [VIEW]	-- Views marked for removal.
			v: VIEW
		do
			create marks.make
			from views.start
			until views.exhausted
			loop
				v := views.item
				if v.is_destroyed then
					marks.extend (v)
				elseif not v.is_draw_disabled then
					from a_set.start
					until a_set.exhausted
					loop
						if v.has_target (a_set.item) then
							v.draw
						end
						a_set.forth
					end
				end
				views.forth
			end
			from marks.start
			until marks.exhausted
			loop
				views.prune (marks.item)
				marks.forth
			end
		end

	draw_all_views
			-- Draw *all* the views in the system.
			-- Also cleans any "destroyed" views for the `views' set.
		local
			marks: LINKED_SET [VIEW]	-- Views marked for removal.
			v: VIEW
		do
			create marks.make
			from views.start
			until views.exhausted
			loop
				v := views.item
				if v.is_destroyed then
					marks.extend (v)
				elseif not v.is_draw_disabled then
					v.draw
				end
				views.forth
			end
			from marks.start
			until marks.exhausted
			loop
				views.prune (marks.item)
				marks.forth
			end
		end

	destroy
			-- To be joined with an EV_WIDGET in descendants and called as
			-- precursor in order to remove Current from the `interface_table'.
		local
			a: EV_ANY
		do
			a ?= Current
			if a /= Void then
				interface_table.unregister_widget (a)
			end
		end

feature -- Status report

	is_destroyed: BOOLEAN
			-- Is `Current' no longer usable?
		local
			a: EV_ANY
		do
			a ?= Current
			if a /= Void then
--				Result := not interface_table.has_widget (Current)
				Result := not interface_table.has_widget (a)
			else
				Result := True
			end
		end

	is_view_empty: BOOLEAN
			-- Are there no objects in this VIEW?
		do
			Result := target_set.is_empty
		ensure
			valid_result: Result implies target_set.is_empty
		end

	is_view_off: BOOLEAN
		do
			Result := target_set.off
		end

	is_view_before: BOOLEAN
		do
			Result := target_set.before
		end

	is_view_after: BOOLEAN
		do
			Result := target_set.after
		end

	is_view_exhausted: BOOLEAN
		do
			Result := target_set.exhausted
		end

	is_draw_disabled: BOOLEAN
			-- Can the view be drawn using 'draw'?

feature -- Status setting

	disable_drawing
			-- Block the view from being redrawn by calling draw.
			-- Used to reduce number of calls to draw.
		do
			is_draw_disabled := True
		ensure
			drawing_not_allowed: is_draw_disabled
		end

	enable_drawing
			-- Allow the view to be redrawn on a call to draw.
		do
			is_draw_disabled := False
		ensure
			drawing_allowed: not is_draw_disabled
		end

feature -- Querry

	has_target (a_target: like target_item): BOOLEAN
		do
			Result := target_set.has (a_target)
		end

	display_name (a_object: ANY): STRING
			-- An identifying "out" value corresponding to `a_object'.
		require
			object_exists: a_object /= Void
		local
			e: EDITABLE
		do
			e ?= a_object
			if e /= Void then
				Result := e.display_name
			else
				Result := "fix me"
				fixme ("VIEW.display_name -- get from once table?")
			end
		ensure
			result_exists: Result /= Void
		end

	yes_cursor (a_target: ANY): EV_CURSOR
			-- Cursor for `a_target'.
		do
--			Result.make_with_pixmap (
			create Result
		end

	no_cursor (a_target: ANY): EV_CURSOR
			-- Cursor for `a_target'.
		do
			create Result
		end

feature -- Cursor movement

	targets_start
		do
			target_set.start
		end

	targets_forth
		require
			not_after: not is_view_after
		do
			target_set.forth
		end

	targets_back
		require
			not_before: not is_view_before
		do
			target_set.back
		end

feature {NONE} -- Implementation

	recursive_parent_window (a_containable: EV_CONTAINABLE): JJ_MAIN_WINDOW
			-- The JJ_MAIN_WINDOW, if any, which contains this tool.
			-- This procedure was provided by Julian Rodgers from the Eiffel
			-- users group.
		local
			a: ANY
			w: EV_WINDOW
			cur_parent: EV_CONTAINER 		-- contains WIDGETS
			con_parent: EV_CONTAINABLE		-- temp_object that may be parented
		do
			if a_containable /= Void then
				a ?= a_containable.parent
				cur_parent ?= a_containable.parent	-- `parent' returns an EV_CONTAINER
				check
					is_container: cur_parent /= Void	-- because parent of EV_CONTAINABLE must be an EV_CONTAINER
				end
				w ?= cur_parent						-- Is `cur_parent' an EV_WINDOW?
				if w /= Void then					-- If yes
					Result ?= w						-- then found the window
				else	--if cur_parent /= Void		-- It is not an EV_WINDOW...
					con_parent ?= cur_parent		-- so, is `cur_parent' an EV_CONTAINABLE?
					check
						parent_is_containable: con_parent /= Void	-- because if `con_parent' is not an EV_WINDOW
					end												-- it must be a EV_CONTAINABLE
					Result := recursive_parent_window (con_parent)	-- so, find its parent, recursively.	
				end
			end
		ensure
			result_exists: Result /= Void
		end

feature {NONE} -- Implemetation

	target_set: LINKED_SET [ANY]
			-- Objects which this view is to display.

	views: LINKED_SET [VIEW]
			-- Global set to hold all the VIEWs in a system.
		once
			create Result.make
		end

invariant

	split_manager_exists: split_manager /= Void
	target_set_exists: target_set /= Void
	contains_target: target /= Void implies target_set.has (target)

end -- class VIEW

