note
	description: "[
				Item for placement into HISTORY_DROPDOWNs, pairing a `editable'
				with a `time_stamp'.
				]"
	author: "Jimmy J. Johnson"
	date: "23 Aug 04"

class
	HISTORY_ITEM

inherit

	COMPARABLE
		undefine
			default_create,
			copy
		end

	VIEW
		rename
			parent as view_parent
		undefine
--			default_create,
			is_equal,
			copy
		redefine
			initialize,
			set_target,
			draw,
			destroy,
			is_destroyed
		end

	EV_LIST_ITEM
		rename
			object_id as ise_object_id
		undefine
			is_equal
		redefine
			initialize,
			destroy,
			is_destroyed
		end

create
	make

feature {NONE} -- Initialization

	make (a_target: like target)
			-- Create object and initialize Current.
		require
			editable_exists: a_target /= Void
		do
			default_create
			create time_stamp
			set_target (a_target)
--			add_object (target.target_label_field)
			draw
		end

	initialize
			-- Setup current
		do
			Precursor {EV_LIST_ITEM}
			Precursor {VIEW}
		end

feature -- Access

	time_stamp: YMDHMS_TIME
			-- To keep track of objects creation time

feature -- Element change

	set_target (a_target: like target)
			-- change `target'
		do
			Precursor {VIEW} (a_target)
			reset_time_stamp
			draw
		end

	reset_time_stamp
			-- Make `time_stamp' be the current system time
		do
			time_stamp := create {YMDHMS_TIME}.set_now
		end

feature -- Querry

	is_simular (a_other: like Current): BOOLEAN
			-- Does Current and other contain the same `target'?
		require
			other_exists: a_other /= Void
		do
			Result := target = a_other.target
		ensure
			definition: Result implies target = a_other.target
		end

feature -- Basic operations

	destroy
			-- Destroy underlying native toolkit object.
			-- Render `Current' unusable.
		do
			Precursor {VIEW}
			Precursor {EV_LIST_ITEM}
		end

feature -- Status report

	is_destroyed: BOOLEAN
			-- Is `Current' no longer usable?
		do
			Result := Precursor {VIEW} and Precursor {EV_LIST_ITEM}
		end

feature -- Comparison

	infix "<" (a_other: like Current): BOOLEAN
			-- Is Current less than `a_other'?
		do
			Result := (time_stamp < a_other.time_stamp)
		end

feature {NONE} -- Implementation

	draw
			-- Put the correct text into this EV_LIST_ITEM.
		local
			e: EDITABLE
		do
			e ?= target
			if e /= Void then
				set_text (e.display_name)
			else
				set_text (target.generating_type + time_stamp.out)
			end
		end

	view_parent: EV_FIXED
			-- Not used.
			-- Result type is just so it conforms for redefinition.
		do
			check
				not_to_be_called: False
			end
		end


	invariant

	time_stamp_exists: time_stamp /= Void
	target_exists: target /= Void

end
