note
	description: "[
				A STORABLE structure to hold the interface names to be associated
				with one or more widgets in an INTERFACE_TABLE, allowing a common
				access point for all strings and pixmaps in a program.
				]"
	author: "Jimmy J. Johnson"
	date: "18 Feb 03"

class
	INTERFACE_ITEM

inherit

	JJ_VISION_FILE_FACILITIES
		undefine
			default_create
		redefine
			default_create
		end

create
	default_create

feature {NONE} -- Initialization

	default_create
			-- Create and initialize an instance
		do
--			Precursor {VIEWABLE}
				-- Because INTERFACE_ITEM is an intergral part of the `interface_table'
				-- it cannot call Precursor which tries to register the widget.
				-- Do not want that to happen as this is a simple "record-like" object
				-- not a widget or EV_ANY as needed by `register_widget' from
				-- INTERFACE_TABLE.
				-- The objects will not be "registered" in the table.
			short_name := "TEST in INTERFACE_ITEM default menu item"
			tooltip := "default tooltip"
			help_text := "default help text"
			pixmap_file := "no data"
		end

feature -- Access

	short_name: STRING
			-- for use in menus

	tooltip: STRING
			-- For use as role-over tips

	help_text: STRING
			-- Help context string

	pixmap_file: STRING
			-- Name of the file from which to make `pixmap'.

	pixmap: EV_PIXMAP
			-- A pixmap created from the file named `pixmap_file'.
		local
			f: RAW_FILE
			s: STRING
		do
			s := image_file_name (pixmap_file)
			create f.make (s)
			if f.exists then
				create Result
				Result.set_with_named_file (s)
			else
				Result := (create {EV_STOCK_PIXMAPS}).Error_pixmap
			end
		ensure
			result_exists: Result /= Void
		end

feature -- Element change

	set_short_name (a_string: STRING)
			-- Change `short_name'
		require
			string_exists: a_string /= Void
		do
			short_name := a_string
		ensure
			short_name_was_set: short_name = a_string
		end

	set_tooltip (a_string: STRING)
			-- Change `tooltip'
		require
			string_exists: a_string /= Void
		do
			tooltip := a_string
		ensure
			tooltip_was_set: tooltip = a_string
		end

	set_help_text (a_string: STRING)
			-- Change `help_text'
		require
			string_exists: a_string /= Void
		do
			help_text := a_string
		ensure
			help_text_was_set: help_text = a_string
		end

	set_pixmap_file (a_file_name: STRING)
			-- Change `pixmap'
		require
			file_name_exists: a_file_name /= Void
		do
			pixmap_file := a_file_name
		ensure
			pixmap_file_was_set: pixmap_file = a_file_name
		end

invariant

	short_name_exists: short_name /= Void
	tooltip_exists: tooltip /= Void
	help_text_exists: help_text /= Void
	pixmap_file_exists: pixmap_file /= Void

end
