note
	description: "[
			Contains a table of menu strings, help texts, bitmap file names,
			etc., which will be associated with widgets.  This is a helper class
			describing objects to be contained in an INTERFACE_TABLE.
		]"
	author: "Jimmy J. Johnson"
	date: "27 Sep 04"

class
	INTERFACE_NAMES

inherit

	ANY
		redefine
			default_create
		end

	JJ_VISION_FILE_FACILITIES
		export
			{NONE} all
		undefine
			default_create
		end

feature {NONE} -- Initialization

	default_create
			-- Initialize `Current'.
		do
--			Precursor {PERSISTENT}
--			make_with_datastore (settings_datastore)
--			set_datastore (settings_datastore)
			create table.make (50)
		end

feature -- Access

	table: HASH_TABLE [INTERFACE_ITEM, STRING]
			-- The actual data which is to be/was persisted.
			-- This will re-load the data from the image in the `datastore'
			-- if the currently held data is older than the image.
			-- The only other fields which are updated by this is `item_id',
			-- `access_time', and `touched_time'.
			-- To reload the base object use `load'.
			-- Redefined to change type

	interface_item (a_key: STRING): INTERFACE_ITEM
			-- The INTERFACE_ITEM from the `item' (HASH_TABLE [INTERFACE_ITEM, STRING])
			-- indexed by `a_key'.
		require
			key_exists: a_key /= Void
			has_key: table.has (a_key)
		do
			Result := table.item (a_key)
		ensure
			definition: Result = table.item (a_key)
		end

	item_for_iteration: INTERFACE_ITEM
			-- The INTERFACE_ITEM at current iteration position
		require
			not_off: not off
		do
			Result := table.item_for_iteration
		ensure
			definition: Result = table.item_for_iteration
		end

	key_for_iteration: STRING
			-- Key at current iteration position
		require
			not_off: not off
		do
			Result := table.key_for_iteration
		ensure
			definition: Result = table.key_for_iteration
		end

	short_name (a_key: STRING): STRING
			-- The `short_text' from the INTERFACE_ITEM of the HASH_TABLE, `table',
			-- at index `a_key'.
		require
			key_exists: a_key /= Void
			has_key: has (a_key)
		do
			Result := table.item (a_key).short_name
		ensure
			definition: Result = table.item (a_key).short_name
		end

	tooltip (a_key: STRING): STRING
			-- The `tool_tip' from the INTERFACE_ITEM of the HASH_TABLE, `table',
			-- at index `a_key'.
		require
			key_exists: a_key /= Void
			has_key: has (a_key)
		do
			Result := table.item (a_key).tooltip
		ensure
			definition: Result = table.item (a_key).tooltip
		end

	help_text (a_key: STRING): STRING
			-- The `help_text' from the INTERFACE_ITEM of the HASH_TABLE, `table',
			-- at index `a_key'.
		require
			key_exists: a_key /= Void
			has_key: has (a_key)
		do
			Result := table.item (a_key).help_text
		ensure
			definition: Result = table.item (a_key).short_name
		end

	pixmap_file (a_key: STRING): STRING
			-- The `pixmap_file' from the INTERFACE_ITEM of the HASH_TABLE, `table',
			-- at index `a_key'.
		require
			key_exists: a_key /= Void
			has_key: has (a_key)
		do
			Result := table.item (a_key).pixmap_file
		ensure
			definition: Result = table.item (a_key).pixmap_file
		end

	pixmap (a_key: STRING): EV_PIXMAP
			-- A EV_PIXMAP created from the `pixmap_file' of the INTERFACE_ITEM
			-- of the HASH_TABLE, `item', at index `a_key'.
		require
			key_exists: a_key /= Void
			has_key: has (a_key)
		do
			Result := table.item (a_key).pixmap
		ensure
			result_exists: Result /= Void
		end

feature -- Basic operations

	extend (a_item: INTERFACE_ITEM; a_key: STRING)
			-- Add `a_item' to the `table' (HASH_TABLE [INTERFACE_ITEM, STRING]).
		require
			item_exists: a_item /= Void
			key_exists: a_key /= Void
			not_has_key: not has (a_key)
		do
			table.extend (a_item, a_key)
		ensure
			has_item: table.has (a_key)
		end

	force (a_item: INTERFACE_ITEM; a_key: STRING)
			-- Add `a_item' to the `table' (HASH_TABLE [INTERFACE_ITEM, STRING]).
		require
			item_exists: a_item /= Void
			key_exists: a_key /= Void
			not_has_key: not has (a_key)
		do
			table.force (a_item, a_key)
		ensure
			has_item: table.has (a_key)
		end

feature -- Status report

	off: BOOLEAN
			-- Has the HASH_TABLE cursor `off'?
		do
			Result := table.off
		ensure
			definition: Result = table.off
		end

feature -- Querry

	has (a_key: STRING): BOOLEAN
			-- Does Current contain an item indexed by `a_key'?
		do
			Result := table.has (a_key)
		ensure
			definition: Result implies table.has (a_key)
		end

feature -- Cursor movement

	start
			-- Move to the first interface_item in the HASH_TABLE.
		do
			table.start
		end

	forth
			-- Move the HASH_TABLE cursor forward to the next INTERFACE_ITEM.
		require
			not_off: not off
		do
			table.forth
		end

feature {NONE} -- Implementation

end -- class INTERFACE_NAMES
