note
	description	: "[
				Provides the connection between the INTERFACE_NAMES currently
				in use and the widgets used by the one `interface_table' declared
				in SHARED from the "Standard Interface" cluster.
				Holds strings and pixmap files which will be visible in a program as
				menu texts, help texts, help contexts, pixmaps, etc.  The `names' in 
				this table can be editted by temporarily adding an INTERFACE_TABLE_TOOL
				to your MAIN_WINDOW.
				
				The table used in a program will not be editted; just the
				INTERFACE_NAMES will be changed.
				]"
	author		: "Jimmy J. Johnson"
	date		: "18 Jan 04"

class
	INTERFACE_TABLE

inherit

	ANY
		redefine
			default_create
		end

create
	default_create
--	load_default_names
--	load_from_file

feature {NONE} -- Initialization

	default_create
			-- Initialize the hash table from the `names'
		do
--			Precursor {VIEWABLE}
				-- Could not call precursor for this object because the interface_table
				-- has not been created yet, and precursor is trying to register this
				-- object with the table.
			create widgets.make (50)
			create changed_widgets.make
			create names
			pixmap_size := Default_pixmap_size
--			create dynamic_names
--			dynamic_names.clear_all
--			register_widget (Current, interface_name)
		end

feature -- Initialization

	load_default_names
			-- Initialize the table with the default_strings.
		do
--			default_create
--			make
			set_defaults
		end

feature -- Access

	Default_interface_item: INTERFACE_ITEM
			-- Default interface item
		once
			create Result
		end

	pixmap_size: INTEGER
			-- The size to make the pixmaps

	Minimum_pixmap_size: INTEGER = 8
			-- Smallest size allowed for `pixmap_size'

	Maximum_pixmap_size: INTEGER = 64
			-- Largest size allowed for `pixmap_size'

	Default_pixmap_size: INTEGER = 16
			-- Default size for `pixmap_size'

	short_name (a_key: STRING): STRING
			-- `short_name' from the INTERFACE_ITEM indexed by `a_key' if `a_key' is in
			-- the table; otherwise return `a_key'.
		require
			key_exists: a_key /= Void
		do
			if names.has (a_key) then
				Result := names.short_name (a_key)
			else
				Result := a_key.twin
			end
		ensure
			Result_if_has_key: names.has (a_key) implies Result = names.short_name (a_key)
			Result_if_not_has_key: not names.has (a_key) implies equal (Result, a_key)
		end

	tooltip (a_key: STRING): STRING
			-- `Tooltip' from the INTERFACE_ITEM indexed by `a_key' or the
			-- tooltip from the `Default_interface_item' if `a_key' is not
			-- in the table.
		require
			key_exists: a_key /= Void
		do
			if names.has (a_key) then
				Result := names.tooltip (a_key)
			else
				Result := Default_interface_item.tooltip
			end
		ensure
			result_if_has_key: names.has (a_key) implies Result = names.tooltip (a_key)
			result_if_not_has_key: not names.has (a_key) implies Result = Default_interface_item.tooltip
		end

	help_text (a_key: STRING): STRING
			-- `Help_text' from the INTERFACE_ITEM indexed by `a_key' or the
			-- tooltip from the `Default_interface_item' if `a_key' is not
			-- in the table.
		require
			key_exists: a_key /= Void
		do
			if names.has (a_key) then
				Result := names.help_text (a_key)
			else
				Result := Default_interface_item.help_text
			end
		ensure
			result_if_has_key: names.has (a_key) implies Result = names.help_text (a_key)
			result_if_not_has_key: not names.has (a_key) implies Result = Default_interface_item.help_text
		end

	pixmap_file (a_key: STRING): STRING
			-- The full filename of the item indexed by `a_key'
		require
			key_exists: a_key /= Void
		do
			if names.has (a_key) then
				Result := names.pixmap_file (a_key)
			else
				Result := Default_interface_item.pixmap_file
			end
		ensure
			result_if_has_key: names.has (a_key) implies Result = names.pixmap_file (a_key)
			result_if_not_has_key: not names.has (a_key) implies Result = Default_interface_item.pixmap_file
		end

	pixmap (a_key: STRING): EV_PIXMAP
			-- A PIXMAP created from the INTERFACE_ITEM indexed by `a_key' or
			-- created from the `Default_interface_item' if there is no item
			-- indexed by `a_key'.
		require
			key_exists: a_key /= Void
		do
			create Result
			if names.has (a_key) then
				Result := names.pixmap (a_key)
			else
				Result := Default_interface_item.pixmap
			end
			Result.stretch (pixmap_size, pixmap_size)
		ensure
--			Result_if_has_key: names.has (a_key) implies Result = names.pixmap (a_key)
--			Result_if_not_has_key: not names.has (a_key) implies Result = Default_interface_item.pixmap
		end

	interface_item (a_key: STRING): INTERFACE_ITEM
			-- Access to the `names' item by `a_key'.
		require
			key_exists: a_key /= Void
		do
			if names.has (a_key) then
				Result := names.interface_item (a_key)
--			elseif dynamic_names.has (a_key) then
--				Result := dynamic_names.item (a_key)
			else
				Result := Default_interface_item
			end
		ensure
			result_exists: Result /= Void
--			result_if_has_key: (names.has (a_key) or dynamic_names.has (a_key)) implies Result = names.item (a_key)
--			result_if_not_has_key: (not names.has (a_key) and not dynamic_names.has (a_key)) implies Result = names.Default_interface_item
			result_if_has_key: names.has (a_key) implies Result = names.interface_item (a_key)
			result_if_not_has_key: not names.has (a_key) implies Result = Default_interface_item
		end

feature -- Element change

	set_names (a_names: like names)
			-- Change the `names' to `a_names'.  This gives the table a different
			-- group of interface items (say in another language).
		require
			names_exists: a_names /= Void
		do
			names := a_names
			refresh_interface
		ensure
			names_assigned: names = a_names
		end

	set_button_size (a_size: INTEGER)
			-- Change `pixmap_size'
		require
			size_big_enough: a_size >= Minimum_pixmap_size
			size_small_enough: a_size <= Maximum_pixmap_size
		do
			pixmap_size := a_size
			buttons.do_all (agent changed_widgets.extend)
			if not is_blocked then
				update_interface
			end
		ensure
			pixmap_size_changed: pixmap_size = a_size
		end

	add_interface_item (a_item: INTERFACE_ITEM; a_key: STRING)
			-- Adds `a_item' to the the `dynamic_names' table.
		require
			item_exists: a_item /= Void
			key_exists: a_key /= Void
			not_has_key: not has_key (a_key)
		do
			names.extend (a_item, a_key)
		ensure
			has_item: names.has (a_key)
		end

	add_item_with_tuple (a_array: ARRAY [STRING])	-- a_tuple: TUPLE [STRING, STRING, STRING, STRING]) is
			-- Usage is [a_key, a_short_name, a_tool_tip, a_help_string, pixmap_file]
			-- Only adds if it does not already have it in the table.
		local
			ii: INTERFACE_ITEM
		do
			if not names.has (a_array.item (1)) then
				create ii
				ii.set_short_name (a_array.item (2))
				ii.set_tooltip (a_array.item (3))
				ii.set_help_text (a_array.item (4))
				ii.set_pixmap_file (a_array.item (5))
--				names.extend (ii, a_array.item (1))
				names.force (ii, a_array.item (1))
			end
		end

feature -- Status report

	is_showing_button_text: BOOLEAN
			-- Should text (if available) be displayed on buttons in the table?

	is_blocked: BOOLEAN
			-- Are the INTERFACE_ITEMs in the table prevented from updating
			-- their associated widgets?
		do
			Result := blocked_count /= 0
		ensure
			definition: Result implies blocked_count /= 0
		end

feature -- Status change

	block
			-- Prevent INTERFACE_ITEMs in the table from updating their
			-- associated widgets when changed.  The changed items will
			-- be collected in `changed_items' and updated manually with
			-- a call to `update_interface'.
			-- Resume by calling `resume' the same number of times as `block'
			-- as called.
		do
			blocked_count := blocked_count + 1
		ensure
			automatic_update_blocked: is_blocked = True
		end

	resume
			-- Allow INTERFACE_ITEMs in the table to update their associated
			-- widgets whenever the items change if `blocked_count' = 0.
			-- Also, if Current goes from a blocked state to an unblocked state
			-- (i.e. `blocked_count' is decremented to zero) then `update_interface'.
			-- Current could still be `is_blocked' if there are more calls to
			-- `block' than to `resume'.
		do
			if blocked_count > 0 then
				blocked_count := blocked_count - 1
				if not is_blocked then
					update_interface
				end
			end
		ensure
			count_decremented: old blocked_count > 0 implies blocked_count = old blocked_count - 1
		end

	force_resume
			-- Make updates resume immediately by setting `blocked_count' to 0.
			-- Update any `changed_widgets'.
			-- Use this feature when you are sure the interface is ready for updating
			-- regardless of whether or not other classes have previousely blocked
			-- the updating.
		do
			blocked_count := 0
			update_interface
		ensure
			not_blocked: not is_blocked
		end

	show_button_text
			-- Make text appear on the buttons under the pixmap.
		do
			is_showing_button_text := True
			buttons.do_all (agent changed_widgets.extend)
			if not is_blocked then
				update_interface
			end
		ensure
			is_showing_button_text: is_showing_button_text
		end

	hide_button_text
			-- Make buttons contain only a pixmap.
		do
			is_showing_button_text := False
			buttons.do_all (agent changed_widgets.extend)
			if not is_blocked then
				update_interface
			end
		ensure
			is_not_showing_button_text: not is_showing_button_text
		end

feature -- Querry

	has_key (a_key: STRING): BOOLEAN
			-- Is `a_key' used as a key to the tables?
		require
			key_exists: a_key /= Void
		do
			Result := names.has (a_key)
		end

	has_widget (a_widget: EV_ANY): BOOLEAN
			-- Has `a_widget' been registered with a call to `register_widget'?
		require
			widget_exists: a_widget /= Void
		local
			ls: LINKED_SET [EV_ANY]
			c: CURSOR
		do
			c := widgets.cursor
			from widgets.start
			until Result or else widgets.off
			loop
				ls := widgets.item_for_iteration
				Result := ls.has (a_widget)
				widgets.forth
			end
			widgets.go_to (c)
		end

feature -- Element change

	register_widget (a_widget:EV_ANY; a_key: STRING)
			-- Associate `a_widget' to the INTERFACE_ITEM indexed by `a_key'.
		require
			widget_exists: a_widget /= Void
			key_exists: a_key /= Void
		local
			ls: LINKED_SET [EV_ANY]
		do
				-- If there already a list of widgets associated whith this
				-- key add `a_widget' to that list.
			if has_widget (a_widget) then
				unregister_widget (a_widget)
			end
			if widgets.has (a_key) then
				ls := widgets.item (a_key)
				ls.extend (a_widget)
			else
				create ls.make
				ls.extend (a_widget)
				widgets.extend (ls, a_key)
			end
			changed_widgets.extend (a_widget)
		ensure
			widgets_has_key: widgets.has (a_key)
			widget_associated_to_key: widgets.item (a_key).has (a_widget)
		end

	unregister_widget (a_widget: EV_ANY)
			-- Remove `a_widget' from the `widgets' table.
		require
			widget_exists: a_widget /= Void
		local
			ls: LINKED_SET [EV_ANY]
			done: BOOLEAN
		do
			from widgets.start
			until done or else widgets.off
			loop
				ls := widgets.item_for_iteration
				if ls.has (a_widget) then
					ls.prune (a_widget)
					done := True
				end
				widgets.forth
			end
		ensure
			not_registered: not has_widget (a_widget)
		end

	unregister_widgets (a_key: STRING)
			-- Relieve this table of the responsiblity of maintaining the appearance of
			-- any widgets associated with `a_key'.
			-- Does not change the appearance of the widgets.
		require
			key_exists: a_key /= Void
		do
			widgets.remove (a_key)
--			changed_widgets.prune (a_key)
		ensure
			widgets_removed: not widgets.has (a_key)
		end

feature -- Basic operations

	refresh_interface
			-- Call `update_widgets' for each item
		do
			from widgets.start
			until widgets.off
			loop
				update_widgets (widgets.key_for_iteration)
				widgets.forth
			end
			changed_widgets.wipe_out
		ensure
			no_changed_items: changed_widgets.is_empty
		end

	update_interface
			-- Call `update_widgets' for each item in `changed_widgets'
		do
			if not is_blocked then
				from changed_widgets.start
				until changed_widgets.exhausted
				loop
					update_widget (changed_widgets.item)
					changed_widgets.forth
				end
				changed_widgets.wipe_out
			end
		ensure
			no_changed_items: not is_blocked implies changed_widgets.is_empty
		end

	update_widget (a_widget: EV_ANY)
			-- Update the interface for `a_widget' only by finding it
			-- in the table and applying the associated interface items.
		require
			widget_exists: a_widget /= Void
			table_has_widget: has_widget (a_widget)
		local
			ls: LINKED_SET [EV_ANY]
			ii: INTERFACE_ITEM
			done: BOOLEAN
			ev_a: EV_ANY
		do
			from widgets.start
			until done or else widgets.off
			loop
				ls := widgets.item_for_iteration
				from ls.start
				until done or else ls.exhausted
				loop
					if a_widget = ls.item then
							-- found the widget in a list.  As a side effect, if the
							-- widget `is destroyed' it should no longer be in the list.
							-- (This is the only place to do this check.)
	-- FIX ME!!  This only deletes EV_WIDGETs; other VIEWABLEs which are normal
	-- eiffel objects are deleted from the `interface_table' when they are removed
	-- from all VIEWs.
						ev_a ?= a_widget
						if ev_a /= Void and then ev_a.is_destroyed then
							ls.prune (ev_a)
							done := True
						else
							if names.has (widgets.key_for_iteration) then
								ii := names.interface_item (widgets.key_for_iteration)
--							elseif dynamic_names.has (widgets.key_for_iteration) then
--								ii := dynamic_names.item (widgets.key_for_iteration)
							else
								ii := Default_interface_item
							end
							update_widget_with_item (a_widget, ii)
							done := True
						end
					end
					ls.forth
				end
				widgets.forth
			end
		end

	update_widgets (a_key: STRING)
			-- Refresh all the widgets associated with `a_key'
		require
			key_exists: a_key /= Void
		local
			ii: INTERFACE_ITEM
			ls: LINKED_SET [EV_ANY]
			ev_a: EV_ANY
			marked_for_removal_set: LINKED_SET [EV_ANY]
		do
			if widgets.has (a_key) then
				create marked_for_removal_set.make
				ls := widgets.item (a_key)
				if names.has (a_key) then
					ii := names.interface_item (a_key)
--				elseif dynamic_names.has (a_key) then
--					ii := dynamic_names.item (a_key)
				else
					ii := Default_interface_item
				end
				from ls.start
				until ls.exhausted
				loop
					ev_a ?= ls.item
					if ev_a /= Void and then ev_a.is_destroyed then
						marked_for_removal_set.extend (ls.item)
					else
						update_widget_with_item (ls.item, ii)
					end
					ls.forth
				end
				from marked_for_removal_set.start
				until marked_for_removal_set.exhausted
				loop
					ls.prune (marked_for_removal_set.item)
					marked_for_removal_set.forth
				end
			end
		end

	update_widget_with_item (a_widget: EV_ANY; a_item: INTERFACE_ITEM)
			-- Refresh `a_widget' using the INTERFACE_ITEM associated
			-- with `a_key'.  If `a_key' is not present then the
			-- `Default_interface_item' is used.
		require
			widget_exists: a_widget /= Void
			item_exists: a_item /= Void
			table_has_widget: has_widget (a_widget)
		local
			ev_a: EV_ANY
			f: EV_FONTABLE
			hc: EV_HELP_CONTEXTABLE
			p: EV_PIXMAPABLE
			t: EV_TEXTABLE
			tt: EV_TOOLTIPABLE
			b: EV_TOOL_BAR_BUTTON
			pix: EV_PIXMAP
			tool: TOOL
		do
			ev_a ?= a_widget
			check
				widget_not_destroyed: ev_a /= Void implies not ev_a.is_destroyed
					-- because this should have been taken care of in calling procedure.
			end
			f ?= a_widget
			hc ?= a_widget
			p ?= a_widget
			t ?= a_widget
			tt ?= a_widget
			b ?= a_widget
			tool ?= a_widget
			if b /= Void then
				if is_showing_button_text then
					b.set_text (a_item.short_name)
				else
					b.remove_text
				end
				b.set_tooltip (a_item.tooltip)
				pix := a_item.pixmap
				pix.stretch (pixmap_size, pixmap_size)
				b.remove_pixmap
				b.set_pixmap (pix)
			end
--			if p /= Void then
--		-- this does not work.  Throws exception on menu items.
--				pix := clone (a_item.pixmap)
--				pix.stretch (pixmap_size, pixmap_size)
--				p.remove_pixmap
--				p.set_pixmap (pix)				
--			end
			if t /= Void and b = Void and tool = Void then
					-- handle any EV_TEXTABLE except for EV_TOOL_BAR_BUTTON
					-- which was handled above.  Also, no text added to TOOLs.
				t.set_text (a_item.short_name)
			end
			if tt /= Void then
				tt.set_tooltip (a_item.tooltip)
			end
		end

feature {INTERFACE_ITEM} -- Implementation

	changed_widgets: LINKED_SET [EV_ANY]
			-- To keep track of the keys of the changed items until an update is requested.

feature {APPLICATION_STATE, INTERFACE_TABLE_VIEW, INTERFACE_TABLE_TOOL} -- Implementation

	names: INTERFACE_NAMES	-- HASH_TABLE [INTERFACE_ITEM, STRING]  --INTERFACE_NAMES
			-- Holds the interface names for the program.

--	dynamic_names: HASH_TABLE [INTERFACE_ITEM, STRING]  INTERFACE_NAMES
			-- Holds interface_items which were created by the program on the fly;
			-- not assigned by the programmer statically.  This is really only used
			-- by VIEW_MANAGER when creating the widgets for the views.

feature {NONE} -- Implementation

	widgets: HASH_TABLE [LINKED_SET [EV_ANY], STRING]
			-- Lists of "registerred" widgets indexed by a key.  (See `register_widget'.)

	pixmapables: LINKED_SET [EV_ANY]
			-- Linear representation of all widgets in the table which are pixmapable.
		local
			ls: LINKED_SET [EV_ANY]
			p: EV_PIXMAPABLE
		do
			create Result.make
			from widgets.start
			until widgets.off
			loop
				ls := widgets.item_for_iteration
				from ls.start
				until ls.exhausted
				loop
					p ?= ls.item
					if p /= Void then
						Result.extend (p)
					end
					ls.forth
				end
				widgets.forth
			end
		end

	buttons: LINKED_SET [EV_ANY]
			-- Linear representation of all the buttons in the table
		local
			ls: LINKED_SET [EV_ANY]
			b: EV_TOOL_BAR_BUTTON
		do
			create Result.make
			from widgets.start
			until widgets.off
			loop
				ls := widgets.item_for_iteration
				from ls.start
				until ls.exhausted
				loop
					b ?= ls.item
					if b /= Void then
						Result.extend (b)
					end
					ls.forth
				end
				widgets.forth
			end
		end

	blocked_count: INTEGER
			-- The number of times `block' has been called without a corresponding
			-- call to `resume'.  This allows descendents to `resume' without having
			-- to worry about a `block' imposed by the precursor.

feature {NONE} -- Implementation

	set_defaults
			-- Set the interface names to default values.
		do
			pixmap_size := Default_pixmap_size
					-- Generic interface names used in various type of objects
--			add_item_with_tuple (<<"System not saved text", "System not saved", "The system has not been saved since last change.", "Changes have been made, but the system has not been save.", "icon_red_cross.ico">>)
--			add_item_with_tuple (<<"Not in system text", "Not in system", "The displayed interface names are not used by the program.", "The displayed interface names are not used by the program.", "icon_exec_quit_color.ico">>)
--			add_item_with_tuple (<<"Not allowed pixmap", "Not allowed", "Operation not allowed", "The operation you are attempting is not allowed.", "icon_delete_color.ico">>)
--			add_item_with_tuple (<<"Empty", "Empty", "Empty view or object", "The view or object is empty.", "icon_class_symbol_gray.ico">>)
					-- MAIN_WINDOW interface names
--			add_item_with_tuple (<<"{MAIN_WINDOW}", "MAIN_WINDOW", "Main Window of the program", "At least one main window is always created.", "icon_new_development_tool_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW}.file_menu", "File", "File actions", "Menu item containing 'open', 'save', 'save_as', etc.", "icon_open_file_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - new system", "New system", "Create a new system", "Makes a new system and targets current window to it", "icon_object_symbol.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - save", "Save", "Save the current system", "This lets you select the file name.", "icon_save_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - open", "Open", "Open an existing file", "This opens a dialog box from which to select a file.", "icon_open_file_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - exit", "Exit", "Exit the program", "Promts to save and ends the program.", "icon_close_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW}.edit_menu", "Edit", "Edit actions", "Menu item containing 'undo', 'redo', 'cut', 'copy', etc.", "icon_format_clickable_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - undo", "Undo", "Undo last command", "Returns to previous state by undoing the last action.", "icon_undo_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - redo", "Redo", "Redo last undone command", "Executes the last undone command.", "icon_redo_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW}.tool_menu", "Tools", "Tools menu", "Contains tools menu items", "icon_tool_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - preferences", "Preferences", "Set preferences", "Allows setting of window parameters.", "icon_system_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW}.window_menu", "Window", "Window actions", "Menu item containing 'new_window', etc", "icon_dummy.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - new window", "New window", "Open an untargetted window", "Creates a new window.", "icon_new_development_tool_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - maximize all", "Maximize all", "Maximizes all windows", "This item allows you to maximize all system windows.", "icon_maximize_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - minimize all", "Minimize all", "Minimizes all windows", "This item allows you to minimize all system windows.", "icon_minimize_all_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - raise all", "Raise all", "Raises all windows", "This item alls you to restore all system windows.", "icon_restore_all_color.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW}.help_menu", "Help", "Help menu", "Contains help menu items", "icon_dummy.ico">>)
--			add_item_with_tuple (<<"{MAIN_WINDOW} - help about", "About", "Display info about program", "Opens a dialog containing program information.", "icon_system_info_color.ico">>)
--			add_item_with_tuple (<<"{TEST_MAIN_WINDOW}", "TEST_MAIN_WINDOW", "Test window for Standard Interface cluster", "Test Window used to test the various functions in Standard Interface cluster.", "icon_new_context_window_color.ico">>)
				-- TOOL interface names				
--			add_item_with_tuple (<<"{TOOL}.back_button", "Back", "Go back to previous item", "Moves back in the list of previously viewed items.", "icon_back_color.ico">>)
--			add_item_with_tuple (<<"{TOOL}.forth_button", "Forth", "Go forward to next item", "Moves forward in the list of previously viewed items.", "icon_forth_color.ico">>)
--			add_item_with_tuple (<<"{TOOL}.maximize_button", "Maximize this tool", "Maximize this tool", "Makes this tool only one visible in window.", "icon_maximize_color.ico">>)
--			add_item_with_tuple (<<"{TOOL}.restore_button", "Restore this tool", "Restore this tool to normal size", "Makes this tool share window with others.", "icon_restore_color.ico">>)
--			add_item_with_tuple (<<"{TOOL}.close_button", "Hide this tool", "Hide this tool from view", "Hide this tool from view.", "icon_close_color.ico">>)
				-- VIEW_MANAGER interface names
--			add_item_with_tuple (<<"single_mode", "Single mode", "Switch view to single mode", "Makes only one view visible in the window.", "icon_format_text_color.ico">>)
--			add_item_with_tuple (<<"split_mode", "Split mode", "Switch view to split mode", "Makes multiple views visible in the window.", "icon_new_development_tool_color.ico">>)
--			add_item_with_tuple (<<"notebook_mode", "Tab mode", "Switch view to tab mode", "Makes views visible in the notebook tabs.", "icon_folders_color.ico">>)
--			add_item_with_tuple (<<"{VIEW_MANAGER} - create_view_string", "Toggle/Raise ", "Toggles or Raises the associated view", "Select this item to Toggle or raise the associated view.", "icon_dummy.ico">>)
				-- INTERFACE_TABLE_TOOL interface names
--			add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}", "INTERFACE_TABLE_TOOL", "Tool used to display info about the interface names.", "Tool used to display info about the interface names.", "icon_bkpt_info_color.ico">>)
--			add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL} - default interface names", "Default Interface Names", "Default interface names are displayed.", "The default interface names are displayed.", "icon_dummy.ico">>)
--			add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}.open_button", "Open", "Open Table", "Read in a file of interface names.", "icon_new_measure_color.ico">>)
--			add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}.save_button", "Save", "Save Table", "Save the interface names to a file.", "icon_save_measure_color.ico">>)
--			add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}.save_as_button", "Save as", "Save Table as new file.", "Opens the file save dialog allowing the displayed names to be save as a new file.", "icon_save_measure_color.ico">>)
--			add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}.apply_button", "Apply", "Apply changes", "Update all the interface items with those displayed in the tool.", "icon_view_measure_plus_color.ico">>)
--			add_item_with_tuple (<<"{INTERFACE_TABLE_TOOL}.xml_button", "temp XML", "Temp button to test XML parser", "Temporary functionf for testing the XML parser routines and classes.", "icon_view_measure_plus_color.ico">>)
				-- VIEWABLE_WIDGET interface names
--			add_item_with_tuple (<<"{VIEWABLE}", "VIEWABLE", "A VIEWABLE object", "This screen object represents a VIEWABLE.", "icon_class_color.ico">>)
--			add_item_with_tuple (<<"{VIEWABLE}.file_open_dialog", "Open File", "Open a file to read in a VIEWABLE_WIDGET object.", "Open a file to read in a VIEWABLE_WIDGET object.", "icon_new_cluster_color.ico">>)
--			add_item_with_tuple (<<"{VIEWABLE}.file_save_dialog", "Save File", "Save a VIEWABLE_WIDGET object to a file.", "Save a VIEWABLE_WIDGET object to a file.", "icon_super_cluster_color.ico">>)
				-- INTERFACE_ITEM_VIEW interface names
--			add_item_with_tuple (<<"{INTERFACE_ITEM_VIEW}.short_name_label", "Menu Item", "Change short_name", "Change the menu item string for this interface item.", "icon_dummy.ico">>)
--			add_item_with_tuple (<<"{INTERFACE_ITEM_VIEW}.tooltip_label", "Tooltip", "Change tooltip", "Change the tooltip string for this interface item.", "icon_dummy.ico">>)
--			add_item_with_tuple (<<"{INTERFACE_ITEM_VIEW}.help_text_label", "Help Text", "Change help_text", "Change the help_text string for this interface item.", "icon_dummy.ico">>)
--			add_item_with_tuple (<<"{INTERFACE_ITEM_VIEW}.pixmap_file_label", "Pixmap File", "The pixmap file name", "The name of the file from which to make a for this interface item.", "icon_dummy.ico">>)
--			add_item_with_tuple (<<"{INTERFACE_ITEM_VIEW}.pixmap_label", "Pixmap", "The pixmap", "This is how the pixmap will appear.", "icon_.ico">>)
				-- FILENAME_VIEW interface names
--			add_item_with_tuple (<<"{FILENAME_VIEW}.browse_button", "Browse", "Press to select a directory", "Press this button to open a dialog for selecting the directory to find pixmap files.", "icon_folders_color.ico">>)	
		end

invariant

	names_exist: names /= Void
	widgets_exists: widgets /= Void
	blocked_count_large_enought: blocked_count >= 0

end -- class INTERFACE_TABLE
