note
	description: "[
		Used by MAIN_WINDOW to persist "state" attributes (size, position,
		height, width, etc.) of the MAIN_WINDOW.
		]"
	author: "Jimmy J. Johnson"
	date: "10 Sep 03"

class
	MAIN_WINDOW_STATE

inherit

	VIEW_STATE
		redefine
			default_create,
			make,
			widget
		end

create
	default_create,
	make

feature {NONE} -- Initialization

	default_create
			-- Initialize with reasonable values
		do
			x := Default_x
			y := Default_y
			height := Default_height
			width := Default_width
			create split_manager_state
		end

	make (a_window: JJ_MAIN_WINDOW)
			-- Initialize from attributes of `a_window'.
		do
			Precursor {VIEW_STATE} (a_window)
			x := a_window.x_position
			y := a_window.y_position
			width := a_window.width
			height := a_window.height
			is_minimized := a_window.is_minimized
			is_maximized := a_window.is_maximized
		end

feature -- Access

	widget: JJ_MAIN_WINDOW
			-- Create a window from the attributes of Current.
		do
			Result ?= Precursor {VIEW_STATE}
			check
				result_exists: Result /= Void
					-- Because Precursor calls `new_widget' defined as "like `widget'".
			end
			Result.set_position (x, y)
			Result.set_size (width, height)
			if is_maximized then
				Result.maximize
			elseif is_minimized then
				Result.minimize
			else
				Result.restore
			end
		end

	x: INTEGER_32
			-- Stores the `x_position' of a window

	y: INTEGER_32
			-- Stores the `y_position' of a window

	height: INTEGER_32
			-- Stores the `height' of a window

	width: INTEGER_32
			-- Stores the `width' of a window

	is_minimized: BOOLEAN
			-- Stores the `is_minimized' state of the window

	is_maximized: BOOLEAN
			-- Stores the `is_maximized' state of the window

feature -- Element change

	set_x (a_value: INTEGER_32)
			-- Change `x'
		require
			value_big_enough: a_value >= 0
		do
			x := a_value
		end

	set_y (a_value: INTEGER_32)
			-- Change `y'
		require
			value_big_enough: a_value >= 0
		do
			y := a_value
		end

	set_height (a_value: INTEGER_32)
			-- Change `height'
		require
			value_big_enough: a_value >= 0
		do
			height := a_value
		end

	set_width (a_value: INTEGER_32)
			-- Change `width'
		require
			value_big_enough: a_value >= 0
		do
			width := a_value
		end

	set_minimized
			-- Make `is_minimized' True
		do
			is_minimized := True
		end

	set_maximized
			-- Make `is_maximized' True
		do
			is_maximized := True
		end

feature {NONE} -- Implementation (constants)

	Default_x: INTEGER = 100
			-- Default value for `x'

	Default_y: INTEGER = 100
			-- Default value for `y'

	Default_height: INTEGER = 300
			-- Default value for `height'

	Default_width: INTEGER = 600
			-- Default value for `width'

invariant

	split_manager_state_exists: split_manager_state /= Void

end
