note
	description: "[
		Structure to hold info about splits and views in SPLIT_MANAGER..
		]"
	author: "Jimmy J. Johnson"
	date: "12 Nov 07"

class
	SPLIT_MANAGER_NODE

inherit

	SHARED
		undefine
			default_create
		end

	S_TREE_NODE [ANY]		-- Correct type is checked by invariant `is_valid_type'.
		redefine
			default_create,
			make,
			test_print
		end

create
	default_create,
	make

feature {NONE} -- Initialization

	default_create
			-- Create an instance containing an EV_VERTICAL_SPLIT_AREA
		do
				-- Just to preserve the invariant.
			create {EV_VERTICAL_SPLIT_AREA} item
		end

	make (a_widget: EV_WIDGET)
			-- Initialize Current making `widget' a `a_widget'
		local
			int_item: INTERFACE_ITEM
			ii: INTERFACE_ITEM
			s: STRING_32
			v: VIEW
		do
			check
				correct_type: is_valid_type (a_widget)
					-- Because could not strengthen precondition with this.
			end
			item := a_widget
			if is_view then
				v := view
				create toggle_button
				create radio_button
				create toggle_item
				create radio_item
--				toggle_button.select_actions.extend (agent on_toggle)
--				toggle_item.select_actions.extend (agent on_toggle)
--				radio_button.select_actions.extend (agent on_select)
--				radio_item.select_actions.extend (agent on_select)
				-- Create an interface item and add to the `interface_table'
				ii := interface_table.interface_item ("{VIEW_MANAGER} - create_view_string")
				s := interface_table.short_name ("{VIEW_MANAGER} - create_view_string") + " " + v.generating_type
				if not interface_table.has_key (s) then
					create int_item
					int_item.set_short_name (v.generating_type)
					int_item.set_tooltip (ii.short_name + v.generating_type)
					int_item.set_help_text (ii.help_text + v.generating_type)
					int_item.set_pixmap_file (interface_table.interface_item (v.generating_type).pixmap_file)
					interface_table.add_interface_item (int_item, s)
				end
				interface_table.register_widget (toggle_button, s)
				interface_table.register_widget (radio_button, s)
				interface_table.register_widget (toggle_item, s)
				interface_table.register_widget (radio_item, s)
				enable
				show
			end
		end

feature -- Access

	visible_widget: EV_WIDGET
			-- The view or split area to be shown.
		require
			is_visible: is_branch_visible
		local
			w1, w2: EV_WIDGET
			sa: EV_SPLIT_AREA
		do
			if is_view then
				check
					is_visible: is_visible
						-- Becuase of precondition and this is a leaf node on the branch.
				end
				Result ?= item
				check
					result_exists: Result /= Void
						-- Because VIEWs should be EV_WIDGET.
				end
			elseif is_split then
				if first.is_branch_visible then
					w1 := first.visible_widget
						-- Make sure the widget doesn't have a parent.
					sa ?= w1.parent
					if sa /= Void then
						sa.prune (w1)
					end
				end
				if second.is_branch_visible then
					w2 := second.visible_widget
						-- Make sure the widget doesn't have a parent.
					sa ?= w2.parent
					if sa /= Void then
						sa.prune (w2)
					end
				end
				if w1 = Void then
					Result := w2
				elseif w2 = Void then
					Result := w1
				else
					check
						is_visible: is_visible
								-- Because both branches are visible (see `is_visible')
					end
					sa := split_area
					sa.set_first (w1)
					sa.set_second (w2)
					Result := sa
				end
			end
		ensure
			result_exists: Result /= Void
		end

	view: VIEW
			-- The `widget' typecast as a VIEW.
		require
			is_view: is_view
		do
			Result ?= item
		ensure
			result_exists: result /= Void
		end

	split_area: EV_SPLIT_AREA
			-- The `widget' typecast as a EV_SPLIT_AREA.
		require
			is_split: is_split
		do
			Result ?= item
		ensure
			result_exists: Result /= Void
		end

	split_position: INTEGER
			-- The last position of the splitter if this is a split area.

	toggle_button: EV_TOOL_BAR_TOGGLE_BUTTON
			-- Button to toggle this view on and off.

	radio_button: EV_TOOL_BAR_RADIO_BUTTON
			-- Button to select this view.

	toggle_item: EV_CHECK_MENU_ITEM
			-- Menu item for toggling this view on and off.

	radio_item: EV_RADIO_MENU_ITEM
			-- Menu item for selecting this view.

	state: SPLIT_MANAGER_NODE_STATE
			-- To save the status of current.
		do
			create Result.make (Current)
		end

feature -- Element change

	set_split_position (a_position: INTEGER)
			-- Change split_position
		require
			positive_position: a_position >= 0
		do
			split_position := a_position
		end

	save_split_position
			-- Save the current splitter offset in `split_position'.
		require
			is_split: is_split
		do
--			if is_full then
				split_position := split_area.split_position
--			end
		end

	restore_split_position
			-- Set the EV_SPLIT_AREA's splitter offset to `split_position'.
		require
			is_split: is_split
		local
			sa: EV_SPLIT_AREA
		do
			sa := split_area
--			if sa.full then
				if split_position < sa.minimum_split_position then
					sa.set_split_position (sa.minimum_split_position)
				elseif split_position > sa.maximum_split_position then
					sa.set_split_position (sa.maximum_split_position)
				else
					sa.set_split_position (split_position)
				end
--			end
		end

	syncronize_buttons
			-- Make sure the state of the buttons and menu items agree with the status settings.
		do
				-- Fix radio items (radios are always selected on syncronize)
--			radio_button.select_actions.block
--			radio_item.select_actions.block
			radio_button.enable_select
			radio_item.enable_select
--			radio_button.select_actions.resume
--			radio_item.select_actions.resume
				-- Fix toggle buttons.	
			toggle_item.select_actions.block
			toggle_button.select_actions.block
			if is_visible then
				toggle_button.enable_select
				toggle_item.enable_select
			else
				toggle_button.disable_select
				toggle_item.disable_select
			end
			toggle_button.select_actions.resume
			toggle_item.select_actions.resume
		end

feature -- Status report

	is_split: BOOLEAN
			-- Does `widget' conform to EV_SPLIT_AREA?
		local
			sa: EV_SPLIT_AREA
		do
			sa ?= item
			Result := sa /= Void
		end

	is_view: BOOLEAN
			-- Does `widget' conform to VIEW
		local
			v: VIEW
		do
			v ?= item
			Result := v /= Void
		end

	is_visible: BOOLEAN
			-- Is `widget' visible?  (I.e. should it be shown?)
		do
			if is_view and not is_hidden and not is_disabled then
				Result := True
			elseif is_split then
				Result := first.is_branch_visible and second.is_branch_visible
			end
		end

	is_branch_visible: BOOLEAN
			-- Does the `first' or `second' have a visible branch (recursively)?
		do
			Result := not is_disabled and then ((is_view and not is_hidden) or else
						(is_split and then (first.is_branch_visible or second.is_branch_visible)))
		end

	is_hidden: BOOLEAN
			-- Is `widget' hidden?  Has the user hidden it?

	is_disabled: BOOLEAN
			-- Is `widget' not showable?

	is_selected: BOOLEAN
			-- Is `widget' the currently selected view?

feature -- Status setting

	hide
			-- Make `is_visible' false.
		require
			allowed_for_views_only: is_view
		do
			is_hidden := True
			syncronize_buttons
		end

	show
			-- Make `is_visible' true.
		require
			allowed_for_views_only: is_view
			not_disabled: not is_disabled
		do
			is_hidden := False
			syncronize_buttons
		end

	enable
			-- Make `is_disabled' False.  (Make the `widget' showable)
		require
			allowed_for_views_only: is_view
		do
			is_disabled := False
			syncronize_buttons
		end

	disable
			-- Make `is_disabled' True.
		require
			allowed_for_views_only: is_view
		do
			is_disabled := True
			syncronize_buttons
		ensure
			is_disabled: is_disabled
		end

feature -- Testing

	test_print (indent: INTEGER)
			-- Show on `io' for testing.
		local
			i: INTEGER
		do
			from i := 1
			until i > indent
			loop
				io.put_string ("%T")
				i := i + 1
			end
			io.put_string ("item = " + item.generating_type)
			io.put_string ("%T")
--			if is_split then
--				io.put_string ("is_split %T")
--				if is_visible then
--					io.put_string ("is_visible")
--				else
--					io.put_string ("NOT visible")
--				end
--				io.put_string ("%T")
--				if is_branch_visible then
--					io.put_string ("is_branch_visible")
--				else
--					io.put_string ("NOT is_branch_visible")
--				end
--			else
--				io.put_string ("NOT is_split")
--			end
			io.new_line
			if is_split then
				first.test_print (indent + 1)
				second.test_print (indent + 1)
			end
		end

feature {NONE} -- Implementation

	is_valid_type (a_widget: like item): BOOLEAN
			-- Is `a_widget' a valid type?  (I.e. one conforming to VIEW or EV_SPLIT_AREA?)
		require
			widget_exists: a_widget /= Void
		local
			v: VIEW
			sa: EV_SPLIT_AREA
		do
			v ?= a_widget
			sa ?= a_widget
			Result := v /= Void or else sa /= Void
		end

	is_buttons_syncronized: BOOLEAN
			-- Does the state of each button reflect the objects state?
		require
			applies_to_view: is_view
		do
			Result := is_toggle_button_syncronized and then
						is_radio_button_syncronized and then
						is_toggle_item_syncronized and then
						is_radio_item_syncronized
		end

	is_toggle_button_syncronized: BOOLEAN
			-- Does the state of the `toggle_button' reflect the state of `is_visible'
		require
			applies_to_view: is_view
		do
			Result := is_visible implies toggle_button.is_selected
		end

	is_radio_button_syncronized: BOOLEAN
			-- Does the state of the `radio_button' reflect the state of `is_selected'
		require
			applies_to_view: is_view
		do
			Result := is_selected implies radio_button.is_selected
		end

	is_toggle_item_syncronized: BOOLEAN
			-- Does the state of the `toggle_item' reflect the state of `is_visible'
		require
			applies_to_view: is_view
		do
			Result := is_visible implies toggle_item.is_selected
		end

	is_radio_item_syncronized: BOOLEAN
			-- Does the state of the `radio_item' reflect the state of `is_selected'
		require
			applies_to_view: is_view
		do
			Result := is_visible implies toggle_button.is_selected
		end

invariant

	correct_type_widget: is_valid_type (item)

	view_has_buttons: is_view implies
		 (toggle_button /= Void and radio_button /= Void and toggle_item /= Void and radio_item /= Void)

end
