note
	description: "Saves the status of a SPLIT_MANAGER_NODE"
	author: "Jimmy J. Johnson"
	date: "17 Jan 08"

class
	SPLIT_MANAGER_NODE_STATE

inherit

	STATE
		redefine
			make,
			widget
		end

create
	make

feature {NONE} -- Initialization

	make (a_node: SPLIT_MANAGER_NODE)
			-- Create from the argument
		local
			vs: EV_VERTICAL_SPLIT_AREA
		do
			is_disabled := a_node.is_disabled
			is_selected := a_node.is_selected
			is_hidden := a_node.is_hidden
			split_position := a_node.split_position
			if a_node.is_view then
				is_view := True
				view_state := a_node.view.state
			else
				check
					is_split: a_node.is_split
						-- Because the nodes are used only for EV_SPLIT_AREAs or VIEWs
				end
				vs ?= a_node.split_area
				if vs /= Void then
					is_vertical := True
				end
				first_node_state := a_node.first.state
				second_node_state := a_node.second.state
			end
		end

feature -- Access

	widget: SPLIT_MANAGER_NODE
			-- The node initialized from attributes of Current.
		local
			v: VIEW
			sa: EV_SPLIT_AREA
		do
			Result := new_widget
			if is_view then
				v := view_state.widget
				Result.set_item (v)
			else
				if is_vertical then
					create {EV_VERTICAL_SPLIT_AREA} sa
				else
					create {EV_VERTICAL_SPLIT_AREA} sa
				end
				Result.set_item (sa)
					-- Recusive call...[sort of]
				Result.set_children (first_node_state.widget, second_node_state.widget)
				Result.set_split_position (split_position)
			end
			if is_disabled then
				Result.disable
			else
				Result.enable
			end
			if is_hidden then
				Result.hide
			else
				Result.show
			end
		end

feature {NONE} -- Implementation

	view_state: VIEW_STATE
			-- The state of the `view' in a SPLIT_MANAGER_NODE

	first_node_state: SPLIT_MANAGER_NODE_STATE
			-- The state of the node's `first' child

	second_node_state: SPLIT_MANAGER_NODE_STATE
			-- The state of the node's `second' child

	is_disabled: BOOLEAN
			-- Corresponds to `is_disabled' in SPLIT_MANAGER_NODE

	is_selected: BOOLEAN
			-- Corresponds to `is_selected' in SPLIT_MANAGER_NODE

	is_hidden: BOOLEAN
			-- Corresponds to `is_hidden' in SPLIT_MANAGER_NODE

	split_position: INTEGER_32
			-- Corresponds to `split_position' in SPLIT_MANAGER_NODE

	is_view: BOOLEAN
			-- Records that a node contains a VIEW

	is_vertical: BOOLEAN
			-- Did the node contain an EV_VERTICAL_SPLIT_AREA?

invariant


end

