note
	description: "[
				Used to make a STORABLE object for saving and restoring the
				state of a SPLIT_MANAGER.
				]"
	author: "Jimmy J. Johnson"
	date: "10 Sep 03"

class
	SPLIT_MANAGER_STATE

inherit

	STATE
		redefine
			make,
			widget
		end

create
	default_create,
	make

feature {NONE} -- Initialization

	make (a_split_manager: SPLIT_MANAGER)
			-- Initialize from attributes of `a_split_manager'.
		do
			Precursor {STATE} (a_split_manager)
			is_mode_frozen := a_split_manager.is_mode_frozen
			is_interface_frozen := a_split_manager.is_interface_frozen
			is_single_mode := a_split_manager.is_single_mode
			is_split_mode := a_split_manager.is_split_mode
			is_notebook_mode := a_split_manager.is_notebook_mode
			if a_split_manager.root /= Void then
				root_state := a_split_manager.root.state
			end
		end

feature -- Access

	widget: SPLIT_MANAGER
			-- The manager initialized from the attributes of Current.
		do
			Result ?= Precursor {STATE}
			check
				result_exists: Result /= Void
					-- Because Precursor calls `new_widget' defined as "like `widget'".
			end
			if root_state /= Void then
				Result.set_root (root_state.widget)
			end
			if is_mode_frozen then
				Result.disable_mode_changes
			else
				Result.enable_mode_changes
			end
			if is_interface_frozen then
				Result.disable_interface_changes
			else
				Result.enable_interface_changes
			end
			if is_single_mode then
				Result.set_single_mode
			elseif is_split_mode then
				Result.set_split_mode
			elseif is_notebook_mode then
				Result.set_notebook_mode
			else
				check
					should_not_happen: False
						-- because there are only three modes.
				end
			end
		end

feature {SPLIT_MANAGER} -- Implementation

	root_state: SPLIT_MANAGER_NODE_STATE
			-- Representation of the top (binary) node in the tree.

	is_mode_frozen: BOOLEAN
			-- Corresponds to `is_mode_frozen' in SPLIT_MANAGER.

	is_interface_frozen: BOOLEAN
			-- Corresponds to `is_veiw_frozen' in SPLIT_MANAGER.

	is_single_mode: BOOLEAN
			-- Corresponds to `is_single_mode' in SPLIT_MANAGER.

	is_split_mode: BOOLEAN
			-- Corresponds to `is_split_mode' in SPLIT_MANAGER.

	is_notebook_mode: BOOLEAN
			-- Corresponds to `is_notebook_mode' in SPLIT_MANAGER.

	selected_view: INTEGER
			-- Corresponds to `selected_view' in VIEW_MANAGER but hold an
			-- integer index to reference the correct view in `views'.

invariant

end
