note
	description: "[
				Used to make a STORABLE object for saving and restoring the
				state of a VIEW_MANAGER.
				]"
	author: "Jimmy J. Johnson"
	date: "10 Sep 03"

class
	VIEW_MANAGER_STATE

inherit

	STATE
		redefine
			default_create,
			make
		end

create
	default_create,
	make

feature {NONE} -- Initialization

	default_create
			-- Create an instance.
		do
			create visible_views.make
			create splits.make
			create splitter_positions.make
		end

	make (a_view_manager: VIEW_MANAGER)
			-- Initialize from attributes of `a_view_manager'.
		local
			vsa: EV_VERTICAL_SPLIT_AREA
			sa: EV_SPLIT_AREA
			wv: VIEW
			i: INTEGER
			v: VIEW
		do
			create visible_views.make
			create splits.make
			create splitter_positions.make
--			create targets.make
			create view_states.make
					-- store simple attribute data
			is_mode_frozen := a_view_manager.is_mode_frozen
			is_view_frozen := a_view_manager.is_view_frozen
			is_single_mode := a_view_manager.is_single_mode
			is_split_mode := a_view_manager.is_split_mode
			is_notebook_mode := a_view_manager.is_notebook_mode
			is_creating_vertical_splits := a_view_manager.is_creating_vertical_splits
--			is_showing_button_text := a_view_manager.is_showing_button_text
					-- store `visible_views' data
			from a_view_manager.visible_views.start
			until a_view_manager.visible_views.exhausted
			loop
				wv := a_view_manager.visible_views.item
				visible_views.extend (a_view_manager.views.index_of (wv, 1))
				a_view_manager.visible_views.forth
			end
					-- store `splits' data
			create vsa
			from a_view_manager.splits.start
			until a_view_manager.splits.exhausted
			loop
				sa := a_view_manager.splits.item
				if sa.same_type (vsa) then
					splits.extend (True)
				else
					splits.extend (false)
				end
				a_view_manager.splits.forth
			end
					-- store `splitter_positions'
			from a_view_manager.splitter_positions.start
			until a_view_manager.splitter_positions.exhausted
			loop
				i := a_view_manager.splitter_positions.item
				splitter_positions.extend (i)
				a_view_manager.splitter_positions.forth
			end
					-- store `selected_view'
			if a_view_manager.views.has (a_view_manager.selected_view) then
				selected_view := a_view_manager.views.index_of (a_view_manager.selected_view, 1)
			else
				selected_view := 0
			end
--					-- Store the `target' and `objects' of each view
--			from a_view_manager.views.start
--			until a_view_manager.views.exhausted
--			loop
--				targets.extend (a_view_manager.views.item.target)
--				objects.extend (a_view_manager.views.item.objects)
--				a_view_manager.views.forth
--			end
			from a_view_manager.views.start
			until a_view_manager.views.exhausted
			loop
				v := a_view_manager.views.item
				view_states.extend (v.state)
				a_view_manager.views.forth
			end
		end

feature {VIEW_MANAGER} -- Implementation

--	targets: LINKED_LIST [VIEWABLE]
--			-- Corresponds to `target' in VIEW.
--	
--	objects: LINKED_LIST [LINEAR [VIEWABLE]]
--			-- Corresponds to `objects' in VIEW.
--
	view_states: LINKED_LIST [VIEW_STATE]
			-- Corresponds to the `state' of each VIEW in `views'.

	is_mode_frozen: BOOLEAN
			-- Corresponds to `is_mode_frozen' in VIEW_MANAGER.

	is_view_frozen: BOOLEAN
			-- Corresponds to `is_veiw_frozen' in VIEW_MANAGER.

	is_single_mode: BOOLEAN
			-- Corresponds to `is_single_mode' in VIEW_MANAGER.

	is_split_mode: BOOLEAN
			-- Corresponds to `is_split_mode' in VIEW_MANAGER.

	is_notebook_mode: BOOLEAN
			-- Corresponds to `is_notebook_mode' in VIEW_MANAGER.

	is_creating_vertical_splits: BOOLEAN
			-- Corresponds to `is_creating_vertical_splits' in VIEW_MANAGER.

--	is_showing_button_text: BOOLEAN
--			-- Corresponds to `is_showing_button_text' in VIEW_MANAGER.
--			

	visible_views: LINKED_LIST [INTEGER]
			-- Corresponds to `visible_views' in VIEW_MANAGER, but instead of holding
			-- VIEWs, it holds integer indexes to reference the correct VIEW
			-- in VIEW_MANAGER's `views'.

	splits: LINKED_LIST [BOOLEAN]
			-- Corresponds to `splits' in VIEW_MANAGER, but instead of holding
			-- EV_SPLIT_AREAs it holds booleans which will represent the type of
			-- split area which should be in that position in the list.
			-- (True for every EV_VERTICAL_SPLIT_AREA and False for every
			-- EV_HORIZONTAL_SPLIT_AREA.

	splitter_positions: LINKED_LIST [INTEGER]
			-- Corresponds to `splitter_positions' in VIEW_MANAGER.

	selected_view: INTEGER
			-- Corresponds to `selected_view' in VIEW_MANAGER but hold an
			-- integer index to reference the correct view in `views'.

invariant

	visible_views_exists: visible_views /= Void
	splits_exists: splits /= Void
	splitter_positions_exists: splitter_positions /= Void

end -- class VIEW_MANAGER_STATE
